// ==========================================================================
//                    Class Specification : ccDataBase
// ==========================================================================
//////////////////////////////////////////////////////////////////////
// $File name   : /ccLib/dev/ccdbLib/ccDataBase.h: 
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// $HomePage    : home.bip.net/CyberCow
// $ICQ         : 7983755
//
// Notice       : Copyright (c) CyberCow AB 2000
//                Ekbacksvgen 60
//                168 69 BROMMA
//                SWEDEN  
//                tel int + 46 (8) 555 362 95
//                
//                The copyright to the computer program(s) herein is the 
//				  property of CyberCow.The program(s) may be used and/or 
//				  copied only with the written permission of CyberCow or 
//				  in accordance with the terms and conditions stipulated 
//				  in the agreement/contract under which the program(s) 
//				  have been supplied. 
//
/////////////////////////////////////////////////////////////////////////////
///** $History$ *************************************************************
// *
// * $Date: 4/27/2000 11:17 $
// * $Revision: 2 $
// * 
// * *****************  Version 1  *****************
// * User: Daniel Date: 9/13/99 14:54
// * Source-Code Traced [ ]; Reviewd [ ] 
// * Created in $/ccLib/dev/ccdbLib
// * Initial release.
// * *****************  Version 2  *****************
// * User: Daniel Date: 3/27/2000 11:17
// * Source-Code Traced [ ]; Reviewd [ ] 
// * Created in $/ccLib/dev/ccdbLib
// * Start Adding ADO
// * 
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
// ** $Properties$ **********************************************************
// * [X] Abstract class (does not have any objects)
// * [X] Uses exceptions
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
/*
$Description : 
	This class has the connection to the dataSource. All datadictionary
	objects will be created in a ccDataBase dervied class. And all
	relations etc will be done there.

*/
/////////////////////////////////////////////////////////////////////////////
#ifndef __CCDATABASE_H__
#define __CCDATABASE_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// DataBase mode.
//
enum CCDB_DATABASETYPE
{
	CCDB_NODB = 0,
	CCDB_ODBC,
	CCDB_ADO,
	CCDB_CCSERVICE
};

class ccDataDictionary;
class ccField;
class ccdbSuperCtrl;
class AFX_EXT_CLASS ccDataBase  
{
protected:
	friend ccDataDictionary;
	friend ccField;

// *** Construction/Destruction
public:
	// Default constructor.
	//
	ccDataBase();

	// Connect to a database throw ODBC.
	//
	void openODBC( CString asConnectDSN );

	// Connect to a database throw ADP
	//	
	void openADO( LPCTSTR asConnectionString, ADODB::CursorLocationEnum adCursorType = ADODB::adUseNone,			\
				   bool bExclusive = FALSE, bool bReadOnly = FALSE, int aiConnectionTimeout = 15 );

	// Open a connection to a MS Sql Server
	//
	void openADOMSSQLServer( LPCTSTR asServerName, LPCTSTR asDatabase, LPCTSTR asUser, LPCTSTR asPassword, \
	 						   ADODB::CursorLocationEnum adCursorType = ADODB::adUseNone,\
							   bool bExclusive = FALSE, bool bReadOnly = FALSE, int aiConnectionTimeout = 15 );

	// Open a connection to an access database
	//
	void openADOAccessDB( LPCTSTR asDataBaseURL, LPCTSTR asDataBasePassword, \
	 						   ADODB::CursorLocationEnum adCursorType = ADODB::adUseNone,\
							   bool bExclusive = FALSE, bool bReadOnly = FALSE, int aiConnectionTimeout = 15 );

protected:
	// If true the database and all its datadictionarys can be used.
	// Must be set in the constructor of the dervied database.
	//
	void enableReadyToUseDB()						{ m_bReadyToUse = true;				};
	bool isReadyToUse()								{ return m_bReadyToUse;				};


	// Connect to a database located in a ccdbService.
	//
	// void openccServiceDataBase( CString asURL );

	// Will close the database
	//
	void close();

	// Defaul destructor
	//
	virtual ~ccDataBase();
private:
	bool m_bReadyToUse;				// see enableReadyToUse;

// *** Transactions
public:
	inline long beginTrans ( )			{ return getConnection()->BeginTrans();			};

	inline HRESULT commitTrans ( )		{ return getConnection()->CommitTrans();		};

	inline HRESULT rollbackTrans ( )	{ return getConnection()->RollbackTrans();		}; 


// *** Attributes
public:
	// Handling the connect DSN.
	//
	void    setConnectDSN( CString asDSN )			{ m_connectDSN = asDSN;				};
	CString getConnectDSN( )						{ return m_connectDSN;				};

	// Check if the database connection is opened.
	//
	BOOL isOpen();

private:
	// Handling the database type
	//
	void setDataBaseType( CCDB_DATABASETYPE aType )	{ m_dataBaseType = aType;			};
	CCDB_DATABASETYPE getDataBaseType()				{ return m_dataBaseType;			};	

private:
	CString m_connectDSN;			// see setConnectDSN

	CCDB_DATABASETYPE m_dataBaseType;



	
// *** Input/Output
public:
	// Get the database object.
	//
	CDatabase * getDataBase()						{ ASSERT(getDataBaseType()==CCDB_ODBC); 
													 return m_dataBase;				};

	// Return the ADO Connection;
	//
	ADODB::_ConnectionPtr getConnection()			{ ASSERT(getDataBaseType()==CCDB_ADO); 
													 return m_connection;				};

	// The ctrl which have the crnt ctrl. Used by the dbToolBar
	//
	void setCrntFocus( ccdbSuperCtrl * aCtrl )		{ m_ctrnFocusCtrl = aCtrl;			};
	ccdbSuperCtrl* getCrntFocus()					{ return m_ctrnFocusCtrl;			};

private:
	static ccdbSuperCtrl * m_ctrnFocusCtrl;	// see setCtrnFocus


// *** Operations
public:
	// Call this member function when you need to execute 
	// an SQL command directly. 
	//
	void executeSQL( CString asqlStmt );


// *** ADO Retrive data from DataBase
public:
	//
	//
	bool getRS(LPCTSTR asSQL, ADODB::_RecordsetPtr &PopSet);

	// Gets single value from a recordset 
	// SQL = LPCTSTR SQL statment
	// Returns VARIANT BSTR containing the value in Row0, column0 of the recordset you created.
	//
	bool getRSVariant(LPCTSTR asSQL, VARIANT *Result);

	// Get the first row in a recordset, executed
	// by a sql statment.
	//
	#define dVarArray(x,y) CArray<_variant_t, _variant_t> x; x.SetSize(y);
	bool getRSVArray( LPCTSTR strSQL, CArray<_variant_t, _variant_t> &result );

	// Executes a valid asSQL returning a VARIANT (BSTR) that contains the recordset
	// with columns delimited by the asDelimiter and recordset delimited by \r\n
	//
	// SQL = BSTR SQL statement
	// Delimiter = the delimiter you want to use.  It can be just about anything
	// Returns VARIANT (BSTR) containingthe delimited recordset
	//
	bool getRSDelimited(LPCTSTR asSQL, LPCTSTR asDelimiter, VARIANT *Result);

	// 
	//
	bool executeSQLNoRet(LPCTSTR asSQL, VARIANT &RecordsEffected);	



// *** Data Members
private:
	
// ** ODBC
private:
	CDatabase * m_dataBase;							// Connection to database.

// ** ADO
private:
	ADODB::_ConnectionPtr m_connection;				// Connection to database.

// ** ccServer
private:
	// *** todo *** ccdbServer * m_dbServer;		// Connection to database.
};


// *** GLOBAL ADO functions.

// Test HRESULT
//
inline void TESTHR( HRESULT _hr ) { if FAILED(_hr) _com_issue_error(_hr); }

// Get error from ADO provider.
//
CString AFX_EXT_API getProviderError( ADODB::_ConnectionPtr pConnection );

// Get com error.
//
CString AFX_EXT_API getComError(_com_error &e);


#endif // __CCDATABASE_H__
