//////////////////////////////////////////////////////////////////////
// $File name   : ccDataDictionary.cpp
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "ccDataBase.h"

#include "ccField.h"

#include "ccDataDictionary.h"

#include "ccNonDBCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// *** Construction/Destruction
/////////////////////////////////////////////////////////////////////////////

ccDataDictionary::ccDataDictionary(ccDataBase * pDataBase) 
{
	setDB ( pDataBase );

	m_dataBaseType = pDataBase->getDataBaseType();

	m_primKey = NULL;
	
	setNewRecord( false );

	enableCanSave( false );
	setModified(false);

	setConfirmSaveMsg( IDS_SAVE_CHANGES );
	setConfirmDeleteMsg( IDS_DELETE_RECORD );
	setConfirmClearMsg( IDS_ABONDON_CHANGES );
}

//

void ccDataDictionary::openADO(LPCSTR aSQLQuery, enum ADODB::CursorTypeEnum aCursorType,
							enum ADODB::LockTypeEnum aLockType,
							enum ADODB::CursorLocationEnum aCursorLocation )
{
	m_cursorType = aCursorType;
	NEEDVALUE (	m_dataBaseType == CCDB_ADO);

	HRESULT hr = S_OK;

	try
	{
		// Open recordset 
		TESTHR(m_recordset.CreateInstance( __uuidof(ADODB::Recordset)) );		

		// Use client cursor to allow use of AbsolutePosition property.
		m_recordset->CursorLocation = aCursorLocation;

		_variant_t vConnection( (IDispatch *)__getDB()->getConnection(), true);

        m_recordset->Open(aSQLQuery, vConnection, aCursorType, aLockType, ADODB::adCmdText);		

		if (!isSupporting())
			ccErr("error","","",MB_OK);

		moveFirst();
	}
	catch(_com_error &e) 
	{ 
		ccThrowccException( getProviderError( __getDB()->getConnection()) ); /*Disable unreference warning*/ (void*)&e; 
	}		
}

//

void ccDataDictionary::close()
{
	switch(m_dataBaseType)
	{
	case CCDB_ADO:
		try
		{
			// Clean up objects before exit.
			if (isOpen() )
			{
				TESTHR( m_recordset->CancelUpdate() );
				m_recordset->Close();
			}
		}
		catch(_com_error &e)
		{						
			ccThrowccException( getComError(e) );
		}		
		break;
	}
}

//

ccDataDictionary::~ccDataDictionary()
{
	m_nonDBBoundCtrls.RemoveAll();

	close();
}

//

void ccDataDictionary::bindNonDBCtrls( ccNonDBCtrl * ctrl)
{
	m_nonDBBoundCtrls.Add( ctrl );	
}

//

bool ccDataDictionary::removeNonDBCtrls( ccNonDBCtrl * ctrl )
{
	NEEDVALUE( ctrl );

	for( int nIndex=0;nIndex<m_nonDBBoundCtrls.GetSize();nIndex++ ) 
	{
        ccNonDBCtrl* lpAttchedCtrls=(ccNonDBCtrl*)m_nonDBBoundCtrls[nIndex];
		if ( lpAttchedCtrls == ctrl)
			m_nonDBBoundCtrls.RemoveAt(nIndex);
	}

	return true;
}

//

void ccDataDictionary::bind( ccField& aoField, UINT nID, LPCSTR asColumnName )
{
	CString lsColumnName = asColumnName;
	lsColumnName.Replace("]","");
	lsColumnName.Replace("[","");
	_variant_t vColumnName;
	vColumnName = (_bstr_t) lsColumnName;

	aoField.bind( this, nID, asColumnName, m_recordset->Fields->Item[vColumnName] );
	
	m_fields.Add( &aoField );
}

//

void ccDataDictionary::reBindAll()
{	
	for( int nIndex=0;nIndex<m_fields.GetSize();nIndex++ ) 
	{
        ccField* pCol=(ccField*)m_fields[nIndex];

		CString lsColumnName = pCol->getColumnName();
		lsColumnName.Replace("]","");
		lsColumnName.Replace("[","");
		_variant_t vColumnName = (_bstr_t) lsColumnName;

		pCol->setADOField( m_recordset->Fields->Item[vColumnName] );
	}
}

//

void ccDataDictionary::setPrimKey( ccField * aoPrimKey)			
{ 
	if ( !aoPrimKey->isColumnSorted() )
	{		
		ccThrowccException( IDS_PRIM_KEY_MUST_BE_SORTED );	
	}

	m_primKey = aoPrimKey;		
}

//

/*
void ccDataDictionary::sortBy(ccField * aoSortField ...)
{
	if ( isOpen() )
	{
		// Enable sort on the first argumented field.
		CString lsSortBy = aoSortField->getColumnName();
		aoSortField->enableColumnSorting( true );

		// Enable sort on the variable numbers of argumented fields.
		va_list ap;
		va_start( ap, aoSortField );
		for (;;)
		{
			ccField * p = va_arg( ap, ccField* );
			if (p == NULL )
				break;
			lsSortBy += ", ";
			lsSortBy += p->getColumnName();		
			p->enableColumnSorting( true );
		}
		va_end( ap );

		
		// Remove the [ and ] from the field names. 
		lsSortBy.Replace("[", "");
		lsSortBy.Replace("]", "");
		m_strSort = lsSortBy;		

		if ( !requery() )
		{
			ccThrowccException( IDS_CANT_RESORT_THE_TABLE );
		}
	}
	else
		ccThrowccException( IDS_DATABASE_ISNT_OPEN );
}
*/


/////////////////////////////////////////////////////////////////////////////
// *** Attribute
/////////////////////////////////////////////////////////////////////////////
BOOL ccDataDictionary::isOpen()
{
	switch(m_dataBaseType)
	{
	case CCDB_ADO:
		return (m_recordset->State == ADODB::adStateOpen);
		break;		
	}
	return false;
}

//

bool ccDataDictionary::isSupporting()
{
	if ( m_cursorType != m_recordset->CursorType )
		return false;

	if (!m_recordset->Supports(ADODB::adAddNew) )
		return false;

	if (!m_recordset->Supports(ADODB::adDelete) )
		return false;

	if (!m_recordset->Supports(ADODB::adUpdate) )
		return false;

	// Those need to be checked for adOpenKeyset and adOpenStatic..
	if (!m_recordset->Supports(ADODB::adMovePrevious) )
		return false;

	return true;

	if (!m_recordset->Supports(ADODB::adBookmark) )
		return false;

	if (!m_recordset->Supports(ADODB::adHoldRecords) )
		return false;

	if (!m_recordset->Supports(ADODB::adResync) )
		return false; 

	return true;
}

//

bool ccDataDictionary::isCathingNativeError(int nNativeError)
{	
    HRESULT  hr = S_OK;        
	CString strAllErrors;

    try
    {
		ADODB::ErrorPtr  pErr = NULL;						

		if( (__getDB()->m_connection->Errors->Count) > 0)
		{
			long nCount = __getDB()->m_connection->Errors->Count;

			// Collection ranges from 0 to nCount -1.
			for(long i = 0; i < nCount; i++)
			{
				pErr = __getDB()->m_connection->Errors->GetItem(i);

				if (pErr->NativeError == nNativeError)
					return true;
			}
		}	
	}
    catch(_com_error &e)
    {
        // Notify the user of errors if any.
        strAllErrors = getComError(e);
    }

	return false;
}

//

bool ccDataDictionary::isModified()
{
	switch(m_dataBaseType)
	{
	case CCDB_ADO:
		{ 
			if ( isEOF() || m_recordset->BOF )
				return false;
			else
			{
				if (m_modified)
					return true;

				if ( (m_recordset->EditMode == ADODB::adEditAdd)
					|| (m_recordset->EditMode == ADODB::adEditInProgress) )
					return true;
				else
					return false;
			}
		}
	break;
	}		
	return false;
}

//

void ccDataDictionary::filter( LPCSTR strFilter)
{
	 getRS()->Filter = (_bstr_t)strFilter;
	 refresh();
}

/////////////////////////////////////////////////////////////////////////////
// *** Confirmation 
/////////////////////////////////////////////////////////////////////////////

BOOL ccDataDictionary::showConfirmMsg( CString asMessage )
{	
	if ( !asMessage.IsEmpty() )		
	{
		CString lsConfirm;
		lsConfirm.LoadString( IDS_CONFIRM );
		if ( MessageBox(NULL, asMessage, lsConfirm, MB_ICONQUESTION |  MB_YESNO ) == IDYES ) 
			return TRUE;
		else
			return FALSE;
	}
	// No message should be shown, 
	// and the confirmation answer is yes.
	else
		return TRUE;
}

//

void ccDataDictionary::setConfirmSaveMsg( int aiResource )		
{ 
	if ( aiResource == 0 )
		m_confirmSaveMsg.Empty();

	m_confirmSaveMsg.LoadString( aiResource );	
}

//

void ccDataDictionary::setConfirmDeleteMsg( int aiResource )		
{ 
	if ( aiResource == 0 )
		m_confirmDeleteMsg.Empty();

	m_confirmDeleteMsg.LoadString( aiResource );	
}

//

void ccDataDictionary::setConfirmClearMsg( int aiResource )		
{ 
	if ( aiResource == 0 )
		m_confirmClearMsg.Empty();

	m_confirmClearMsg.LoadString( aiResource );	
}

/////////////////////////////////////////////////////////////////////////////
// *** General
/////////////////////////////////////////////////////////////////////////////

E_ccErrRet ccDataDictionary::refresh()
{
	// Refresh bound ccFields.
	for( int nIndex=0;nIndex<m_fields.GetSize();nIndex++ ) 
	{
        ccField* pCol=(ccField*)m_fields[nIndex];
		pCol->refresh();
	}

	// Refresh nonDBCtrls
	for( nIndex=0;nIndex<m_nonDBBoundCtrls.GetSize();nIndex++ ) 
	{
        ccNonDBCtrl* pCtrl=(ccNonDBCtrl*)m_nonDBBoundCtrls[nIndex];
		if (pCtrl)
		pCtrl->refresh();
	}

	return CCSUCCEDED;
}

//

E_ccErrRet ccDataDictionary::refreshRelations()
{
	// Refresh parents before childs
	for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		if ( pParentField->getDD()->refresh() == CCFAILED )
			return CCFAILED;
	}		

	// Clear child dd
	for( nIndex=0;nIndex<m_fields.GetSize();nIndex++ ) 
	{
        ccField* pCol=(ccField*)m_fields[nIndex];
		pCol->refresh();
	}
	return CCSUCCEDED;
}

/////////////////////////////////////////////////////////////////////////////
// *** Navigation
/////////////////////////////////////////////////////////////////////////////

E_ccErrRet ccDataDictionary::moveFirst()
{
	try
	{				
		try
		{			
			TESTHR( m_recordset->CancelUpdate() );

			// The table is empty
			if ( m_recordset->adoEOF || m_recordset->BOF )
			{
				MessageBeep( MB_ICONEXCLAMATION );

				if ( clearRecord() )
					return CCSUCCEDED;
				else
					return CCFAILED;
			}

			TESTHR( m_recordset->MoveFirst() );

			relateFind();		
			if (__getDB()->isReadyToUse()) OnFieldChangeComplete();
			refresh();		
			enableCanSave( true );
			setNewRecord( false );								

			return CCSUCCEDED;
		}
		catch(_com_error &e) 
		{ 
			if ( isCathingNativeError(-2147217885) )
			{				
				requery();
				moveFirst();
				ccErrEM( IDS_THE_RECORD_YOU_WHERE_MOVING_TO_WAS_DELETED, NULL, NULL, MB_OK );
				return CCSUCCEDED;
			}
			ccThrowccException( getComError(e) ); /*Disable unreference warning*/ (void*)&e; 
		}		
	}
	catch(ccException *e) 
	{
		ccErr(*e,"","", MB_OK);	
	}	
	catch(...) 
	{
		ccErr("DB Error", "","", MB_OK);		
	}

	return CCFAILED;
}

//

E_ccErrRet ccDataDictionary::movePrev()
{
	try
	{				
		try
		{			
			TESTHR( m_recordset->CancelUpdate() );

			// The table is empty
			if ( m_recordset->adoEOF || m_recordset->BOF )
			{
				MessageBeep( MB_ICONEXCLAMATION );

				if ( clearRecord() )
					return CCSUCCEDED;
				else
					return CCFAILED;
			}
			
			if ( !m_recordset->BOF ) 				
				TESTHR( m_recordset->MovePrevious() );

			if ( m_recordset->BOF )
			{
				TESTHR( m_recordset->MoveFirst() );
				MessageBeep( MB_ICONEXCLAMATION );
			}

			relateFind();	
			if (__getDB()->isReadyToUse()) OnFieldChangeComplete();
			refresh();		
			enableCanSave( true );
			setNewRecord( false );						
			
			return CCSUCCEDED;
		}
		catch(_com_error &e) 
		{ 
			if ( isCathingNativeError(-2147217885) )
			{
				ccErrEM( IDS_THE_RECORD_YOU_WHERE_MOVING_TO_WAS_DELETED, NULL, NULL, MB_OK );
				requery();
				moveFirst();
				return CCSUCCEDED;
			}
			ccThrowccException( getProviderError( __getDB()->m_connection) );	/*Delete unreference warning*/ (void*)&e; 
		}
	}
	catch(ccException *e) 
	{
		ccErr(*e,"","", MB_OK);	
	}
	catch(...) 
	{
		ccErr("DB Error", "","", MB_OK);		
	}

	return CCFAILED;
}

//

E_ccErrRet ccDataDictionary::moveNext()
{		
	try
	{				
		try
		{			
			TESTHR( m_recordset->CancelUpdate() );

			// The table is empty
			if ( m_recordset->adoEOF || m_recordset->BOF )
			{
				MessageBeep( MB_ICONEXCLAMATION );

				if ( clearRecord() )
					return CCSUCCEDED;
				else
					return CCFAILED;
			}

			if ( !m_recordset->adoEOF ) 
				TESTHR( m_recordset->MoveNext() );

			if ( m_recordset->adoEOF )
			{
				TESTHR( m_recordset->MoveLast() );
				MessageBeep( MB_ICONEXCLAMATION );
			}

			relateFind();		
			if (__getDB()->isReadyToUse()) OnFieldChangeComplete();
			refresh();		
			enableCanSave( true );
			setNewRecord( false );			
			
			return CCSUCCEDED;
		}
		catch(_com_error &e) 
		{ 
			if ( isCathingNativeError(-2147217885) )
			{
				ccErrEM( IDS_THE_RECORD_YOU_WHERE_MOVING_TO_WAS_DELETED, NULL, NULL, MB_OK );
				requery();
				moveFirst();
				return CCSUCCEDED;
			}
			else
				ccThrowccException( getProviderError( __getDB()->m_connection) );	/*Delete unreference warning*/ (void*)&e; 
		}
	}
	catch(ccException *e) 
	{
		ccErr(*e,"","", MB_OK);	
	}
	catch(...) 
	{
		ccErr("DB Error", "","", MB_OK);		
	}

	return CCFAILED;
}

//

E_ccErrRet ccDataDictionary::moveLast()
{	
	try
	{	
		try
		{			
			TESTHR( m_recordset->CancelUpdate() );

			// The table is empty
			if ( m_recordset->adoEOF || m_recordset->BOF )
			{
				MessageBeep( MB_ICONEXCLAMATION );

				if ( clearRecord() )
					return CCSUCCEDED;
				else
					return CCFAILED;
			}

			TESTHR( m_recordset->MoveLast() );
			
			relateFind();		
			if (__getDB()->isReadyToUse()) OnFieldChangeComplete();
			refresh();
			enableCanSave( true );
			setNewRecord( false );			
			
			return CCSUCCEDED;
		}
		catch(_com_error &e) 
		{ 
			if ( isCathingNativeError(-2147217885) )
			{
				ccErrEM( IDS_THE_RECORD_YOU_WHERE_MOVING_TO_WAS_DELETED, NULL, NULL, MB_OK );
				requery();
				moveFirst();
				return CCSUCCEDED;
			}
			ccThrowccException( getProviderError( __getDB()->m_connection) );	/*Delete unreference warning*/ (void*)&e; 
		}
	}
	catch(ccException *e) 
	{
		ccErr(*e,"","", MB_OK);	
	}
	catch(...) 
	{
		ccErr("DB Error", "","", MB_OK);		
	}

	return CCFAILED;
}

//

E_ccErrRet ccDataDictionary::find(LPCSTR strCriteria)
{	
	try
	{	
		try
		{			
			TESTHR( m_recordset->CancelUpdate() );

			// The table is empty
			if ( m_recordset->adoEOF || m_recordset->BOF )
			{
				MessageBeep( MB_ICONEXCLAMATION );

				if ( clearRecord() )
					return CCSUCCEDED;
				else
					return CCFAILED;
			}

			getRS()->Find((_bstr_t)strCriteria, 0, ADODB::adSearchForward, "");							
			
			relateFind();		
			if (__getDB()->isReadyToUse()) OnFieldChangeComplete();
			refresh();
			enableCanSave( true );
			setNewRecord( false );			
			
			return CCSUCCEDED;
		}
		catch(_com_error &e) 
		{ 
			if ( isCathingNativeError(-2147217885) )
			{
				ccErrEM( IDS_THE_RECORD_YOU_WHERE_MOVING_TO_WAS_DELETED, NULL, NULL, MB_OK );
				requery();
				moveFirst();
				return CCSUCCEDED;
			}
			ccThrowccException( getProviderError( __getDB()->m_connection) );	/*Delete unreference warning*/ (void*)&e; 
		}
	}
	catch(ccException *e) 
	{
		ccErr(*e,"","", MB_OK);	
	}
	catch(...) 
	{
		ccErr("DB Error", "","", MB_OK);		
	}

	return CCFAILED;
}

//

/*
E_ccErrRet ccDataDictionary::setBookmark(const CDBVariant &varBookmark)
{	

	NEEDVALUE( &varBookmark );
	
	if ( CanBookmark() )
	{
		TRY
		{
			CRecordset::SetBookmark(varBookmark);
			
			relateFind();			
			if (__getDB()->isReadyToUse()) OnFieldChangeComplete();
			refresh();
			Edit();
			enableCanSave( true );
			setNewRecord( false );			
			
		}

		CATCH(CDBException, e)
		{
			AfxMessageBox(e->m_strError);
			exit(1);
		}		
		AND_CATCH(CMemoryException , e)
		{			
			TCHAR   szCause[255];
			e->GetErrorMessage( szCause, 255 );
      
			AfxMessageBox( szCause );
			exit(1);
		}
		END_CATCH	
		return CCSUCCEDED;
	}
	else
		return CCFAILED;		
	
	return CCSUCCEDED;
}
*/

/////////////////////////////////////////////////////////////////////////////
// *** Update the database
/////////////////////////////////////////////////////////////////////////////

E_ccErrRet ccDataDictionary::requery()
{ 
	if ( isOpen() )  //&& CanRestart()
	{
		try
		{
			_variant_t vBookMarkCurrentRecord;
			vBookMarkCurrentRecord.vt = VT_EMPTY;

			if (isNewRecord()) 
			{
				TESTHR( m_recordset->CancelUpdate() );
				setNewRecord(false);
			}
			else
			{
				vBookMarkCurrentRecord = m_recordset->Bookmark;					
			}

			TESTHR(m_recordset->Requery( ADODB::adOptionUnspecified ) );  // Maybe adAsyncExecute
			
			reBindAll();
		
			// Move back to the old record. But only if their is records 
			// in the recordset. And if a current record was selected.
			if ( !isNewRecord() )
			{
				try
				{
					if (vBookMarkCurrentRecord.vt == VT_EMPTY)
					{
						if (!( m_recordset->adoEOF && m_recordset->BOF ))
							return moveFirst();	
					}
					else
					{					
						if (!( m_recordset->adoEOF && m_recordset->BOF ))
							m_recordset->Bookmark = vBookMarkCurrentRecord;          			
					}					
				}
				catch(_com_error &e) 
				{
					// If the Bookmark is invalid and the recordset has
					// records move to the firstRecord. Else clear the record
					//
					if (isCathingNativeError(-68420629))
					{
						if (!( m_recordset->adoEOF && m_recordset->BOF ))
							return moveFirst();		
						else
							return clearRecord();
					}
					else
						throw e;
				}
			}
			else
				return moveFirst();		

			// If the recordset is now empty, then clear the fields
			// left over 
			if ( m_recordset->adoEOF && m_recordset->BOF )
			{
				if ( clearRecord() )
					return CCSUCCEDED;
				else
					return CCFAILED;
			}

			// If we moved off the end of file, then move back to last record			
			if ( m_recordset->adoEOF )
				TESTHR( m_recordset->MoveLast() );

			// If we moved before start of the file, then move back to first record	
			if ( m_recordset->BOF )
				TESTHR( m_recordset->MoveFirst() );

			// Copy fields from recordset to form, thus
			// overwriting any changes user may have made
			// on the form
			relateFind();		
			if (__getDB()->isReadyToUse()) OnFieldChangeComplete();
			refresh();			
			enableCanSave( true );
			setNewRecord( false );			
			
			return CCSUCCEDED;
		}
		catch(_com_error &e) 
		{ 
			//ccThrowccException( getProviderError( __getDB()->getConnection()) ); /*Disable unreference warning*/ (void*)&e; 
			ccErr( getProviderError( __getDB()->getConnection()) );/*Disable unreference warning*/ (void*)&e; 
			ccErr( getComError(e) );
			return CCFAILED;
		}							
	}
	else
	{
		ccErr( IDS_REQUERY_FAILED );
	    return CCFAILED;
	}
	return CCSUCCEDED;
}	
	

//

E_ccErrRet ccDataDictionary::saveRecord()
{
	try //mAYBE THE TRY CATCH SHOULD BE SOMEWHERE ELSE
	{
		try
		{
			if ( saveRecordNoClear() == CCSUCCEDED)
			{
				//moveNext();
				requery(); //todo test if performence will be okej with this on..
							// Maybe this should be in relateSave for relational datadictionarys

				return clearRecord();
			}
			else
				return CCFAILED;	
		}
		catch(_com_error &e) 
		{ 
			ccThrowccException( getProviderError( __getDB()->m_connection) );	/*Delete unreference warning*/ (void*)&e; 
		}
	}
	catch(ccException *e)
	{
		__getDB()->rollbackTrans();		// Maybe this only works in nonrelational dd:s
		ccErr(*e,"","", MB_OK);	
	}
	catch(...) 
	{
		ccErr("DB Error", "","", MB_OK);		
	}
	return CCFAILED;
}

//

E_ccErrRet ccDataDictionary::saveRecordNoClear()
{					
	if ( relateCanSave() )
	{
		if ( relateIsModified() )
		{
			if ( showConfirmMsg( getConfirmSaveMsg() ) == TRUE )
			{
				__getDB()->beginTrans();	// Maybe this only works in nonrelational dd:s
					relatePrepareSave();
					relateBackout();
					relateCreating();
					relateUpdateing();
					if ( relateSave() == CCFAILED )
					{
						ccErrEM( IDS_FATAL_SAVE_ERROR, NULL, NULL, MB_OK );
					}
				__getDB()->commitTrans(); // Maybe this only works in nonrelational dd:s

				// To see the new record 
				
				requery(); //todo test if performence will be okej with this on..
							// Maybe this should be in relateSave for relational datadictionarys

				// OnDeletecomplete nonDBCtrls
				for( int nIndex=0;nIndex<m_nonDBBoundCtrls.GetSize();nIndex++ ) 
				{
					ccNonDBCtrl* pCtrl=(ccNonDBCtrl*)m_nonDBBoundCtrls[nIndex];
					pCtrl->OnSaveComplete();
				}
			}
			else
				return CCFAILED;
		}
	}
	else
	{		
		ccErrEM( IDS_CANT_SAVE_RECORD, NULL, NULL, MB_OK );
		return CCFAILED;
	}

	return CCSUCCEDED;	
}

//

E_ccErrRet ccDataDictionary::deleteRecord()
{	
	if  (!isNewRecord() )
	{
		if ( showConfirmMsg( getConfirmDeleteMsg() ) == TRUE )
		{		
			// If already in add mode, then complete previous new record
			try
			{	

				// No cascade delete in this version.
				//relateDelete( NULL, NULL );
				
				TESTHR( m_recordset->CancelUpdate() );

				deleting();
				relateBackout();				

				TESTHR(m_recordset->Delete( ADODB::adAffectCurrent) );

				moveNext();

				// Update the recordset and move to next record, or first
				// record if anything is wrong.
				requery();

				if (__getDB()->isReadyToUse()) OnFieldChangeComplete();

				// OnDeletecomplete nonDBCtrls
				for( int nIndex=0;nIndex<m_nonDBBoundCtrls.GetSize();nIndex++ ) 
				{
					ccNonDBCtrl* pCtrl=(ccNonDBCtrl*)m_nonDBBoundCtrls[nIndex];
					pCtrl->OnDeleteComplete();
				}

				return CCSUCCEDED;
			}
			catch(_com_error &e) 
			{ 
				//ccThrowccException( getProviderError( __getDB()->getConnection()) ); /*Disable unreference warning*/ (void*)&e; 
				ccErr( getProviderError( __getDB()->getConnection() ), "","",MB_OK );/*Disable unreference warning*/ (void*)&e; 
				return CCFAILED;
			}							
		}
		else
			return CCFAILED;
	}
	else
		return CCSUCCEDED;
}

//

E_ccErrRet ccDataDictionary::clearRecord( bool bDontShowConfirmMsg /*=false*/)
{	
	try
	{
		try
		{
			if ( relateCanClear() )
			{
				// Only ask user for clear confirmation if dd is modified.
				if (!bDontShowConfirmMsg)
					if ( relateIsModified() )
						if ( showConfirmMsg( getConfirmClearMsg() ) == FALSE )
							return CCFAILED;

				E_ccErrRet nRet = relateClear();
					
				return nRet;
			}
			else
			{
				ccErr( IDS_CANT_CLEAR_RECORD );
				return CCFAILED;
			}
			return CCSUCCEDED;
		}
		catch(_com_error &e) 
		{ 
			//ccThrowccException( getProviderError( __getDB()->getConnection()) ); /*Disable unreference warning*/ (void*)&e; 
			ccErr( getProviderError( __getDB()->getConnection()) );/*Disable unreference warning*/ (void*)&e; 
			return CCFAILED;
		}	
	}
	catch(ccException *e) 
	{
		ccErr(*e,"","", MB_OK);	
	}	
	catch(...) 
	{
		ccErr("Unknown DB Error", "","", MB_OK);		
	}
	return CCFAILED;
}


//

E_ccErrRet ccDataDictionary::findRecord( ccField * aField)
{
	/* Not tested */
	/*
	try
	{				
		TESTHR( m_recordset->CancelUpdate() );

		// The table is empty
		if ( m_recordset->adoEOF || m_recordset->BOF )
		{
			MessageBeep( MB_ICONEXCLAMATION );

			if ( clearRecord() )
				return CCSUCCEDED;
			else
				return CCFAILED;
		}

		if ( !m_recordset->adoEOF ) 
		{
			ccString strCriteria;
			strCriteria << aField->getColumnName() << " = '" << aField->getValue(true) << "'";			
			TESTHR( m_recordset->Find((_bstr_t)strCriteria, 0, ADODB::adSearchForward, "") );							
		}

		if ( m_recordset->adoEOF )
		{
			TESTHR( m_recordset->MoveLast() );
			MessageBeep( MB_ICONEXCLAMATION );
		}

		relateFind();		
		if (__getDB()->isReadyToUse()) OnFieldChangeComplete();
		refresh();		
		enableCanSave( true );
		setNewRecord( false );
		
		
		return CCSUCCEDED;
	}
	catch(ccException *e) 
	{
		ccErr(*e,"","", MB_OK);	
	}
	catch(...) 
	{
		ccErr("DB Error", "","", MB_OK);		
	}
	
	return CCFAILED;
	*/

	return CCSUCCEDED;
}

/////////////////////////////////////////////////////////////////////////////
// *** Relation
/////////////////////////////////////////////////////////////////////////////

void ccDataDictionary::relate( ccField & aChild, ccField & aParent)
{
	// Is parent sorted
	if ( !aParent.isColumnSorted() )
	{		
		ccThrowccException( IDS_PARENT_ISNT_SORTED );		
	}

	// Parent must have a primkey
	if ( aParent.getDD()->getPrimKey() == NULL )
	{		
		ccThrowccException( IDS_PARENT_MUST_HAVE_A_PRIM_KEY );		
	}

	// Is child a member of this datadictionary
	if ( aChild.getDD() != this )
	{		
		ccThrowccException( IDS_CHILD_MUST_BE_MEMBER_OF_THE_CURRENT_DD );		
	}

	// Is child already related.
	for( int nIndex=0;nIndex<m_childField.GetSize();nIndex++ ) 
	{
        ccField* pField=(ccField*)m_childField[nIndex];
		if ( pField->getFieldID() == aChild.getFieldID() )
		{			
			ccThrowccException(IDS_CHILD_ALREADY_HAS_A_RELATION );			
		}		
	}		

	// Is parent datadictionary already related.
	for( nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{
        ccField* pField=(ccField*)m_parentField[nIndex];
		
		if ( pField->getDD() == aParent.getDD() )
		{			
			ccThrowccException( IDS_A_RELATION_TO_THE_PARENT_DD_ALREADY_EXIST );			
		}		
	}		

	// Do the relation.
	m_childField.Add( &aChild );
	m_parentField.Add( &aParent );

	aParent.getDD()->addChildDDField( aChild, aParent );
}

//

void ccDataDictionary::addChildDDField( ccField & aChildField, ccField & aParentField  )
{
	m_childDDSChildField.Add( &aChildField );
	m_childDDSParentField.Add( &aParentField );
}

//

void ccDataDictionary::relateFind()
{
	for( int nIndex=0;nIndex<m_childField.GetSize();nIndex++ ) 
	{
        ccField* pChildField=(ccField*)m_childField[nIndex];
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		pParentField->setValue( pChildField->getValue() );
		pParentField->getDD()->findRecord( pParentField );
	}		
}

//

void ccDataDictionary::relateDelete( ccField * aColumn, ccField * aValue )
{
	MessageBox(NULL, "Not Supported Yet", "Error", NULL);
/*
	// Delete all childs first.
	for( int nIndex=0;nIndex<m_childDDSChildField.GetSize();nIndex++ ) 
	{    
		ccField* pChildDDSChildField=(ccField*)m_childDDSChildField[nIndex];
		ccField* pChildDDSParentField=(ccField*)m_childDDSParentField[nIndex];

		pChildDDSChildField->getDD()->relateDelete( pChildDDSChildField,
													pChildDDSParentField
												   );
	}		

	if ( aColumn != NULL)
	{
		CString lsTable;  lsTable  = aColumn->getDD()->GetTableName();
		CString lsColumn; lsColumn = aColumn->getColumnName();
		CString lsValue;  lsValue  = aValue->getValue();

		CancelUpdate();
		for (;;)
		{

			// run those members for every deleted record.
			deleting();
			relateBackout();
			relateUpdateing();

			// find record

			// delete record

			// if no more records 
			   // break			  
		}

		// Update the recordset and move to next record, or first
		// record if anything is wrong.
		requery();				
	}
*/
}

//

BOOL ccDataDictionary::relateCanClear()
{
	// Return false if this dd or any of the parents canUpdate()
	// returns false.
	if (canUpdate() == false)
		return FALSE;
	for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		if ( pParentField->getDD()->canUpdate() == false )
			return FALSE;
	}		

	return TRUE;
}

//

E_ccErrRet ccDataDictionary::relateClear()
{
	// Clear parents before childs
	for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		if ( pParentField->getDD()->relateClear() == CCFAILED )
			return CCFAILED;
	}		

	// Clear child dd

	// If already in add mode, then complete previous new record
	try
	{			
		TESTHR( m_recordset->CancelUpdate() );
		TESTHR(m_recordset->AddNew() );
	}
	catch(_com_error &e) 
	{ 
		//ccThrowccException( getProviderError( __getDB()->getConnection()) ); /*Disable unreference warning*/ (void*)&e; 
		ccErr( getProviderError( __getDB()->getConnection()) );/*Disable unreference warning*/ (void*)&e; 
		return CCFAILED;
	}	

	if (__getDB()->isReadyToUse()) OnFieldClearComplete();						

	refresh();	
	enableCanSave( true );
	setNewRecord( true );
	// TODO Delete setModified( false );

	return CCSUCCEDED;
}

//

E_ccErrRet  ccDataDictionary::relateSave()
{
	// Save the parents before the childs
	for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		pParentField->getDD()->relateSave();
	}			

	try
	{		
		TESTHR(m_recordset->Update() );
		
		setModified(false);

	}
	catch(_com_error &e) 
	{ 
		ccThrowccException( getProviderError( __getDB()->getConnection()) ); /*Disable unreference warning*/ (void*)&e; 
		//ccErr( getProviderError( __getDB()->getConnection()), NULL );/*Disable unreference warning*/ (void*)&e; 
		return CCFAILED;
	}	

	return CCSUCCEDED;
}

//

void ccDataDictionary::relatePrepareSave()
{
	// Prepare the parents before the childs
	for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		pParentField->getDD()->relatePrepareSave();
	}			

	// Prepare child dd
	for( nIndex=0;nIndex<m_childField.GetSize();nIndex++ ) 
	{
        ccField* pChildField=(ccField*)m_childField[nIndex];
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		pChildField->setValue( pParentField->getValue() );		
	}		
}

//

BOOL ccDataDictionary::relateCanSave()
{	
	int nIndex;

	// Return false if this dd or any of the parents canSave()
	if (canSave() == false)
		return false;
	for( nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		if ( pParentField->getDD()->canSave() == false )
			return FALSE;
	}		
			
	// Return false if this dd or any of the parents canUpdate()
	if (canUpdate() == false)
		return false;
	for( nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		if ( pParentField->getDD()->canUpdate() == false )
			return FALSE;
	}		
	
	// Return false if this dd or any of the parents IsDeleted()
	if (isDeleted() == TRUE)
		return false;
	for( nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		if ( pParentField->getDD()->isDeleted() == TRUE )
			return FALSE;
	}		

	return TRUE;
}					

//

BOOL ccDataDictionary::relateIsModified() 
{
	if ( isModified() )
		return TRUE;
	else
	{		
		// return true if this dd or any of the parents IsModified()
		for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
		{    
			ccField* pParentField=(ccField*)m_parentField[nIndex];
			if ( pParentField->getDD()->relateIsModified() == TRUE)
				return TRUE;
		}		

		return FALSE;
	}
}

//

void ccDataDictionary::relateBackout()
{
	// relateBackout parents before childs
	for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		pParentField->getDD()->relateBackout();			
	}			

	if ( !isNewRecord() )	
		backout();
}

//

void ccDataDictionary::relateCreating()
{
	// relateCreating parents before childs
	for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		pParentField->getDD()->relateCreating();
	}			

	if ( isNewRecord() )	
		creating();
}

//

void ccDataDictionary::relateUpdateing()
{
	updateing();

	// relateUpdateing childs before parents
	for( int nIndex=0;nIndex<m_parentField.GetSize();nIndex++ ) 
	{    
		ccField* pParentField=(ccField*)m_parentField[nIndex];
		pParentField->getDD()->relateUpdateing();
	}			
}

/////////////////////////////////////////////////////////////////////////////
// *** Constrain
/////////////////////////////////////////////////////////////////////////////
/*
void ccDataDictionary::clearConstrain()
{
	m_constrainFilter = "";
}

//

void ccDataDictionary::addConstrain(ccField *aoConstrainField, CString aCompOp, int aiConstrainValue)
{
	NEEDVALUE ( aiConstrainValue );

	if ( !m_constrainFilter.IsEmpty() )
		m_constrainFilter += " AND ";
	m_constrainFilter += aoConstrainField->getColumnName();
	m_constrainFilter += " ";
	m_constrainFilter += aCompOp;
	m_constrainFilter += " ";
	m_constrainFilter += ccString( aiConstrainValue) ;			

	m_constrainFilter.Replace("[", "");
	m_constrainFilter.Replace("]", "");
}

//

void ccDataDictionary::addConstrain(ccField *aoConstrainField, CString aCompOp, CString asConstrainValue)
{
	NEEDVALUE( asConstrainValue.IsEmpty() );

	if ( !m_constrainFilter.IsEmpty() )
		m_constrainFilter += " AND ";
	m_constrainFilter += aoConstrainField->getColumnName();
	m_constrainFilter += " ";
	m_constrainFilter += aCompOp;
	m_constrainFilter += " ";
	m_constrainFilter += "'" + asConstrainValue + "'";

	m_constrainFilter.Replace("[", "");
	m_constrainFilter.Replace("]", "");
}

//

E_ccErrRet ccDataDictionary::executeConstrain()
{
	if ( filterBy( "" ) == CCFAILED )
	{
		ccErr( IDS_CONSTRAIN_FAILED );
		return CCFAILED;
	}	

	//All changes will be abandon when changing constrain	
	clearRecord();
	return CCSUCCEDED;
}

//

E_ccErrRet ccDataDictionary::filterBy(CString asFilter)
{
	// Add the constrained filter.
	m_strFilter = m_constrainFilter;
	if ( !m_strFilter .IsEmpty() && !asFilter.IsEmpty() )
		m_strFilter += " AND ";
	
	m_strFilter += asFilter;		

	if ( !requery() )
	{
		DEBUGOUTR( IDS_CANT_REFILTER );
		return CCFAILED;
	}						
	return CCSUCCEDED;
}
*/
/////////////////////////////////////////////////////////////////////////////
// *** Construction/Destruction
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// *** Construction/Destruction
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// *** Construction/Destruction
/////////////////////////////////////////////////////////////////////////////

