//////////////////////////////////////////////////////////////////////
// $File name   : ccField.cpp
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "ccdbCtrl.h"
#include "ccDataBase.h"
#include "ccField.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// *** Construction/Destruction
//////////////////////////////////////////////////////////////////////

ccField::ccField()
{	
	m_oDD = NULL;
	
	enableColumnSorting( FALSE );	
	setCtrlType( CCSC_EDIT );

	m_customMask = "";
	m_placeHolder = '_';	

	//m_RFXpublicValue = "";	

	m_adoField = NULL;
}

//

void ccField::bind( ccDataDictionary * aoDD, UINT nID, LPCSTR asColumnName, ADODB::Field * aADOField )
{
	NEEDZERO( m_oDD );
	NEEDVALUE( aoDD );
	NEEDVALUE( nID );

	m_oDD = aoDD;
		
	setFieldID( nID );

	setColumnName( asColumnName );	
	
	m_adoField = aADOField;
	
	//Set caption to the column name without [ ] and _..
	CString lsCaption = asColumnName;
	lsCaption.Replace("[","");
	lsCaption.Replace("]","");
	lsCaption.Replace("_"," ");
	setShortCaption( lsCaption );	
	setLongCaption( lsCaption );		

	initFieldType();
}

//

ccField::~ccField()
{

}

//

BOOL ccField::addAttachedCtrl( ccdbCtrl * aodbCtrl )
{
	NEEDVALUE( aodbCtrl );

	m_attachedCtrls.Add( aodbCtrl );

	return true;
}

//

BOOL ccField::removeAttachedCtrl( ccdbCtrl * aodbCtrl )
{
	NEEDVALUE( aodbCtrl );

	for( int nIndex=0;nIndex<m_attachedCtrls.GetSize();nIndex++ ) 
	{
        ccdbCtrl* lpAttchedCtrls=(ccdbCtrl*)m_attachedCtrls[nIndex];
		if ( lpAttchedCtrls == aodbCtrl)
			m_attachedCtrls.RemoveAt(nIndex);
	}

	return true;
}
//////////////////////////////////////////////////////////////////////
// *** Attributes
//////////////////////////////////////////////////////////////////////
ADODB::Field * ccField::getADOField()
{ 
	if ( getDD()->getRS()->Status == ADODB::adRecDeleted)
		return NULL;

	if ( getDD()->getRS()->Status == ADODB::adRecDBDeleted)
		return NULL;


	if ( m_adoField->Status == ADODB::adFieldOK)
		return m_adoField;	
	else
		return NULL;					
};
//

CString ccField::getADOFieldValue()
{ 	
	if ( getDD() )
		if (!(getDD()->getRS()->adoEOF || getDD()->getRS()->BOF ))
		{
			if (getADOField())
			{
				_variant_t vSTR = getADOField()->Value;		
				if (vSTR.vt != VT_NULL)	
					return (char*)(_bstr_t)vSTR;
			}
		}
	
	return "";						
};

//////////////////////////////////////////////////////////////////////
// *** Attributes of the table field, and the connected ccdbSuperCtrl
//////////////////////////////////////////////////////////////////////
void ccField::initFieldType( )
{		
	switch ( getFieldType() )
	{
	case ADODB::adEmpty :		
		ccErrEM( IDS_UNDEFINED_TYPE );
		break;
    case ADODB::adBoolean :		
		setCtrlType( CCSC_CHECKBOX );
		break;
    case ADODB::adBinary :		
	case ADODB::adVarBinary :		
		setCtrlType( CCSC_PICTURE);
		break;

	case ADODB::adDate :
	case ADODB::adLongVarWChar :		
    case ADODB::adVarWChar :		//Access type 'Text', SQL-Type 'VarChar'    
	case ADODB::adInteger :		
		setCtrlType( CCSC_EDIT );
		break;
    
    case ADODB::adTinyInt :		
	case ADODB::adVarChar :		
    case ADODB::adSmallInt :		    
    case ADODB::adBigInt :		
    case ADODB::adUnsignedTinyInt :		
    case ADODB::adUnsignedSmallInt :		
    case ADODB::adUnsignedInt :		
    case ADODB::adUnsignedBigInt :		
    case ADODB::adSingle :		
    case ADODB::adDouble :		
    case ADODB::adCurrency :		
    case ADODB::adDecimal :		
    case ADODB::adNumeric :	
    case ADODB::adError :		
    case ADODB::adUserDefined :		
    case ADODB::adVariant :		
    case ADODB::adIDispatch :		
    case ADODB::adIUnknown :		
    case ADODB::adGUID :		    
    case ADODB::adDBDate :		
    case ADODB::adDBTime :		
    case ADODB::adDBTimeStamp :		
    case ADODB::adBSTR :		
    case ADODB::adChar :		
    case ADODB::adLongVarChar :		
    case ADODB::adWChar :		        
    case ADODB::adLongVarBinary :		
    case ADODB::adChapter :		
    case ADODB::adFileTime :		
    case ADODB::adPropVariant :		
    case ADODB::adVarNumeric :		
    case ADODB::adArray :		
		{
			CString strMessage;
			strMessage.Format(IDS_FIELD_TYPE_NOT_SUPORTED, (int)getFieldType());	
			ccErr(  strMessage);
		}
		break;

	default:
		{
			CString strMessage;
			strMessage.Format(IDS_INVALID_FIELD_TYPE , (int)getFieldType());	
			ccErr(  strMessage);
		}
	}			 
};

//

void  ccField::modifyOptions( DWORD aRemove, DWORD aAdd)
{
	DWORD dwNewOptions = (m_options & ~aRemove) | aAdd;
	
	if (m_options == dwNewOptions)
		return ;

	// Remove Options
	//if ( aRemove & DD_AUTOBACK )

	//if ( aRemove & DD_AUTOFIND )

	//if ( aRemove &  DD_AUTOFIND_GE )

	//if ( aRemove &  DD_AUTORETURN )

	//if ( aRemove &  DD_CAPSLOCK )

	//if ( aRemove &  DD_DISPLAYONLY )

	//if ( aRemove &  DD_FINDREQ )

	//if ( aRemove &  DD_FORCEPUT )

	//if ( aRemove &  DD_NOENTER )

	//if ( aRemove &  DD_NOPUT )

	//if ( aRemove &  DD_REQUIRED )

	//if ( aRemove &  DD_RETAIN )

	//if ( aRemove &  DD_RETAIN_ALL )

	//if ( aRemove &  DD_SKIPFOUND )

	//if ( aRemove &  DD_ZERO_SUPPRESS )


	// Add Options
	//if ( aRemove & DD_AUTOBACK )

	//if ( aRemove & DD_AUTOFIND )

	//if ( aRemove &  DD_AUTOFIND_GE )

	//if ( aRemove &  DD_AUTORETURN )

	//if ( aRemove &  DD_CAPSLOCK )

	//if ( aRemove &  DD_DISPLAYONLY )

	//if ( aRemove &  DD_FINDREQ )

	//if ( aRemove &  DD_FORCEPUT )

	//if ( aRemove &  DD_NOENTER )

	//if ( aRemove &  DD_NOPUT )

	//if ( aRemove &  DD_REQUIRED )

	//if ( aRemove &  DD_RETAIN )

	//if ( aRemove &  DD_RETAIN_ALL )

	//if ( aRemove &  DD_SKIPFOUND )

	//if ( aRemove &  DD_ZERO_SUPPRESS )
}


//////////////////////////////////////////////////////////////////////
// *** 	Attributes for the connected ccdbSuperCtrl
//////////////////////////////////////////////////////////////////////
void ccField::setCtrlType( E_CCSC aCtrlType )
{ 
	m_ctrlType = aCtrlType;				

	switch ( m_ctrlType )
	{
	case CCSC_EDIT:
		setStyle( ES_AUTOHSCROLL );
		setexStyle(0);
		setMaskType( CCE_DB_DEFAULT );		
		break;

	case CCSC_COMBO:
		setStyle( ES_AUTOHSCROLL );
		setexStyle(0);
		break;

	case CCSC_CHECKBOX:
		setStyle(0);
		setexStyle(0);
		break;

	case CCSC_PICTURE:
		break;

	case CCSC_INVALID:		
	default:
		ccErr( IDS_INVALID_CTRL_TYPE );
		DEBUGOUT( "IDS_INVALID_CTRL_TYPE" );
		break;
	}

};

//

void ccField::enableWindow( BOOL abEnable )
{
	if ( abEnable)
	{
		setStyle( getStyle() | WS_DISABLED );	
		setStyle( getStyle() | WS_DISABLED );		
	}
	else
	{
		setStyle( getStyle() & ~WS_DISABLED );
		setStyle( getStyle() & ~WS_DISABLED );		
	}
}

//

void ccField::setEditMultiLine(BOOL abEnable)
{
	if ( abEnable)
	{
		setStyle( getStyle() | ES_AUTOVSCROLL | ES_MULTILINE | ES_WANTRETURN );		
		setStyle( getStyle() & ~ES_AUTOHSCROLL );
		
	}
	else
	{
		setStyle( getStyle() & ~(ES_AUTOVSCROLL | ES_MULTILINE | ES_WANTRETURN ) );
		setStyle( getStyle() | ES_AUTOHSCROLL  );	
	}
}


//////////////////////////////////////////////////////////////////////
// *** 	Attributes for the connected ccdbSuperCtrl if ctrlType is CCSC_EDIT and CCSC_CHECKBOX
//////////////////////////////////////////////////////////////////////

void ccField::setCustomMask( CString asMask, char acPlaceHolder /*= '_' */)
{
	setMaskType( CCE_CUSTOM );
	m_customMask = asMask;
	m_placeHolder = acPlaceHolder;	
}

//

BOOL ccField::refresh( void )
{
	for( int nIndex=0;nIndex<m_attachedCtrls.GetSize();nIndex++ ) 
	{
        ccdbCtrl* lpAttchedCtrls=(ccdbCtrl*)m_attachedCtrls[nIndex];
		lpAttchedCtrls->refresh();
	}
	return true;
}

//////////////////////////////////////////////////////////////////////
// *** Input/Output
//////////////////////////////////////////////////////////////////////

void ccField::setValue( CString &asValue )
{
	try
	{
		try
		{
			/* Update the datadictionary value and Do a 
			   refresh on the "Refresh ProcList", so all 
			   controls connected to this field will get
			   the latest value.*/
			if ( getADOFieldValue() != asValue )
			{	
				if ( getADOField() )
				{
					getADOField()->Value = (bstr_t)asValue;		// TODO Is this right
					refresh();
				}
			};
		}
		catch(_com_error &e) 
		{ 
			if ( getDD()->isCathingNativeError(-66585593) )
			{
				getDD()->requery();
				getDD()->moveFirst();
				ccErrEM( IDS_THE_RECORD_YOU_WHERE_MOVING_TO_WAS_DELETED, NULL, NULL, MB_OK );
				return;
			}
			ccThrowccException( getProviderError( __getDB()->m_connection) );	/*Delete unreference warning*/ (void*)&e; 			
		}	
	}
	catch(ccException *e) 
	{
		ccErr(*e,"","", MB_OK);	
	}	
	catch(...) 
	{
		ccErr("DB Error", "","", MB_OK);		
	}
}

//

void ccField::setValue( int aiValue )
{
	CString lsValue;
	lsValue.Format("%d", aiValue);
	/* Update the datadictionary value and Do a 
	   refresh on the "Refresh ProcList", so all 
	   controls connected to this field will get
	   the latest value.*/	
	if ( getADOFieldValue() != lsValue )
	{		
		if ( getADOField() )
		{
			getADOField()->Value = (bstr_t)lsValue;		// TODO Is this right
			refresh();
		}
	};
}

//

void ccField::setValue( ccTime & aTime )
{
	CString lsValue;
	
	// Get the string formated date and time.
	switch ( getMaskType() ) 
	{
	case CCE_DATETIME:
		lsValue = aTime.getDateTime();
		break;
	case CCE_DATE:
		lsValue = aTime.getDate();
		break;
	case CCE_TIME:
		lsValue = aTime.getTime();
		break;
	case CCE_TEXT:
		lsValue = aTime.getDateTime();
		break;
	default:
		ccErr( IDS_NOT_A_DATE_MASKED_FIELD );		
		return;
	}

	/* Update the datadictionary value and Do a 
	   refresh on the "Refresh ProcList", so all 
	   controls connected to this field will get
	   the latest value.*/
	if ( getADOFieldValue() != lsValue )
	{	
		if ( getADOField() )
		{
			getADOField()->Value = (_bstr_t)lsValue;		// TODO Is this right
			refresh();
		}
	};
}

void ccField::setCurrentDate()
{
	ccTime * lCurrentDate;
	lCurrentDate = new ccTime;
	lCurrentDate  = (ccTime*)(&ccTime::GetCurrentTime());	
	setValue( *lCurrentDate  );
}

//

CString	ccField::getValue(BOOL abGetRFXValue /*=false*/)
{ 
	// TODO if(!m_recordset->adoEOF) ngot i stil med detta br ka in hr...
	try
	{
		if ( abGetRFXValue == false )	
		{			
			switch ( getFieldType() )
			{
			case ADODB::adEmpty:
				ccErrEM( IDS_UNDEFINED_TYPE );
				break;
						
			//The value set to the ccdbSuperCtrl must be without '-', ' ' and ':' so the mask edit can do it's job.
			case ADODB::adDate:			
				{
					CString lsDateValue;
					lsDateValue = getADOFieldValue();					
					lsDateValue.Replace(":","");
					lsDateValue.Replace("-","");
					lsDateValue.Replace(" ","");
					
					return lsDateValue;
				}
				break;
				
				
			case ADODB::adLongVarWChar:
			case ADODB::adVarWChar:
			case ADODB::adBoolean:
			case ADODB::adInteger:
				return getADOFieldValue();
				break;

			case ADODB::adVarBinary:
			case ADODB::adBinary:
				break;

			case ADODB::adTinyInt:
			case ADODB::adSmallInt:		
			case ADODB::adBigInt:
			case ADODB::adUnsignedTinyInt:
			case ADODB::adUnsignedSmallInt:
			case ADODB::adUnsignedInt:
			case ADODB::adUnsignedBigInt:
			case ADODB::adSingle:
			case ADODB::adDouble:
			case ADODB::adCurrency:
			case ADODB::adDecimal:
			case ADODB::adNumeric:		
			case ADODB::adError:
			case ADODB::adUserDefined:
			case ADODB::adVariant:
			case ADODB::adIDispatch:
			case ADODB::adIUnknown:
			case ADODB::adGUID:			
			case ADODB::adDBDate:
			case ADODB::adDBTime:
			case ADODB::adDBTimeStamp:
			case ADODB::adBSTR:
			case ADODB::adChar:		
			case ADODB::adLongVarChar:
			case ADODB::adWChar:		
			case ADODB::adVarChar:									
			case ADODB::adLongVarBinary:
			case ADODB::adChapter:
			case ADODB::adFileTime:
			case ADODB::adPropVariant:
			case ADODB::adVarNumeric:
			case ADODB::adArray:
				{
				CString strMessage;
				strMessage.Format(IDS_FIELD_TYPE_NOT_SUPORTED, (int)getFieldType());	
				ccErr(  strMessage);
				}
				break;

			default:
				{
				CString strMessage;
				strMessage.Format(IDS_INVALID_FIELD_TYPE , (int)getFieldType());	
				ccErr(  strMessage);
				}
			}
		}
		else
			return getADOFieldValue();
	}
	catch(_com_error &e) 
	{ 		
		ccThrowccException( getComError(e) );	/*Delete unreference warning*/ (void*)&e; 
	}	
	return "";
};	

//

ccField::operator int()
{
	// ** TODO check so the getFieldType returns a numeric type.
		
	return atoi( getValue() );
}

//

ccField::operator CString()
{
	return getValue();
}


//////////////////////////////////////////////////////////////////////
// *** Field events.
//////////////////////////////////////////////////////////////////////

BOOL ccField::OnSetFocus(ccdbSuperCtrl *aoCtrl)
{
	DD_DISPINFO	lDD_DispInfo;
	lDD_DispInfo.m_field     = this;
	lDD_DispInfo.m_superCtrl = aoCtrl;

	return getDD()->OnFieldEvent( CCDD_ONSETFOCUS, &lDD_DispInfo );
}

//

BOOL ccField::OnKillFocus(ccdbSuperCtrl *aoCtrl)
{
	DD_DISPINFO	lDD_DispInfo;
	lDD_DispInfo.m_field     = this;
	lDD_DispInfo.m_superCtrl = aoCtrl;

	return getDD()->OnFieldEvent( CCDD_ONKILLFOCUS, &lDD_DispInfo );
}

BOOL ccField::OnValidate(ccdbSuperCtrl *aoCtrl)
{
	DD_DISPINFO	lDD_DispInfo;
	lDD_DispInfo.m_field     = this;
	lDD_DispInfo.m_superCtrl = aoCtrl;

	return getDD()->OnFieldEvent( CCDD_ONVALIDATE, &lDD_DispInfo );
}

//

BOOL ccField::OnChange(ccdbSuperCtrl *aoCtrl)
{
	DD_DISPINFO	lDD_DispInfo;
	lDD_DispInfo.m_field     = this;
	lDD_DispInfo.m_superCtrl = aoCtrl;

	return getDD()->OnFieldEvent( CCDD_ONCHANGE, &lDD_DispInfo );
}

//

BOOL ccField::OnPopup(ccdbSuperCtrl *aoCtrl)
{
	DD_DISPINFO	lDD_DispInfo;
	lDD_DispInfo.m_field     = this;
	lDD_DispInfo.m_superCtrl = aoCtrl;

	return getDD()->OnFieldEvent( CCDD_ONPOPUP, &lDD_DispInfo );
}

//

BOOL ccField::hasPopup()
{
	DD_DISPINFO	lDD_DispInfo;
	lDD_DispInfo.m_field = this;
										
	return getDD()->OnFieldEvent( CCDD_HASPOPUP, &lDD_DispInfo );
}



//////////////////////////////////////////////////////////////////////
// *** Construction/Destruction
//////////////////////////////////////////////////////////////////////


