//////////////////////////////////////////////////////////////////////
// $File name   : ccInPlaceEdit.cpp
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TCHAR.h"
#include "ccDataBase.h"
#include "ccdbSuperCtrl.h"
#include "ccInplaceSuperCtrl.h"
#include "ccdbGrid.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// *** Construction
/////////////////////////////////////////////////////////////////////////////

ccInplaceSuperCtrl::ccInplaceSuperCtrl(CWnd* pParent, CRect& rect, DWORD dwStyle, UINT nID,
                           int nRow, int nColumn, ccField *atableField, 
                           UINT nFirstChar)
{

	adddbField( atableField  );

    m_sInitText     = atableField->getValue();
    m_nRow          = nRow;
    m_nColumn       = nColumn;
    m_nLastChar     = 0; 
	m_floatMenuID   = -1;		
    
    DWORD dwEditStyle =  WS_CHILD | WS_VISIBLE ;
    
	enableCaption( false );
    
    if (!Create(rect, pParent, nID, dwEditStyle)) return;
	    
	if ( m_ccdbEdit != NULL)
	{		    
		m_ccdbEdit->SetWindowText(m_sInitText);
		    
		switch (nFirstChar){
			case VK_LBUTTON: 
			case VK_RETURN:   m_ccdbEdit->SetSel( (int)_tcslen(m_sInitText), -1); return;

			case VK_BACK:     m_ccdbEdit->SetSel( (int)_tcslen(m_sInitText), -1); break;

			case VK_TAB:
			case VK_DOWN: 
			case VK_UP:   
			case VK_RIGHT:
			case VK_LEFT:  
			case VK_NEXT:  
			case VK_PRIOR: 
			case VK_HOME:
			case VK_SPACE:
			case VK_END:      m_ccdbEdit->SetSel(0,-1); return;

			default:          m_ccdbEdit->SetSel(0,-1);
		}
    
		m_ccdbEdit->SendMessage(WM_CHAR, nFirstChar);
	}

	if ( m_ccdbCheckBox != NULL )
	{
		m_ccdbCheckBox->SetCheck(*atableField);
		m_ccdbCheckBox->SendMessage(WM_CHAR, nFirstChar);		
	}	
}

/////////////////////////////////////////////////////////////////////////////
// *** Destruction
/////////////////////////////////////////////////////////////////////////////

ccInplaceSuperCtrl::~ccInplaceSuperCtrl()
{
}

/////////////////////////////////////////////////////////////////////////////
// *** Properties
/////////////////////////////////////////////////////////////////////////////

void ccInplaceSuperCtrl::setPos(CRect aRect, int aiCol, int aiRow)
{    
	m_nColumn = aiCol;
	m_nRow    = aiRow;   
	MoveWindow( &aRect);

	if ( m_ccdbEdit != NULL)
	{
		m_ccdbEdit->SetWindowText( getdbField( getCrntField() )->getValue() );
	}

	if ( m_ccdbCheckBox != NULL )
	{
		m_ccdbCheckBox->SetCheck( *getdbField( getCrntField() ) );		
	}	
}

//

DWORD ccInplaceSuperCtrl::getEditexStyle()
{
	DWORD ldwexStyle =  m_dwexEditStyle | getdbField(  getCrntField()  )->getexStyle();
	ldwexStyle = ldwexStyle & ~WS_EX_STATICEDGE;
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// *** Generated message map functions
/////////////////////////////////////////////////////////////////////////////

BEGIN_MESSAGE_MAP(ccInplaceSuperCtrl, ccdbSuperCtrl)
    //{{AFX_MSG_MAP(ccInplaceSuperCtrl)        
    ON_WM_KEYDOWN()
	ON_WM_CONTEXTMENU()    
	ON_WM_LBUTTONDBLCLK()		
	ON_WM_GETDLGCODE()
	ON_WM_KILLFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//

void ccInplaceSuperCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// If an arrow key (or associated) is pressed and The Ctrl key is down, exit!	
    if (
		( 
		 (
		  nChar == VK_PRIOR || nChar == VK_NEXT ||		
          nChar == VK_DOWN  || nChar == VK_UP  
		 ) 
		) ||		
		(
		 (nChar == VK_HOME || nChar == VK_END ||
		  nChar == VK_LEFT || nChar == VK_RIGHT
		 )
		  && GetKeyState(VK_CONTROL) < 0
		)
	   )
    {
        m_nLastChar = nChar;
        GetParent()->SetFocus();
        return;
    }

    if (nChar == VK_TAB || nChar == VK_RETURN)
    {
        m_nLastChar = nChar;
        GetParent()->SetFocus();    // This will destroy this window
        return;
    }
    if (nChar == VK_ESCAPE) 
    {
        SetWindowText(m_sInitText);    // restore previous text
        m_nLastChar = nChar;        
        return;
    }	

    ccdbSuperCtrl::OnKeyDown(nChar, nRepCnt, nFlags);
	
}

//

void ccInplaceSuperCtrl::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	// if no menu, just return. 
	if ( m_floatMenuID == -1 )
	{ 		
		DefWindowProc(WM_CONTEXTMENU, LONG(m_hWnd), 
			MAKELPARAM(point.x,point.y)); 

		TRACE0(_T("Warning: No control bar menu defined.\n")); 
		return; 
	}

	if (point.x == -1 && point.y == -1)
	{
		//keystroke invocation
		CRect rect;
		GetClientRect(rect);
		ClientToScreen(rect);
		
		point = rect.TopLeft();
		point.Offset(5, 5);
	}
	
	CMenu menu;
	VERIFY(menu.LoadMenu(m_floatMenuID));
	
	CMenu* pPopup = menu.GetSubMenu(0);
	ASSERT(pPopup != NULL);
	CWnd* pWndPopupOwner = this;
	
	while (pWndPopupOwner->GetStyle() & WS_CHILD)
		pWndPopupOwner = pWndPopupOwner->GetParent();
	
	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,
		this);
}

//

void ccInplaceSuperCtrl::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    m_nLastChar = VK_RETURN;
    GetParent()->SetFocus();
    return;
}

//

UINT ccInplaceSuperCtrl::OnGetDlgCode() 
{
    return DLGC_WANTALLKEYS;
}

////////////////////////////////////////////////////////////////////////////
// *** Overrides
////////////////////////////////////////////////////////////////////////////

BOOL ccInplaceSuperCtrl::PreTranslateMessage(MSG* pMsg) 
{	
	
	switch( pMsg->message )
	{
	// Stoopid win95 accelerator key problem workaround - Matt Weagle.
    // Catch the Alt key so we don't choke if focus is going to an owner drawn button
	case WM_SYSCHAR:
		return TRUE;		
    case WM_KEYDOWN:
		{
			int nVK  = pMsg->wParam;//Virtual Key

			switch (nVK)
			{
			// Cancel those "database function keys" function, 
			case VK_F2:
			case VK_F3:
			case VK_F4:
			case VK_F5:
			case VK_F6:
			case VK_F7:
			case VK_F8:
				return true;			

			// Trap those message only if ctrl is done.				
			case VK_HOME: 
			case VK_END: 
			case VK_LEFT: 
			case VK_RIGHT:
				 if (!isCTRLPressed() )
					 break;
			case VK_PRIOR: 
			case VK_NEXT: 

			case VK_DOWN:  
			case VK_UP:  
			case VK_TAB:
			case VK_RETURN:
				// Is the crnt control the edit box, and is the keydown message sent to that ctrl, 		
				// Send the keydown message to the super ctrl instead
				if ( m_ccdbEdit != NULL)
					if ( m_ccdbEdit->m_hWnd == pMsg->hwnd)
					{					
						SendMessage( pMsg->message, pMsg->wParam,pMsg->lParam);				
						return TRUE;
					}

				// Is the crnt control the checkbox, and is the keydown message sent to that ctrl, 
				// Send the keydown message to the super ctrl instead
				if ( m_ccdbCheckBox != NULL)
					if ( m_ccdbCheckBox->m_hWnd == pMsg->hwnd)
					{
						SendMessage( pMsg->message, pMsg->wParam,pMsg->lParam);				
						return TRUE;
					}			
				break;
			default:				
				break;
			}
		}
		break;
	case WM_RBUTTONDOWN:
		SendMessage(WM_CONTEXTMENU, pMsg->wParam, MAKELPARAM(pMsg->pt.x, pMsg->pt.y) );
		return true;

	case WM_CONTEXTMENU:    
    case WM_GETDLGCODE:
	case WM_LBUTTONDBLCLK:
		SendMessage(pMsg->message, pMsg->wParam, pMsg->lParam );
		return true;
	}
    
    return ccdbSuperCtrl::PreTranslateMessage(pMsg);	

}

//

BOOL ccInplaceSuperCtrl::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if ( m_ccdbEdit != NULL)
	{
		// Handles the floating popup menu.
		if (HIWORD(wParam) == 0)
		{
			switch LOWORD(wParam)
			{
			case ID_SELECTEDITEM_OPEN_GRID_ITEM:
				m_nLastChar = VK_RETURN;
				GetParent()->SetFocus();
				return true;

			case ID_SELECTEDITEM_DELETE_GRID_ITEM:
				MessageBox("Not Implemented", "Not Implemented");
				return true;

			case ID_EDIT_UNDO:
				m_ccdbEdit->Undo();
				return true;

			case ID_EDIT_CUT:
				m_ccdbEdit->Cut();
				return true;

			case ID_EDIT_COPY:
				m_ccdbEdit->Copy();	
				return true;

			case ID_EDIT_PASTE:
				m_ccdbEdit->Paste();
				return true;

			case ID_EDIT_DELETE:
				m_ccdbEdit->SetSel(0,-1);
				m_ccdbEdit->ReplaceSel("",TRUE);
				return true;

			case ID_EDIT_SELECT_ALL:
				m_ccdbEdit->SetSel(0,-1);
				return true;	

			//Strange,,, thougt this should come with the HIWORD(wParam) == BN_CLICKED, or something, now
			// it comes like it was a message from a menu.... But hey, it works...
			case ID_CCDBSUPERCTRL_COMBO_BUTTON:
				OnComboButtonButton();
				break;

			default:
				{
					CWnd* pWndPopupOwner = m_ccdbEdit;
					
					while (pWndPopupOwner->GetStyle() & WS_CHILD)
						pWndPopupOwner = pWndPopupOwner->GetParent();

					pWndPopupOwner->SendMessage(WM_COMMAND, wParam, lParam );
					return true;
				}
			}
		}
	}
	
	return ccdbSuperCtrl::OnCommand(wParam, lParam);
}

//

void ccInplaceSuperCtrl::endEdit()
{
	CString str;
	GetWindowText(str);
		
	// Send Notification to parent
	GV_DISPINFO dispinfo;

	dispinfo.hdr.hwndFrom = GetSafeHwnd();
	dispinfo.hdr.idFrom   = GetDlgCtrlID();
	dispinfo.hdr.code     = GVN_ENDLABELEDIT;

	dispinfo.item.mask    = LVIF_TEXT|LVIF_PARAM;
	dispinfo.item.row     = m_nRow;
	dispinfo.item.col     = m_nColumn;
	dispinfo.item.szText  = str;
	dispinfo.item.lParam  = (LPARAM) m_nLastChar; 
	m_nLastChar = 0;

	CWnd* pOwner = GetOwner();
	if (pOwner)
		pOwner->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&dispinfo );

	m_nLastChar = 0;

}

