//////////////////////////////////////////////////////////////////////
// $File name   : ccListPopupDialog.cpp
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ccDataBase.h"
#include "ccListPopupDialog.h"
#include "ccdbSuperCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// ccListPopupDialog dialog


ccListPopupDialog::ccListPopupDialog(ccdbSuperCtrl* aSuperCtrl/*=NULL*/)
	: ccPopupDialog(ccListPopupDialog::IDD, aSuperCtrl)
{
	//{{AFX_DATA_INIT(ccListPopupDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT			
	m_superCtrl = aSuperCtrl;
}

//

void ccListPopupDialog::DoDataExchange(CDataExchange* pDX)
{
	ccPopupDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(ccListPopupDialog)
	DDX_Control(pDX, IDC_POPUP_DIALOG_LIST, m_list);
	//}}AFX_DATA_MAP
}

//

BEGIN_MESSAGE_MAP(ccListPopupDialog, ccPopupDialog)
	//{{AFX_MSG_MAP(ccListPopupDialog)
	ON_WM_SIZE()	
	ON_NOTIFY(NM_KILLFOCUS, IDC_POPUP_DIALOG_LIST, OnKillfocusPopupDialogList)
	ON_NOTIFY(NM_DBLCLK, IDC_POPUP_DIALOG_LIST, OnDblclkPopupDialogList)	
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_POPUP_DIALOG_LIST, OnItemchangedPopupDialogList)
	//}}AFX_MSG_MAP
	ON_COMMAND(IDOK, OnOK)
	ON_COMMAND(IDCANCEL, OnCancel)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// ccListPopupDialog message handlers
BOOL ccListPopupDialog::OnInitDialog() 
{
	ccPopupDialog::OnInitDialog();

	// get the point where the menu is to be displayed.    
	CRect lClientRect;
	
	CRect lWindowRect;
	m_superCtrl->GetClientRect(lWindowRect);	
	m_superCtrl->GetWindowRect(lClientRect);	
	lClientRect.top  = lClientRect.bottom+1;	
	lClientRect.right = lWindowRect.Width();
	
	lClientRect.bottom = CCLPD_HEIGHT;		
		
	SetWindowPos(NULL, lClientRect.left, lClientRect.top, lClientRect.right, lClientRect.bottom,SWP_NOZORDER);

	m_list.InsertColumn(0,"",LVCFMT_LEFT, lWindowRect.Width()-10 );	
	m_list.SetExtendedStyle( LVS_EX_FULLROWSELECT );

	for( int i = 0; i < m_itemText.GetSize(); i++)
	{
		VERIFY( m_list.InsertItem(LVIF_TEXT|LVIF_PARAM, m_itemValue[i], m_itemText[i], 0, 0, 0, m_itemValue[i]) != -1);				
	}
  
	m_list.SetFocus();
	
	return FALSE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

//

CString ccListPopupDialog::getItem(int liItem)
{
	if (liItem < m_itemText.GetSize())
		return m_itemText[liItem];
	else
		return CString("");
}

//

void ccListPopupDialog::addItem(int liItem, CString asValue)
{
	if ( !asValue.IsEmpty() )
	{
		m_itemValue.Add( liItem );
		m_itemText.Add( asValue );	
	}
}

//

void ccListPopupDialog::OnSize(UINT nType, int cx, int cy) 
{
	ccPopupDialog::OnSize(nType, cx, cy);
	
	if (IsWindow(m_list.m_hWnd ) )
	{
		m_list.SetWindowPos(NULL,0,0,cx,cy,SWP_NOMOVE | SWP_NOZORDER);	
		m_list.SetColumnWidth(0, cx-10 );	
	}
}

//

void ccListPopupDialog::popup()
{
	m_bItemSelected = false;
	Create(ccListPopupDialog::IDD, m_superCtrl);
}

//

void ccListPopupDialog::OnKillfocusPopupDialogList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	DestroyWindow();
	
	*pResult = 0;
}

//

void ccListPopupDialog::OnCancel()
{	
	m_superCtrl->SetFocus();	// Execute onkillfocus
}

//

void ccListPopupDialog::OnOK()
{		
	m_superCtrl->SetFocus();	// Execute onkillfocus
}

//

void ccListPopupDialog::OnDblclkPopupDialogList(NMHDR* pNMHDR, LRESULT* pResult) 
{	
	m_superCtrl->SetFocus();	// Execute onkillfocus
	
	*pResult = 0;
}

//

void ccListPopupDialog::OnItemchangedPopupDialogList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	
	m_bItemSelected = true;

	CString lsText = getItem( m_list.GetItemData( pNMListView->iItem ) );
	m_superCtrl->getdbField(m_superCtrl->getCrntField())->setValue( lsText );

	*pResult = 0;
}

//

BOOL ccListPopupDialog::DestroyWindow() 
{
	BOOL bRet = TRUE;
	if ( ::IsWindow(this->m_hWnd) )
	{	
		if ( m_bItemSelected == true)
		{
			//Call the OnEndPopup function.
			DD_DISPINFO	lDD_DispInfo;
			DDEP_DISPINFO * lDDEP_DispInfo;		
			lDDEP_DispInfo	= new DDEP_DISPINFO;			
			lDD_DispInfo.m_endPopupInfo = lDDEP_DispInfo;
			lDD_DispInfo.m_field		= m_superCtrl->getdbField( m_superCtrl->getCrntField() );
			lDD_DispInfo.m_superCtrl    = m_superCtrl;

			lDDEP_DispInfo->m_itemNum  = m_list.GetItemData( m_list.GetSelectionMark() );		// The data on the selected item
			lDDEP_DispInfo->m_itemText = getItem( lDDEP_DispInfo->m_itemNum  );					// The string on the selected item
			
			lDD_DispInfo.m_field->getDD()->OnFieldEvent( CCDD_ONENDPOPUP, &lDD_DispInfo );
					
			delete lDDEP_DispInfo;
			
			m_superCtrl->SetFocus();
		}
		
		
		bRet = ccPopupDialog::DestroyWindow();

		//delete this;
	}

	return bRet;
}