// ==========================================================================
//                    Class Specification : ccdbGrid
// ==========================================================================
//////////////////////////////////////////////////////////////////////
// $File name   : /ccLib/dev/ccdbLib/ccdbGrid.h: 
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// $HomePage    : home.bip.net/CyberCow
// $ICQ         : 7983755
//
// Notice       : Copyright (c) CyberCow 1999
//                Knypplerskevagen 24
//                168 38 BROMMA
//                SWEDEN  
//                tel int + 46 (8) 26 45 95
//                
//                The copyright to the computer program(s) herein is the 
//				  property of CyberCow.The program(s) may be used and/or 
//				  copied only with the written permission of CyberCow or 
//				  in accordance with the terms and conditions stipulated 
//				  in the agreement/contract under which the program(s) 
//				  have been supplied. 
//
/////////////////////////////////////////////////////////////////////////////
///** $History$ *************************************************************
// *
// * $Date: 10/15/99 10:17 $
// * $Revision: 1 $
// * 
// * *****************  Version 1  *****************
// * User: Daniel Date: 10/15/99 10:17
// * Source-Code Traced [ ]; Reviewd [ ] 
// * Created in $/ccLib/dev/ccdbLib
// * Initial release.
// * 
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
// ** $Properties$ **********************************************************
// * [ ] Abstract class (does not have any objects)
// * [ ] Uses exceptions
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
/*
$Description : 
	Used to show the information in one or more ccField objects(from a ccDataDictionary object)
	on the screen. It will be displayed in a grid.
*/
/////////////////////////////////////////////////////////////////////////////
#ifndef __CCDBGRID_H__
#define __CCDBGRID_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//
//
#define GRIDCTRL_CLASSNAME    _T("CCDBGRID")	// Window class name
#define IDC_INPLACE_CONTROL   8                 // ID of inplace edit controls
#define GRID_LINE_WIDTH 1						// The width of the line between the cells.
	

// Used for Get/SetItem calls.
//
typedef struct _GV_ITEM { 
    int			row,col;     // Row and Column of item
	CDBVariant	m_bookMark;	 // Bookmark in the database for the item.
    UINT		mask;        // Mask for use in getting/setting cell data
    UINT		state;       // cell state (focus/hilighted etc)
    UINT		nFormat;     // Format of cell
    CString		szText;      // Text in cell
    int			iImage;      // index of the list view items icon
    COLORREF	crBkClr;     // Background colour (or CLR_DEFAULT)
    COLORREF	crFgClr;     // Forground colour (or CLR_DEFAULT)
    LPARAM		lParam;      // 32-bit value to associate with item 
    LOGFONT		lfFont;      // Cell font
} GV_ITEM; 

// Grid line selection
//
#define GVL_NONE                0
#define GVL_HORZ                1
#define GVL_VERT                2
#define GVL_BOTH                3

// Cell data mask
//
#define GVIF_TEXT               LVIF_TEXT
#define GVIF_IMAGE              LVIF_IMAGE
#define GVIF_PARAM              LVIF_PARAM
#define GVIF_STATE              LVIF_STATE
#define GVIF_BKCLR              (GVIF_STATE<<1)
#define GVIF_FGCLR              (GVIF_STATE<<2)
#define GVIF_FORMAT             (GVIF_STATE<<3)
#define GVIF_FONT               (GVIF_STATE<<4)

// Cell states
//
#define GVIS_FOCUSED            0x0001
#define GVIS_SELECTED           0x0002
#define GVIS_DROPHILITED        0x0004
#define GVIS_READONLY           0x0008
#define GVIS_FIXED              0x0010  // not yet used
#define GVIS_MODIFIED           0x0020

// Cell Searching options
//
#define GVNI_FOCUSED            0x0001
#define GVNI_SELECTED           0x0002
#define GVNI_DROPHILITED        0x0004
#define GVNI_READONLY           0x0008
#define GVNI_FIXED              0x0010  // not yet used
#define GVNI_MODIFIED           0x0020

#define GVNI_ABOVE              LVNI_ABOVE
#define GVNI_BELOW              LVNI_BELOW
#define GVNI_TOLEFT             LVNI_TOLEFT
#define GVNI_TORIGHT            LVNI_TORIGHT
#define GVNI_ALL                (LVNI_BELOW|LVNI_TORIGHT|LVNI_TOLEFT)
#define GVNI_AREA               (LVNI_BELOW|LVNI_TORIGHT)

// Hit test values (not yet implemented)
//
#define GVHT_DATA               0x0000
#define GVHT_TOPLEFT            0x0001
#define GVHT_COLHDR             0x0002
#define GVHT_ROWHDR             0x0004
#define GVHT_COLSIZER           0x0008
#define GVHT_ROWSIZER           0x0010
#define GVHT_LEFT               0x0020
#define GVHT_RIGHT              0x0040
#define GVHT_ABOVE              0x0080
#define GVHT_BELOW              0x0100

//
//
typedef struct tagNM_GRIDVIEW { 
    NMHDR hdr; 
    int   iRow; 
    int   iColumn; 
	_variant_t	 m_bookMark;

} NM_GRIDVIEW;

//
//
typedef struct tagGV_DISPINFO { 
    NMHDR   hdr; 
    GV_ITEM item; 
} GV_DISPINFO;

// Messages sent to the grid's parent (More will be added in future)
//
#define	GVN_BEGINDRAG           LVN_BEGINDRAG        // LVN_FIRST-9
#define	GVN_BEGINLABELEDIT      LVN_BEGINLABELEDIT   // LVN_FIRST-5
#define	GVN_BEGINRDRAG          LVN_BEGINRDRAG
#define GVN_COLUMNCLICK         LVN_COLUMNCLICK
#define	GVN_DELETEITEM          LVN_DELETEITEM
#define GVN_ENDLABELEDIT        LVN_ENDLABELEDIT     // LVN_FIRST-6
#define GVN_SELECTED            BN_CLICKED

enum E_CCGR_MR
{
	CCGR_MRERROR = 0,
	CCGR_MRLASTROW,
	CCGR_MRPAGEDOWNROW,
	CCGR_MRPAGEUPROW
};

class ccdbGrid;

// DDX_GridControl is used where a DDX_Control call is needed. In some strange
// situations the usual DDX_Control does not result in ccdbGrid::SubclassWindow
// or ccdbGrid::PreSubclassWindow being called. Using this version calls 
// ccdbGrid::SubclassWindow directly - ensuring that cell metrics are set properly
void AFX_EXT_CLASS AFXAPI DDX_GridControl(CDataExchange* pDX, int nIDC, ccdbGrid& rControl);


class AFX_EXT_CLASS ccdbGrid : public CWnd
{
// *** In classes classes.
private:
	//
	//
	class cGridColumns : public CObject
	{
	public:
		cGridColumns( CString& aName ) : m_editCtrl(NULL) { m_name = aName;};
		~cGridColumns() { /*if ( m_editCtrl != NULL) 
							m_editCtrl->DestroyWindow(); */
					  };

		//
		//
		void setName( CString& asName )	{ m_name = asName;						}
		CString getName()				{ return m_name;						}

		// The width of a column in the grid.
		//
		void setWidth( int aWidth )		{ m_width = aWidth;						}
		int  getWidth( )				{ return m_width;						}

	private:
		CString m_name;
		int		m_width;				// see setWidth

	public:
		ccInplaceSuperCtrl *m_editCtrl;
	};

	//
	//
	class cGridRow : public CObject
	{
	public:	
		_variant_t	 m_bookMark;
		CStringArray m_cellData;		
	};


// *** Construction
public:
	// Standard Constructor
	//
	ccdbGrid();
	DECLARE_DYNCREATE(ccdbGrid)

	// Connect the grid to a dialog resource.
	//
	BOOL SubclassDlgItem(UINT nID, CWnd* pParent);
		
	//
	//
	bool insertdbField(LPCSTR asHeader, ccField *accField, int aiWidth = -1 );	

	//
	//
	void fill();

private:
	// Register the window class if it has not already been registered.
	//
	BOOL RegisterWindowClass();
	

// *** Destruction
public:
	// Standard Destructor
	//
	virtual ~ccdbGrid();

// *** General Attributes
public:
	//
	//
	void setDD( ccDataDictionary * aoDD );
	ccDataDictionary * getDD() const			{ return m_dd;									}

	// The height of a row.
	//
	int getRowHeight();	

	// Set the normal size of the m_cellMargin
	// The width in pixel of one character
	//
	void initCellMargin();
	
	//
	//
	int getVisibleGridCols()				{ return m_nVisibleGridCols;			};

	// The maximum numbers of visible data cell rows in the grid.
	//
	void initVisibleGridRows( int aiGridHeight );
	int  getVisibleGridRows( )				{ return m_nVisibleGridRows;			};
	void setVisibleGridRows( int aiRows)	{ m_nVisibleGridRows = aiRows;			};

	//
	//
	int getTextWidth(CString& asText);

	//
	//
	void loadUnSelectedContactFloatMenu(int aiMenuID);

	//
	//
	void loadSelectedItemFloatMenu(int aiMenuID);
private:
	ccDataDictionary * m_dd;		// see setDD
	
	int m_unselectedItemMenuID;		// see loadUnSelectedContactFloatMenu
	int m_selectedItemMenuID;		// see loadSelectedItemFloatMenu

private:
	CTypedPtrArray<CObArray, ccField*>      m_ccFields;
	CTypedPtrArray<CObArray, cGridColumns*> m_gridColumns;
	CTypedPtrArray<CObArray, cGridRow*>     m_gridRows;				// Cache for the table data.

	// General Attributes
	CRect m_gridClipRect;	// The position and size on the screen.

	CRect m_clipRect;		// TODO Delete This use m_gridClipRect instead
	CRect m_dataRowRect;

	bool m_bDoubleBuffer;


	// Fonts
	CFont		m_gridFont;
				
	int			m_cellMargin;				// see initCellMargin()

	//Cell Attributes	
	int			m_nVisibleGridCols;			// see getVisibleGridCols
	int			m_nVisibleGridRows;			// See initVisibleGridRows
	
	CPoint		m_idCurrentCell;			// -1 invalid cell
											// -2 TopInputRow
	CPoint		m_idOldCurrentCell;
	
	CUIntArray	m_cellXPos;					//The postion in pixels where the cell are placed in the grid.
	CUIntArray	m_cellYPos;	

	CPoint		m_inputTopRowYPos;			// The Y postion and height of the top input row. The X postion and
											// size is the same as for the cells.

																								
// *** Color attributes																			
public:
	//
	//
    void     setLabelRowBkColor(COLORREF clr)   { m_labelRowBkColour = clr;						}
    COLORREF getLabelRowBkColor() const         { return m_labelRowBkColour;					} 
    void     setLabelRowTextColor(COLORREF clr) { m_labelRowTextColour = clr;					}
    COLORREF getLabelRowTextColor() const       { return m_labelRowTextColour;					} 

	//
	//
    void     setCellRowBkColor(COLORREF clr)    { m_cellRowBkColour = clr;						}
    COLORREF getCellRowBkColor() const          { return m_cellRowBkColour;						} 
    void     setCellRowTextColor(COLORREF clr)  { m_cellRowTextColour = clr;					}
    COLORREF getCellRowTextColor() const        { return m_cellRowTextColour;					} 

	//
	//
    void     setInputRowColor(COLORREF clr)     { m_inputRowBkColour = clr;						}
    COLORREF getInputRowColor() const           { return m_inputRowBkColour;					} 

	//
	//
    void     setBkColor(COLORREF clr)			{ m_bkColour = clr;								}
    COLORREF getBkColor() const					{ return m_bkColour;							} 

	//
	//
    void     setPenColor(COLORREF clr)			{ m_penColour = clr;							}
    COLORREF getPenColor() const				{ return m_penColour;							} 	

private:	
    COLORREF	m_labelRowBkColour, m_labelRowTextColour;	// see setLabelRowBkColor
	COLORREF	m_cellRowBkColour,  m_cellRowTextColour;	// see setCellRowBkColor
	COLORREF	m_inputRowBkColour;							// see setInputRowColor
	COLORREF	m_bkColour;	    							// see setBkColor
	COLORREF	m_penColour;								// see setPencolor


// DataBase Attributes
public:
	//
	//
	void enableInputTopRow(BOOL abEnable = TRUE ){ m_bAddTopRow = abEnable;						}
	BOOL isInputTopRowAllowed()					{ return m_bAddTopRow;							}	

private:
	BOOL m_bAddTopRow;						// To add records from the row under the label, like in 
										    // outlook 98 tasks gris..

// *** Drawing
protected:	
	//
	//
	virtual void OnDraw(CDC* pDC);

	//
	//
	void eraseBkgnd(CDC* pDC);

	//
	//
	void writeLabels( CDC *pDC );

	//
	//
	void drawRow(int aiRow, CDC *pDC );

	//
	//
	void drawCell(int aiCol, int aiRow, CDC *pDC );


// *** Overrides
public:
	//
	//
	virtual BOOL upRow();

	//
	//
	virtual BOOL downRow();

	//
	//
	void changeRow();
	
	//
	//
	virtual void saveRow();
	
	//
	//
	virtual void OnSelectCell(CPoint aSelectedCell);
	
	//
	//
	virtual void OnEditCell(int nCol, int nRow, UINT nChar);	
	
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(ccdbGrid)
	protected:	
	//}}AFX_VIRTUAL


// *** Implementation
public:	
	//
	//
	void firstRow();

	void moveRow( E_CCGR_MR aMoveRow );

	void lastRow();

	void pageUpRow();

	void pageDownRow();	

	//
	//
	void readRowFromTable( int aiRow );	

	LRESULT SendMessageToParent(int nCol, int nRow, int nMessage);

	void calcInplaceCtrlPos();
	
	void refillGrid();
	void refreshGrid();
	
	CString getCellText(int nCol, int nRow);
	void setCellText(int nCol, int nRow, CString &str);
	
	BOOL isValidCell(int aiCol, int aiRow);
	void calcCellPos();


	CPoint  getCellPosFromPt(CPoint aPoint);
	CRect   getCellRectFromPos(int aiCol, int aiRow);

// *** Generated message map functions
protected:
	//{{AFX_MSG(ccdbGrid)
	afx_msg void OnPaint();	
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);	
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);	
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSetFocus(CWnd* pOldWnd);		
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

	//
	//
	void OnEndInPlaceEdit(NMHDR* pNMHDR, LRESULT* pResult);		
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // __CCDBGRID_H__
