//////////////////////////////////////////////////////////////////////
// $File name   : ccdbSuperCtrl.cpp
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "ccDataBase.h"
#include "ccField.h"
#include "ccDataDictionary.h"
#include "ccdbCtrl.h"
#include "ccdbSuperCtrl.h"
#include "ccddCtrlContainer.h"
#include "resource.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void AFXAPI DDX_ccdbSuperCtrl(CDataExchange* pDX, int nIDC, ccdbSuperCtrl& rControl)
// **********************************************************
// *** 
// *** The DDX_ccdbSuperCtrl function manages the transfer of data 
// *** between a subclassed control in a dialog box, form view, or 
// *** control view object and a CWnd data member of the dialog box, 
// *** form view, or control view object. 
// *** 
// *** Modifies:
// *** None
// **********************************************************
{
    if (rControl.GetSafeHwnd() == NULL)    // not subclassed yet
    {
        ASSERT(!pDX->m_bSaveAndValidate);

        HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
		
        if (!rControl.SubclassWindow(hWndCtrl))
        {
            ASSERT(FALSE);      // possibly trying to subclass twice?
            AfxThrowNotSupportedException();
        }
#ifndef _AFX_NO_OCC_SUPPORT
        else
        {
            // If the control has reparented itself (e.g., invisible control),
            // make sure that the CWnd gets properly wired to its control site.
            if (pDX->m_pDlgWnd->GetSafeHwnd() != ::GetParent(rControl.GetSafeHwnd()))
                rControl.AttachControlSite(pDX->m_pDlgWnd);
        }
#endif //!_AFX_NO_OCC_SUPPORT
		
		//Initialize the super ctrl. This is also done in the OnCreate, but OnCreate won't be called
		//when ctrl is created with dialog template.
		rControl.initSuperCtrl();

    }
}

BEGIN_MESSAGE_MAP(ccdbSuperCtrl, CWnd)
	//{{AFX_MSG_MAP(ccdbSuperCtrl)
	ON_WM_CREATE()
	ON_WM_MOVE()
	ON_WM_SIZE()
	ON_WM_SHOWWINDOW()	
	ON_WM_KEYDOWN()	
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(ID_CCDBSUPERCTRL_FIELD_CHANGER, OnFieldChangerButton)
	ON_BN_CLICKED(ID_CCDBSUPERCTRL_COMBO_BUTTON, OnComboButtonButton)
	
	ON_EN_CHANGE(ID_CCDBSUPERCTRL_EDIT_BOX,    OnChangeEdit)
	ON_EN_KILLFOCUS(ID_CCDBSUPERCTRL_EDIT_BOX, OnKillFocusEdit)
	ON_EN_SETFOCUS(ID_CCDBSUPERCTRL_EDIT_BOX,  OnSetFocusEdit)

	ON_BN_CLICKED(ID_CCDBSUPERCTRL_CHECK_BOX,   OnCheckCheckBox)
	ON_EN_KILLFOCUS(ID_CCDBSUPERCTRL_CHECK_BOX, OnKillFocusCheckBox)
	ON_EN_SETFOCUS(ID_CCDBSUPERCTRL_CHECK_BOX,  OnSetFocusCheckBox)
END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////
// ccdbSuperCtrl
/////////////////////////////////////////////////////////////////////////////

ccdbSuperCtrl::ccdbSuperCtrl()
:m_caption(NULL), m_ccdbEdit(NULL), m_ccdbCheckBox(NULL), m_filedChanger(NULL), m_comboButton(NULL),
m_nFields(0), m_crntField(-1), m_bEditing(FALSE), m_defaultField(0)
{
	// Make sure the class is registerd in windows before it used on 
	// a dialog resource.
	RegisterWindowClass();	

	// The default properties for the m_Caption ctrl
	enableCaption( );
	setCaptionOffset( -60,0);
	setCaptionAlignement( SS_LEFT );
}

//

BOOL ccdbSuperCtrl::SubclassDlgItem(UINT nID, CWnd* pParent, ccddCtrlContainer * aOwnerContainer)
{
	BOOL bRet = CWnd::SubclassDlgItem( nID, pParent );
	initSuperCtrl();

	setOwnerContainer( aOwnerContainer );
	if ( getOwnerContainer()->getFirstCtrl() == NULL )
		getOwnerContainer()->setFirstCtrl( this ); 

	return bRet;
}

//
void ccdbSuperCtrl::initSuperCtrl()
{  
	// creates the ctrl for that current field
	setCurrentField( getDefaultField() );

	createFieldChangerObject();

	createComboButtonObject();

	if (( m_filedChanger != NULL) && (m_nFields > 1)) 
		m_filedChanger->ShowWindow(SW_SHOW);	 // ** TODO Check this one, should it really be here
}

//

bool ccdbSuperCtrl::adddbField(ccField *aodbField)
{
	NEEDVALUE( aodbField );

	//Attach the ccField to the ctrl
	m_odbField.Add( aodbField );
	aodbField->addAttachedCtrl( this );
	m_nFields++;
	
	return true;
}

//

BOOL ccdbSuperCtrl::RegisterWindowClass()
{
    WNDCLASS wndcls;
    HINSTANCE hInst = AfxGetInstanceHandle();
   // HINSTANCE hInst = AfxGetResourceHandle(); 

    if (!(::GetClassInfo(hInst, CCDBSUPERCTRL_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class
        wndcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
        wndcls.hbrBackground    = (HBRUSH) (COLOR_3DFACE + 1);
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = CCDBSUPERCTRL_CLASSNAME;		

        if (!AfxRegisterClass(&wndcls)) {
            AfxThrowResourceException();
            return FALSE;
        }
    }

    return TRUE;

}

/////////////////////////////////////////////////////////////////////////////
// *** Destruction
/////////////////////////////////////////////////////////////////////////////

ccdbSuperCtrl::~ccdbSuperCtrl()
{
	//Delete all ctrls.
	if ( m_ccdbEdit != NULL )
	{
		delete m_ccdbEdit;
		m_ccdbEdit = NULL;
	}

	if ( m_ccdbCheckBox != NULL )
	{
		delete m_ccdbCheckBox;
		m_ccdbCheckBox = NULL;
	}

	if ( m_caption != NULL )
	{
		delete m_caption;
		m_caption = NULL;
	}

	if ( m_comboButton != NULL)
	{
		delete m_comboButton;
		m_comboButton = NULL;
	}

	if (m_filedChanger != NULL)
	{
		delete m_filedChanger;
		m_filedChanger = NULL;
	}

	
	// Remove this ctrl from the connected ccFields
	while ( m_odbField.GetSize() )
	{
		ccField * lField = (ccField*)m_odbField[0];
		lField->removeAttachedCtrl( this );		
		m_odbField.RemoveAt(0);
	}
}

/////////////////////////////////////////////////////////////////////////////
// *** Properties
/////////////////////////////////////////////////////////////////////////////

DWORD ccdbSuperCtrl::m_dwEditStyle;								// The style for all the edit boxes..
DWORD ccdbSuperCtrl::m_dwexEditStyle = WS_EX_STATICEDGE;		// The style for all the edit boxes..	

//

ccField * ccdbSuperCtrl::getdbField(int anField )
{
	ASSERT( isCtrlValid( m_crntField ) );	
	ASSERT( m_odbField[anField] != NULL );
	
	return (ccField *)m_odbField[anField];
}

//

bool ccdbSuperCtrl::setCurrentField(int anField)
{

	if ( anField != m_crntField)
	{

		//Delete all the old ctrls.
		if ( m_ccdbEdit != NULL )
		{
			delete m_ccdbEdit;
			m_ccdbEdit = NULL;
		}

		if ( m_ccdbCheckBox != NULL )
		{
			delete m_ccdbCheckBox;
			m_ccdbCheckBox = NULL;
		}

		if ( m_caption != NULL )
		{
			delete m_caption;
			m_caption = NULL;
		}

		if ( m_comboButton != NULL)
		{
			delete m_comboButton;
			m_comboButton = NULL;
		}

		//Create the new ctrl
		m_crntField = anField;
		ccField * lCrntField = getdbField( anField );

		switch ( lCrntField->getCtrlType() )
		{
		case CCSC_PICTURE:
			// TODO Create an ole controller something.
			// TODO Create a Picture input ctrl.
			break;
		case CCSC_COMBO:
		case CCSC_EDIT:
			createccdbEditObject();
			createComboButtonObject();
			break;
		case CCSC_CHECKBOX:
			createccdbCheckBoxObject();
			break;
		default:
			MessageBox("Invalid ctrl type","Error");
			return false;
			break;
		}

		//Update the caption text
		if ( isCaptionEnabled()  )
			createCaptionObject();	

		//Update the data
		refresh();
	}
	return true;
}


//

DWORD ccdbSuperCtrl::getEditexStyle()
{
	ASSERT( isCtrlValid( m_crntField ) );

	DWORD ldwexStyle =  m_dwexEditStyle | getdbField( m_crntField )->getexStyle();
	return ldwexStyle;
}

//

DWORD ccdbSuperCtrl::getEditStyle()
{
	ASSERT( isCtrlValid( m_crntField ) );
	
	DWORD ldwStyle =  m_dwEditStyle | getdbField( m_crntField )->getStyle() | 
		WS_CHILD | WS_VISIBLE ;
	
	return ldwStyle;
}

//

CCE_TYPE ccdbSuperCtrl::getEditMaskType()
{	
	ASSERT( isCtrlValid( m_crntField ) );

	CCE_TYPE lMasktype = getdbField( m_crntField )->getMaskType();

	// Take the standard mask for the current database table field type.
	if ( lMasktype == CCE_DB_DEFAULT )
	{			
		switch ( getdbField( m_crntField )->getFieldType() )
		{
		case ADODB::adEmpty :		
			ccErrEM( IDS_UNDEFINED_TYPE, NULL, NULL, MB_OK );

		case ADODB::adDate:	// The date/time format from access odbc.		
			lMasktype = CCE_DATETIME;				
			break;

		case ADODB::adLongVarWChar :		
	    case ADODB::adVarWChar :		//Access type 'Text', SQL-Type 'VarChar'    
			lMasktype = CCE_TEXT;
			break;

		case ADODB::adInteger :		
			lMasktype = CCE_NUMERIC;				
			break;

		case ADODB::adVarBinary :		// Picture

			break;

		case ADODB::adBoolean :		
	    case ADODB::adBinary :		
		case ADODB::adTinyInt :		
		case ADODB::adVarChar :		
		case ADODB::adSmallInt :		    
		case ADODB::adBigInt :		
		case ADODB::adUnsignedTinyInt :		
		case ADODB::adUnsignedSmallInt :		
		case ADODB::adUnsignedInt :		
		case ADODB::adUnsignedBigInt :		
		case ADODB::adSingle :		
		case ADODB::adDouble :		
		case ADODB::adCurrency :		
		case ADODB::adDecimal :		
		case ADODB::adNumeric :	
		case ADODB::adError :		
		case ADODB::adUserDefined :		
		case ADODB::adVariant :		
		case ADODB::adIDispatch :		
		case ADODB::adIUnknown :		
		case ADODB::adGUID :				
		case ADODB::adDBDate :		
		case ADODB::adDBTime :		
		case ADODB::adDBTimeStamp :		
		case ADODB::adBSTR :		
		case ADODB::adChar :		
		case ADODB::adLongVarChar :		
		case ADODB::adWChar :						
		case ADODB::adLongVarBinary :		
		case ADODB::adChapter :		
		case ADODB::adFileTime :		
		case ADODB::adPropVariant :		
		case ADODB::adVarNumeric :		
		//case ADODB::adArray :		

			ccErr( IDS_FIELD_TYPE_NOT_SUPORTED );
			lMasktype = CCE_TEXT;			// Maybe the text field can handle the not supported data type
			break;
			
		default:
			ccErr( IDS_INVALID_FIELD_TYPE );	// Maybe the text field can handle the invalid data type
			lMasktype = CCE_TEXT;
			break;
		 }			 		
	}
	return lMasktype;
}

/////////////////////////////////////////////////////////////////////////////
// *** Ctrl objects.
/////////////////////////////////////////////////////////////////////////////

void ccdbSuperCtrl::createccdbEditObject()
{
	if (m_ccdbEdit == NULL)
	{	
		m_ccdbEdit = new ccEdit;

		DWORD ldwStyle = getEditStyle();
		
		m_ccdbEdit->Create( ldwStyle, getInputCtrlRect(), this, ID_CCDBSUPERCTRL_EDIT_BOX );

		//Set the font for the edit ctrl
		CFont guiFont;
		guiFont.Attach(GetStockObject( DEFAULT_GUI_FONT ));
		m_ccdbEdit->SetFont( &guiFont ); 

		m_ccdbEdit->ModifyStyleEx(0,getEditexStyle() ,SWP_DRAWFRAME );

		switch ( getEditMaskType()) 
		{
		case CCE_DATETIME:
			m_ccdbEdit->setDateTimeMask();
			break;
		case CCE_DATE:
			m_ccdbEdit->setDateMask();
			break;
		case CCE_TIME:
			m_ccdbEdit->setTimeMask();
			break;
		case CCE_NUMERIC:
			m_ccdbEdit->setNumericMask(getdbField( m_crntField )->getFieldPrecision() );
			break;
		case CCE_TEXT:
			m_ccdbEdit->setTextMask(getdbField( m_crntField )->getFieldPrecision() );
			break;
		case CCE_CUSTOM:
			m_ccdbEdit->setCustomMask( getdbField( m_crntField )->getCustomMask(),
									   getdbField( m_crntField )->getPlaceHolder() );
			break;

		default:		
			break;

		}
		// Never set focus to this ctrl when pressing tab and ctrl-tab.
		if ( ldwStyle & WS_DISABLED )
			ModifyStyle( WS_TABSTOP,0,0);
		else
			ModifyStyle( 0,WS_TABSTOP ,0);		
	}
}

//

void ccdbSuperCtrl::createccdbCheckBoxObject()
{
	if (m_ccdbCheckBox ==NULL)
	{	
		m_ccdbCheckBox = new ccdbSuperCtrl_CheckBox;
				
		// Set the flags
		DWORD ldwStyle =  WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX | getdbField( m_crntField )->getStyle();

		m_ccdbCheckBox->Create( "", ldwStyle, getInputCtrlRect(), this, ID_CCDBSUPERCTRL_CHECK_BOX );

		// Never set focus to this ctrl when pressing tab and ctrl-tab.
		if ( ldwStyle & WS_DISABLED)
			ModifyStyle( WS_TABSTOP,0,SWP_DRAWFRAME );
		else
			ModifyStyle( 0,WS_TABSTOP ,SWP_DRAWFRAME );			
	}
}

//

CRect ccdbSuperCtrl::getInputCtrlRect()
{
	CRect lClientRect;
	GetClientRect(lClientRect);

	// Is the fieldChanger Object is created.
	// The input ctrls should be created at the right side of the
	// fieldChanger button.

	if (m_nFields > 1)
		lClientRect.left += FIELD_CHANGER_OBJECT_WIDTH + CONTROL_SEPARATOR;

	if (getdbField( m_crntField )->hasPopup() )
		lClientRect.right -= FIELD_CHANGER_OBJECT_WIDTH + CONTROL_SEPARATOR;	

	return lClientRect;
}

//

void ccdbSuperCtrl::createCaptionObject()
{
	if (m_caption == NULL)
	{		
		m_caption = new CStatic;		

		//Left or right aligned
		DWORD ldwStyle = getCaptionAlignement() | WS_CHILD | WS_VISIBLE ;
		
		m_caption->Create( getdbField( m_crntField )->getLongCaption(), 
			ldwStyle, getCaptionRect(), GetParent() );

		//Set the font for the caption
		CFont guiFont;
		guiFont.Attach(GetStockObject( DEFAULT_GUI_FONT ));
		m_caption->SetFont(&guiFont);
	}
}

//

CRect ccdbSuperCtrl::getCaptionRect()
{
	// Get the font size.
	CWindowDC dc(this);	
	CFont guiFont;
	guiFont.Attach(GetStockObject( DEFAULT_GUI_FONT ));
	CFont *pFontDC = dc.SelectObject( &guiFont );	
	CSize lFontSize = dc.GetTextExtent( getdbField( m_crntField )->getLongCaption() );
	dc.SelectObject( pFontDC );


	// Get the placement of the super ctrl
	WINDOWPLACEMENT  lWndPlaceMent;
	lWndPlaceMent.length = sizeof( WINDOWPLACEMENT );
	GetWindowPlacement( &lWndPlaceMent );


	//Calc the position and size	
	CRect lClientRect;

	lClientRect        = lWndPlaceMent.rcNormalPosition;			
	lClientRect       += getCaptionOffset();
	lClientRect.right  = lClientRect.left + lFontSize.cx;
	lClientRect.bottom = lClientRect.top  + lFontSize.cy;

	//So the marker in OnSetFocusCheckBox fit around the text
	lClientRect.InflateRect(0,0,0,2);

	return lClientRect;
}

//

void ccdbSuperCtrl::createFieldChangerObject()
{	
	if (m_filedChanger ==NULL)
	{	
		m_filedChanger = new ccMenuButton;

		// Set the flags, No WS_VISIBLE flag, don't show before needed.
		DWORD ldwStyle =  WS_CHILD | BS_OWNERDRAW | BS_PUSHBUTTON | BS_CENTER;

		m_filedChanger->Create( "", ldwStyle, getFieldChangerRect(), this, ID_CCDBSUPERCTRL_FIELD_CHANGER );				
	}
}

//

CRect ccdbSuperCtrl::getFieldChangerRect()
{
	CRect lClientRect;
	GetClientRect(lClientRect);

	lClientRect.right  = lClientRect.left + FIELD_CHANGER_OBJECT_WIDTH;
	lClientRect.bottom = lClientRect.top  + FIELD_CHANGER_OBJECT_HEIGHT;
	return lClientRect;
}

//

void ccdbSuperCtrl::createComboButtonObject()
{	
	if ( getdbField( m_crntField )->hasPopup() )
	{
		if (m_comboButton ==NULL)
		{	
			m_comboButton = new CButtonST;

			// Set the flags, 
			DWORD ldwStyle =  WS_CHILD | WS_VISIBLE;

			 m_comboButton->Create( "", ldwStyle, getComboButtonRect(), this, ID_CCDBSUPERCTRL_COMBO_BUTTON );				
			 m_comboButton->SetIcon( IDI_ARROW_DOWN );
			 //m_comboButton->SetFlat(FALSE);
			 //m_comboButton->SetInactiveFgColor(RGB(255,255,0));
			 //m_comboButton->SetInactiveBgColor(RGB(0,128,128));
		}
	}
}

//

CRect ccdbSuperCtrl::getComboButtonRect()
{
	CRect lClientRect;
	GetClientRect(lClientRect);

	lClientRect.left  = lClientRect.right - FIELD_CHANGER_OBJECT_WIDTH;
	//lientRect.bottom = lClientRect.top  + FIELD_CHANGER_OBJECT_HEIGHT;
	
	return lClientRect;
}

/////////////////////////////////////////////////////////////////////////////
// ** General
/////////////////////////////////////////////////////////////////////////////

bool ccdbSuperCtrl::refresh()
{		
	if ( isCtrlValid( m_crntField ) )
	{
		ccField * lCrntField = getdbField( m_crntField );
		switch ( lCrntField->getCtrlType() )
		{
		case CCSC_COMBO:		
		case CCSC_EDIT:
			{
				CString lsFormValue;
				m_ccdbEdit->GetWindowText( lsFormValue );
				//Change the text in the ctrl only if the text is changed.
				//It's not changed in the current edit box.
				if (lsFormValue != lCrntField->getValue( true ) )
					m_ccdbEdit->SetWindowText( lCrntField->getValue() );
			}		
			break;
		case CCSC_CHECKBOX:
			{
				int liCheckState;
				liCheckState = m_ccdbCheckBox->GetCheck();
				//Change the check in the ctrl only if the check is changed.
				//It's not changed in the current checkbox.
				if (liCheckState != lCrntField->getValue() )				
					m_ccdbCheckBox->SetCheck( *lCrntField );
			}		
			break;
		case CCSC_PICTURE:
			// TODO Picture support
			break;
		default:
			MessageBox("Invalid ctrl type","Error");
			return false;
			break;
		}	
	}

	return CCSUCCEDED;
}

/////////////////////////////////////////////////////////////////////////////
// *** Controlling the datadictionary.
/////////////////////////////////////////////////////////////////////////////

void ccdbSuperCtrl::OnRecordPopup()
{		
	getdbField(m_crntField)->OnPopup(this);		
}

//

void ccdbSuperCtrl::OnRecordRefresh()
{	
	getOwnerContainer()->getDDServer()->requery();
	getOwnerContainer()->getDDServer()->refreshRelations();
}

//

void ccdbSuperCtrl::OnRecordClear() 
{			
	if ( getOwnerContainer()->getDDServer()->clearRecord() == CCSUCCEDED)
		getOwnerContainer()->getFirstCtrl()->SetFocus();
}

//

void ccdbSuperCtrl::OnRecordClearAll() 
{		
	MessageBox("not implemented");
	//	getOwnerContainer()->getDDServer()->saveRecord();
}

//

void ccdbSuperCtrl::OnRecordDelete() 
{			
	if ( getOwnerContainer()->getDDServer()->deleteRecord() == CCSUCCEDED)
		getOwnerContainer()->getFirstCtrl()->SetFocus();
}

//

void ccdbSuperCtrl::OnRecordNext() 
{	
	m_bEditing = false;
	if ( getdbField(m_crntField)->isColumnSorted() )
		getdbField(m_crntField)->getDD()->moveNext();
	else
		ccErrEM( IDS_FIELD_NOT_INDEXED, NULL, NULL, MB_OK );
	m_bEditing = true;
}

//

void ccdbSuperCtrl::OnRecordPrev() 
{	
	m_bEditing = false;
	if ( getdbField(m_crntField)->isColumnSorted() )
		getdbField(m_crntField)->getDD()->movePrev();	
	else
		ccErrEM( IDS_FIELD_NOT_INDEXED, NULL, NULL, MB_OK );
	m_bEditing = true;
}

//

void ccdbSuperCtrl::OnRecordFirst() 
{	
	m_bEditing = false;
	if ( getdbField(m_crntField)->isColumnSorted() )
		getdbField(m_crntField)->getDD()->moveFirst();
	else
		ccErrEM( IDS_FIELD_NOT_INDEXED, NULL, NULL, MB_OK );
	m_bEditing = true;
}

//

void ccdbSuperCtrl::OnRecordLast() 
{	
	m_bEditing = false;
	if ( getdbField(m_crntField)->isColumnSorted() )
		getdbField(m_crntField)->getDD()->moveLast();	
	else
		ccErrEM( IDS_FIELD_NOT_INDEXED, NULL, NULL, MB_OK );
	m_bEditing = true;
}

//

void ccdbSuperCtrl::OnRecordSave() 
{	
	if ( getOwnerContainer()->getDDServer()->saveRecord() == CCSUCCEDED)
		getOwnerContainer()->getFirstCtrl()->SetFocus();
}

//

void ccdbSuperCtrl::OnRecordFind() 
{	
	if ( getdbField(m_crntField)->isColumnSorted() )
		getdbField(m_crntField)->getDD()->findRecord( getdbField(m_crntField) );	
	else
		ccErrEM( IDS_FIELD_NOT_INDEXED, NULL, NULL, MB_OK );
}


/////////////////////////////////////////////////////////////////////////////
// *** Overrides
/////////////////////////////////////////////////////////////////////////////

BOOL ccdbSuperCtrl::Create(const RECT& rect, CWnd* pParentWnd, UINT nID, DWORD dwStyle)
// **********************************************************
// ***
// *** Overloaded so it will be easier to create the 
// *** object. ( The user of the class don't have to know
// *** anything about the CCDBSUPERCTRL_CLASSNAME)
// ***
// *** Modifies:
// ***		None
// **********************************************************
{
    ASSERT(pParentWnd->GetSafeHwnd());
		
    if (!CWnd::Create(CCDBSUPERCTRL_CLASSNAME, NULL, dwStyle, rect, pParentWnd, nID)) 
        return FALSE;

	return TRUE;
}

//

BOOL ccdbSuperCtrl::PreTranslateMessage(MSG* pMsg) 
{
	// Trap the key down messages from the edit and checkbox ctrl, and send them
	// to this ctrl instead
	if ( pMsg->message == WM_KEYDOWN)
	{
		int nVK  = pMsg->wParam; // Virtual Key

		switch (nVK)
		{
		case VK_F2:
		case VK_F3:
		case VK_F4:
		case VK_F5:
		case VK_F6:
		case VK_F7:
		case VK_F8:
		case VK_F9:
		case VK_TAB:

			// Is the crnt control the edit box, and is the keydown message sent to that ctrl, 		
			// Send the keydown message to the super ctrl instead
			if ( m_ccdbEdit != NULL)
				if ( m_ccdbEdit->m_hWnd == pMsg->hwnd)
				{					
					SendMessage( pMsg->message, pMsg->wParam,pMsg->lParam);				
					return TRUE;
				}

			// Is the crnt control the checkbox, and is the keydown message sent to that ctrl, 
			// Send the keydown message to the super ctrl instead
			if ( m_ccdbCheckBox != NULL)
				if ( m_ccdbCheckBox->m_hWnd == pMsg->hwnd)
				{
					SendMessage( pMsg->message, pMsg->wParam,pMsg->lParam);				
					return TRUE;
				}
		}
	}

	return CWnd::PreTranslateMessage(pMsg);
}

void ccdbSuperCtrl::PreSubclassWindow() 
{
	CWnd::PreSubclassWindow();
	
}

//

void ccdbSuperCtrl::endEdit()
{
	
}

//

void ccdbSuperCtrl::GetWindowText( CString& rString ) 
{
	// Show the edit Ctrl
	if (m_ccdbEdit != NULL)
	{				
		m_ccdbEdit->GetWindowText(rString);
	}

	// Show the CheckBox ctrl
	if (m_ccdbCheckBox != NULL)
	{
		rString.Format("%u", m_ccdbCheckBox->GetCheck() );   			
	}	
}

//

int  ccdbSuperCtrl::GetWindowText( LPTSTR lpszStringBuf, int nMaxCount )
{
	NEEDVALUE( nMaxCount );
	CString lsString;
	GetWindowText( lsString );
		
	int liStringLength = lsString.GetLength();

	// If string length is larger then allowed
	// truncate it to allowed size.
	if ( liStringLength > nMaxCount )
	{
		strcpy( lpszStringBuf, lsString.Left( nMaxCount ) );
		liStringLength = nMaxCount;
	}
	else 
		strcpy( lpszStringBuf, lsString );

	return liStringLength;
}

/////////////////////////////////////////////////////////////////////////////
// *** Generated message map functions
/////////////////////////////////////////////////////////////////////////////

int ccdbSuperCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	// This is also done in the DDX_ccdbSuperCtrl function. And will be
	// called there and not here if the object is created throw
	// a dialog resource.
	initSuperCtrl();
	
	return 0;
}

//

void ccdbSuperCtrl::OnMove(int x, int y) 
{
	CWnd::OnMove(x, y);

	// Move the caption ctrl
	if ( m_caption != NULL )
	{
		CRect lCapRect = getCaptionRect();
		m_caption->SetWindowPos(NULL,lCapRect.left,lCapRect.top,0,0, SWP_NOSIZE | SWP_NOZORDER);						
	}
	
	// Move the filedChangerCtrl
	if (( m_filedChanger != NULL) && (m_nFields > 1))
	{
		CRect lFieldRect = getFieldChangerRect();
		m_filedChanger->SetWindowPos(NULL,lFieldRect.left,lFieldRect.top,0,0, SWP_NOSIZE | SWP_NOZORDER);						
	}

	// Move the comboButtonCtrl
	if ( m_comboButton != NULL)
	{
		CRect lComboRect = getComboButtonRect();
		m_comboButton->SetWindowPos(NULL,lComboRect.left,lComboRect.top,0,0, SWP_NOSIZE | SWP_NOZORDER);						
		m_comboButton->Invalidate();			//It won't be redrawn in the ccdGrid without this.
	}
	
	// Move the edit Ctrl
	if (m_ccdbEdit != NULL)
	{		
		CRect lInputCtrlRect = getInputCtrlRect();
		m_ccdbEdit->SetWindowPos(NULL,lInputCtrlRect.left,lInputCtrlRect.top,0,0, SWP_NOSIZE | SWP_NOZORDER);								
	}

	// Move the CheckBox ctrl
	if (m_ccdbCheckBox != NULL)
	{
		CRect lInputCtrlRect = getInputCtrlRect();
		m_ccdbCheckBox->SetWindowPos(NULL,lInputCtrlRect.left,lInputCtrlRect.top,0,0, SWP_NOSIZE | SWP_NOZORDER);						
	}	
}

//

void ccdbSuperCtrl::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);

	// Resize the caption ctrl
	if ( m_caption != NULL )
	{
		CRect lCapRect = getCaptionRect();
		m_caption->SetWindowPos(NULL,0,0,lCapRect.Width(),lCapRect.Height(), SWP_NOMOVE | SWP_NOZORDER);					
	}
	
	// Resize the filedChangerCtrl
	if (( m_filedChanger != NULL) && (m_nFields > 1))
	{
		CRect lFieldRect = getFieldChangerRect();
		m_filedChanger->SetWindowPos(NULL,0,0,lFieldRect.Width(),lFieldRect.Height(), SWP_NOMOVE | SWP_NOZORDER);					
	}

	// Resize the comboButtonCtrl
	if ( m_comboButton != NULL)
	{
		CRect lComboRect = getComboButtonRect();
		m_comboButton->SetWindowPos(NULL,0,0,lComboRect.Width(),lComboRect.Height(), SWP_NOMOVE | SWP_NOZORDER);					
	}

	// Resize the edit Ctrl
	if (m_ccdbEdit != NULL)
	{		
		CRect lInputCtrlRect = getInputCtrlRect();
		m_ccdbEdit->SetWindowPos(NULL,0,0,lInputCtrlRect.Width(),lInputCtrlRect.Height(), SWP_NOMOVE | SWP_NOZORDER);					
	}

	// Resize the CheckBox ctrl
	if (m_ccdbCheckBox != NULL)
	{
		CRect lInputCtrlRect = getInputCtrlRect();
		m_ccdbCheckBox->SetWindowPos(NULL,0,0,lInputCtrlRect.Width(),lInputCtrlRect.Height(), SWP_NOMOVE | SWP_NOZORDER);					
	}		
}

//

void ccdbSuperCtrl::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CWnd::OnShowWindow(bShow, nStatus);

	// Show the caption ctrl
	if ( m_caption != NULL )
	{
		CRect lCapRect = getCaptionRect();
		m_caption->ShowWindow( bShow ? SW_SHOW : SW_HIDE );
	}
	
	// Show the filedChangerCtrl
	if (( m_filedChanger != NULL) && (m_nFields > 1))
	{
		CRect lFieldRect = getFieldChangerRect();
		m_filedChanger->ShowWindow( bShow ? SW_SHOW : SW_HIDE );
	}

	// Show the comboButtonCtrl
	if ( m_comboButton != NULL)
	{
		CRect lComboRect = getComboButtonRect();
		m_comboButton->ShowWindow( bShow ? SW_SHOW : SW_HIDE );
	}

	// Show the edit Ctrl
	if (m_ccdbEdit != NULL)
	{		
		CRect lInputCtrlRect = getInputCtrlRect();
		m_ccdbEdit->ShowWindow( bShow ? SW_SHOW : SW_HIDE );
	}

	// Show the CheckBox ctrl
	if (m_ccdbCheckBox != NULL)
	{
		CRect lInputCtrlRect = getInputCtrlRect();
		m_ccdbCheckBox->ShowWindow( bShow ? SW_SHOW : SW_HIDE );
	}	
}

//

void ccdbSuperCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
// **********************************************************
// ***
// *** Will also recevie ON_KEY messages from the edit and checkbox
// *** control throu the PreTranslateMessage function.
// ***
// *** Modifies:
// ***		None
// **********************************************************
{
	switch ( nChar )  
	{
	case VK_F2:
		OnRecordSave();
		break;

	case VK_F3:
		OnRecordDelete();
		break;

	case VK_F4:
		OnRecordPopup();		
		break;

	case VK_F5:
		OnRecordRefresh();		
		break;

	case VK_F6:
		if ( isCTRLPressed() )
			OnRecordClearAll();
		else
			OnRecordClear();
		break;

	case VK_F7:
		if ( isCTRLPressed() )
			OnRecordFirst();
		else
			OnRecordPrev();
		break;

	case VK_F8:
		if ( isCTRLPressed() )
			OnRecordLast();
		else
			OnRecordNext();
		break;
	case VK_F9:
		OnRecordFind();
		break;

	case VK_TAB:			
		if ( getdbField(m_crntField)->OnValidate(this) == CCFAILED )
		{			
			// If the validation failed set the focus to this ctrl again. 
			// The KillFocus won't be canceled. 			
			return;
		}
		
		//Move To next or prev ctrl
		CWnd * lNextCtrl;
		if ( isSHIFTPressed() )
			lNextCtrl = GetParent()->GetNextDlgTabItem(this, TRUE);
		else
			lNextCtrl = GetParent()->GetNextDlgTabItem(this, FALSE);

		lNextCtrl->SetFocus();
		break;
	};	
	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

//

void ccdbSuperCtrl::OnSetFocus(CWnd* pOldWnd) 
// **********************************************************
// ***
// *** If the focus is set to the super ctrl, forward it to 
// *** the current editable ctrl.
// ***
// *** Modifies:
// ***		None
// **********************************************************
{
	CWnd::OnSetFocus(pOldWnd);

	if ( isCtrlValid( m_crntField ))
	{
		ccField * lCrntField = getdbField( m_crntField );
		if ( lCrntField != NULL)
		{
			switch ( lCrntField->getCtrlType() )
			{
			case CCSC_COMBO:		
			case CCSC_EDIT:		
				if (m_ccdbEdit != NULL )
					m_ccdbEdit->SetFocus();
				break;
			case CCSC_CHECKBOX:
				if (m_ccdbCheckBox != NULL )
				{
					m_ccdbCheckBox->SetFocus();
				}
				break;

			case CCSC_PICTURE:
				// TODO Picture support
				break;

			default:
				MessageBox("Invalid ctrl type","Error");			
				break;
			}	
		}

		// Tell the database which ctrl who is the crnt ctrl.
		// Used by dbToolBar.
		lCrntField->__getDB()->setCrntFocus( this );
	}
}

//

void ccdbSuperCtrl::OnKillFocus(CWnd* pNewWnd) 
{
	CWnd::OnKillFocus(pNewWnd);
	
	ccField * lCrntField = getdbField( m_crntField );
	lCrntField->__getDB()->setCrntFocus( NULL );
	
}

//

void ccdbSuperCtrl::OnChangeEdit()
{
	if ( m_ccdbEdit != NULL)
	{
		// Should only be executed if the focus is on this ctrl.
		// Or else it will be some kind of recurisen. 
		// The setValue in the ccField will call the OnChangeEdit
		// Who will call setValue etc...
		if (m_bEditing == TRUE)
		{
			CString lsFormValue;
			m_ccdbEdit->GetWindowText( lsFormValue );
			getdbField(m_crntField)->setValue( lsFormValue );

			//If failed reset to the old value.
			if ( getdbField(m_crntField)->OnChange(this) == CCFAILED )
			{
				getdbField(m_crntField)->setValue( lsFormValue );				
				return;
			}		
		};
	}
}

//

void ccdbSuperCtrl::OnKillFocusEdit()
{
	if ( m_ccdbEdit != NULL)
	{
		// Set the dataDictionary field to the same value as the ctrl.
		CString lsFormValue;
		m_ccdbEdit->GetWindowText( lsFormValue );
		getdbField(m_crntField)->setValue( lsFormValue );	

		//Execute the DD kill focus function
		if ( getdbField(m_crntField)->OnKillFocus(this) == CCFAILED )
		{
			// Set the focus to the ctrl again. The KillFocus will be canceled.
			// The OnSetFocus will be executed again. But shouldn't so it will be 
			// Canceled in the OnSetFocus function.
			m_ccdbEdit->SetFocus();
			return;
		}
		endEdit();
	}
	
	m_bEditing = FALSE;		
}

//

void ccdbSuperCtrl::OnSetFocusEdit()
{
	// Tell the database which ctrl who is the crnt ctrl.
	// Used by dbToolBar.
	ccField * lCrntField = getdbField( m_crntField );
	if (lCrntField) lCrntField->__getDB()->setCrntFocus( this );


	// Shouldn't be executed if it has been called from the OnKillFocusEdit
	if ( m_bEditing == FALSE)
	{
		m_bEditing = TRUE;	

		getdbField(m_crntField)->OnSetFocus(this);
	}
}

//

void ccdbSuperCtrl::OnCheckCheckBox()
{
	if ( m_ccdbCheckBox != NULL)
	{
		// Should only be executed if the focus is on this ctrl.
		// Or else it will be some kind of recurisen. 
		// The setValue in the ccField will call the OnChangeEdit
		// Who will call setValue etc...
		if (m_bEditing == TRUE)
		{
			int liCheckState;
			liCheckState = m_ccdbCheckBox->GetCheck();
			getdbField(m_crntField)->setValue( liCheckState );

			//If failed reset to the old value.
			if ( getdbField(m_crntField)->OnChange(this) == CCFAILED )
			{
				getdbField(m_crntField)->setValue( liCheckState );
				return;
			}		
		}		
	}
}

//

void ccdbSuperCtrl::OnSetFocusCheckBox()
{
	// Tell the database which ctrl who is the crnt ctrl.
	// Used by dbToolBar.
	ccField * lCrntField = getdbField( m_crntField );
	if (lCrntField) lCrntField->__getDB()->setCrntFocus( this );

	// Shouldn't be executed if it has been called from the OnKillFocusCheckBox
	if ( m_bEditing == FALSE)
	{
		m_bEditing = TRUE;	
		getdbField(m_crntField)->OnSetFocus(this);

		//Draw a marker around the caption so the user knows where the current focus is.
		if (m_caption != NULL)
		{
			CDC * dc = m_caption->GetDC();
			CRect lClientRect;
			m_caption->GetClientRect(lClientRect);

			lClientRect.DeflateRect(0,0,1,1);
			lClientRect.top=lClientRect.bottom-1;

			dc->Draw3dRect(lClientRect,GetSysColor( COLOR_HIGHLIGHT ),GetSysColor( COLOR_HIGHLIGHT ));
		}
	}
}


void ccdbSuperCtrl::OnKillFocusCheckBox()
{
	if ( m_ccdbCheckBox != NULL)
	{
		// Set the dataDictionary field to the same value as the ctrl.
		int liCheckState;
		liCheckState = m_ccdbCheckBox->GetCheck();
		getdbField(m_crntField)->setValue( liCheckState );

		//Execute the DD kill focus function
		if ( getdbField(m_crntField)->OnKillFocus(this) == CCFAILED )
		{
			// Set the focus to the ctrl again. The KillFocus will be canceled.
			// The OnSetFocus will be executed again. But shouldn't so it will be 
			// Canceled in the OnSetFocus function.
			m_ccdbCheckBox->SetFocus();
			return;
		}		
		endEdit();
	}	

	if (m_caption != NULL)
	{
		//Destory the marker around the caption object
		m_caption->Invalidate();
	}

	m_bEditing = FALSE;		
}

//

void ccdbSuperCtrl::OnComboButtonButton()
// **********************************************************
// ***
// *** Will call the attached ccField:s popup
// ***
// *** Modifies:
// ***		None
// **********************************************************
{
	getdbField(m_crntField)->OnPopup( this );
}

//

void ccdbSuperCtrl::OnFieldChangerButton()
// **********************************************************
// ***
// *** Will show a menu with the fields connected to the 
// *** superCtrl. It will also mark the menu option
// *** for the fields who have fileds which are non zero.
// ***
// *** Modifies:
// ***		m_filedChanger
// **********************************************************
{
	if (( m_filedChanger != NULL) && (m_nFields > 1))
	{
		m_filedChanger->clearMenu();

		for (int i = 1; i <= m_odbField.GetSize();i++)
		{
			ccField * lField = (ccField*)m_odbField[i-1];
			ASSERT(lField != NULL);			
			m_filedChanger->addMenuItem( lField->getLongCaption(), i);
			
			//Set a checkmark on the menuitem if any data exist in the field.
			m_filedChanger->enableMenuItemCheck(i, !getdbField( i-1 )->getValue().IsEmpty() );			
		}
		
		// Show the menu. If the user make a choice
		// change to that field.
		DWORD dwSelection = m_filedChanger->showMenu();
		if ( dwSelection != 0)
			setCurrentField( dwSelection -1);
	}
}

