// ==========================================================================
//                    Class Specification : ccdbSuperCtrl
// ==========================================================================
//////////////////////////////////////////////////////////////////////
// $File name   : /ccLib/dev/ccdbLib/ccdbSuperCtrl.h: 
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// $HomePage    : home.bip.net/CyberCow
// $ICQ         : 7983755
//
// Notice       : Copyright (c) CyberCow 1999
//                Knypplerskevagen 24
//                168 38 BROMMA
//                SWEDEN  
//                tel int + 46 (8) 26 45 95
//                
//                The copyright to the computer program(s) herein is the 
//				  property of CyberCow.The program(s) may be used and/or 
//				  copied only with the written permission of CyberCow or 
//				  in accordance with the terms and conditions stipulated 
//				  in the agreement/contract under which the program(s) 
//				  have been supplied. 
//
/////////////////////////////////////////////////////////////////////////////
///** $History$ *************************************************************
// *
// * $Date: 9/17/99 15:09 $
// * $Revision: 1 $
// * 
// * *****************  Version 1  *****************
// * User: Daniel Date: 9/17/99 15:09
// * Source-Code Traced [ ]; Reviewd [ ] 
// * Created in $/ccLib/dev/ccdbLib
// * Initial release.
// * 
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
// ** $Properties$ **********************************************************
// * [X] Abstract class (does not have any objects)
// * [ ] Uses exceptions
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
/*
$Description : 

Used to show the information in a ccField object(from a ccDataDictionary object)
on the screen. It can be displayed in a editBox or a checkbox.

Also the caption text in the ccField object will be shown
in a CStatic object somewhere around the superCtrl, depending
on the captionOffset.

To use this you drag and drop a Custom Ctrl to dialog template. Then you 
fill in the class name "CCDBSUPERCTRL" into the class edit form.

*/
/////////////////////////////////////////////////////////////////////////////
#ifndef __CCDBSUPERCTRL_H__
#define __CCDBSUPERCTRL_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ccdbCtrl.h"
#include "ccdbSuperCtrl_CheckBox.h"
#include "ccField.h"

// The class name used in the dialog template.
//
#define CCDBSUPERCTRL_CLASSNAME    _T("CCDBSUPERCTRL")  // Window class name

// Size and postion of the Field Changer object. 
//
#define FIELD_CHANGER_OBJECT_WIDTH  17
#define FIELD_CHANGER_OBJECT_HEIGHT 17
#define CONTROL_SEPARATOR 3

//
//
//#define isCtrlValid(x) ( ((x)!=-1) && ((x)<=m_nFields) )
#define isCtrlValid(x) ( ((x)>=0) && ((x)<=m_nFields) )


// Used in the DoDataExchange in the dialog class.
//
class ccdbSuperCtrl;
class ccddCtrlContainer;
__declspec(dllexport)  void AFXAPI DDX_ccdbSuperCtrl(CDataExchange* pDX, int nIDC, ccdbSuperCtrl& rControl);


class AFX_EXT_CLASS ccdbSuperCtrl : public CWnd, public ccdbCtrl
{
// *** Construction
public:
	// Default Constructor
	//
	ccdbSuperCtrl();	

	BOOL SubclassDlgItem(UINT nID, CWnd* pParent, ccddCtrlContainer * aOwnerContainer );

	// Create the input ctrls, caption ctrl and filedChanger
	// object if neccessary. 
	// Called from OnCreate when ctrl is created manually
	// Called from DDX_ccdbSuperCtrl when ctrl is created from a dialog resource
	//
	virtual void initSuperCtrl();

	// Add a ccField to the superCtrl, if more then one 
	// field is added show the fieldChanger object.
	//
	bool adddbField(ccField *aodbField);		


private:
	// Registers the ccdbSuperCtrl class for subsequent use in  
	// calls to the CreateWindow or CreateWindowEx function. 
	// That means that you use the ctrl on a dialog resource.
	//
	BOOL RegisterWindowClass();		


// *** Destruction
public:
	virtual ~ccdbSuperCtrl();

// *** Properties
public:
	// Get the connected ccFields
	//
	ccField * getdbField(int anField );

	// The field that the superCtrl will have when the program starts up.
	// This only works when more then one field is connected to the ctrl.
	//			
	void setDefaultField(int aiField)				{ ASSERT( isCtrlValid( aiField ) );
													  m_defaultField = aiField;				};
	int	 getDefaultField() const					{ return m_defaultField;				};
	
	// Handle the current active input ctrl.
	// 
	bool setCurrentField(int anField );
	int	getCrntField() const						{ return m_crntField;					};	

	// Where the m_caption ctrl ctrl should be placed (in pixel)related 
	// to the top, left corner of the super ctrl.
	// (-50,0) mean at the left side of the ctrl.
	//
	void   setCaptionOffset(int x = 0, int y = 0)	{ setCaptionOffset( CPoint(x,y) );		};
	void   setCaptionOffset(CPoint aPoint)			{ m_captionOffset = aPoint;				};
	CPoint getCaptionOffset()						{ return m_captionOffset;				};

	//How the text in the caption should be aligned.
	//SS_LEFT, SS_CENTER or SS_RIGHT
	//
	void  setCaptionAlignement(DWORD adwStyle)		{ m_captionAlignement = adwStyle;		};
	DWORD getCaptionAlignement()					{ return m_captionAlignement;			};

	// True if the captoin ctrl should be showed.
	//
	void enableCaption( bool abEnabled = TRUE )	{ m_captionEnabled = abEnabled;			};
	bool isCaptionEnabled()						{ return m_captionEnabled;				};

	// The style for all instances of the edit box.
	// Should set this before creating any superCtrl object.(aka, calling the Create function)
	//
	virtual	DWORD getEditexStyle();
			void  setEditexStyle(DWORD aexEditStyle){ m_dwexEditStyle = aexEditStyle;		};
			DWORD getEditStyle();
			void  setEditStyle(DWORD aEditStyle)    { m_dwEditStyle = aEditStyle;			};

	// Get the mask style from the edit Ctrl.
	//
	CCE_TYPE getEditMaskType();

private:
		CPtrArray	m_odbField;				// see adddbField
		int			m_nFields;				// Number of attached fields
		int			m_defaultField;			// see setDefaultField
		int			m_crntField;			// see getCtrntField


		// Attributes for the connected ccdbSuperCtrl
		CPoint	m_captionOffset;				// See setCaptionOffset	
		DWORD	m_captionAlignement;			// See enableLeftAlignedCaption
		bool	m_captionEnabled;				// See enableCaption()

protected:
static	DWORD m_dwEditStyle;			// see setEditexStyle
static	DWORD m_dwexEditStyle;			// see setEditexStyle


// *** Ctrl objects.
public:
	// Create the visible ctrl if CCSC_EDIT or CCSC_COMBO mode
	//
	void createccdbEditObject();

	// Create the visible ctrl if CCSC_CHECKBOX mode
	//
	void createccdbCheckBoxObject();

	// Calc the position and size for the ccdbEdit and ccdbCheckBox
	//
	CRect getInputCtrlRect();

	// Create the control that displays the caption text
	//
	void createCaptionObject();

	// Calc the position and size for the caption ctrl.
	//
	CRect getCaptionRect();

	// Create the control with which you can change the current
	// visible ccField. But don't show it before we need it. 
	//
	void createFieldChangerObject();

	// Calc the position and size for the fieldChangerRect
	//
	CRect getFieldChangerRect();

	// This will be shown if the ccfield Connected
	// to this ctrl has an popup function.	
	//
	void createComboButtonObject();

	// Calc the position and size for the comboButtonRect
	//
	CRect getComboButtonRect();

protected:
	ccEdit					* m_ccdbEdit;		// see createccdbEditObject
	ccdbSuperCtrl_CheckBox	* m_ccdbCheckBox;	// see cteateccdbCheckBoxObject

	CStatic		 * m_caption;					// see createCaptionObject

	ccMenuButton * m_filedChanger;				// see createFieldChangerObject

	CButtonST	 * m_comboButton;				// see createComboButtonObject


// ** General
public:
	// Will be called from the connected ccField when the
	// value in the field has been changed
	//
	virtual bool refresh();	

	// The container this superctrl is placed in.
	//
	void setOwnerContainer( ccddCtrlContainer * aOwner)	{ m_ownerContainer = aOwner;		};
	ccddCtrlContainer * getOwnerContainer()				{ return m_ownerContainer;			};

private:
	ccddCtrlContainer * m_ownerContainer;		// see setOwnerContainer	

// *** Data Members
private:	
	bool m_bEditing;				// true if the focus are in the edit ctrl


// *** Controlling the datadictionary.
public:	
	// Show the popup list.
	//
	void OnRecordPopup();

	// Reread the data from the table to the datadictionary
	//
	void OnRecordRefresh();

	// Clear all records from this and all parent 
	// datadictionarys	
	//
	void OnRecordClear();				

	// Clear all ctrls from the owner window.
	//
	void OnRecordClearAll();				

	// Delete the current record
	//
	void OnRecordDelete();				

	// Go to next record if this column is sortable.
	//
	void OnRecordNext();				

	// Go to prev record if this column is sortable.
	//
	void OnRecordPrev();				

	// Go to first record if this column is sortable.
	//
	void OnRecordFirst();				

	// Go to last record if this column is sortable.
	//
	void OnRecordLast();

	// Save the current record.
	//
	void OnRecordSave();				

	// Find the record with the index typed in the indexed ctrls
	//
	void OnRecordFind();

// *** Overrides
public:
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(ccdbSuperCtrl)
	public:
	virtual BOOL Create(const RECT& rect, CWnd* pParentWnd, UINT nID, DWORD dwStyle);
	virtual BOOL PreTranslateMessage(MSG* pMsg);		
	protected:
	virtual void PreSubclassWindow();
	//}}AFX_VIRTUAL
	
	virtual void endEdit();			// Called when focus kills. Used by CInPlaceEdit	

	// Make the member compatible with the diffrent control styles..
	//
	void GetWindowText( CString& rString );
	int  GetWindowText( LPTSTR lpszStringBuf, int nMaxCount );
	

// *** Generated message map functions
protected:
	//{{AFX_MSG(ccdbSuperCtrl)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnMove(int x, int y);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	//}}AFX_MSG
	afx_msg void OnChangeEdit();
	afx_msg void OnKillFocusEdit();
	afx_msg void OnSetFocusEdit();

	afx_msg void OnCheckCheckBox();
	afx_msg void OnSetFocusCheckBox();
	afx_msg void OnKillFocusCheckBox();
	
	afx_msg void OnComboButtonButton();
	afx_msg void OnFieldChangerButton();

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // __CCDBSUPERCTRL_H__
