//////////////////////////////////////////////////////////////////////
// $File name   : ccDataBase.cpp
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ccDataBase.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// *** Construction/Destruction
//////////////////////////////////////////////////////////////////////

ccDataBase::ccDataBase()
{
	m_bReadyToUse = false;

	setCrntFocus( NULL );

	setDataBaseType( CCDB_NODB );
	m_dataBase = NULL;
	m_connection = NULL;

	// *** todo *** ccdbServer * m_dbServer = NULL;
}

//

void ccDataBase::openODBC( CString asConnectDSN )
{
	if ( getDataBaseType() == CCDB_NODB )
	{
		TRY
		{	
			NEEDCLEAR( m_dataBase );

			setConnectDSN( asConnectDSN );			

			// Create database object.			
			m_dataBase = new CDatabase;		

			// TODO maybe the userCursorLib don't need too be here, 
			// if the data base is on sql server and not access.
			BOOL lbOpenSucceded = m_dataBase->OpenEx( getConnectDSN(), CDatabase::noOdbcDialog | CDatabase::useCursorLib );
			NEEDVALUE( lbOpenSucceded );			
			setDataBaseType( CCDB_ODBC );
		}

		CATCH(CDBException, e)
		{
			ccThrowccException(  e->m_strError );			
		}
		END_CATCH
	}
	else
	// Database already opened.
	{
		ccThrowccException( IDS_DB_ALREADY_OPENED );		
	}	
}

//

void ccDataBase::openADO( LPCTSTR asConnectionString, ADODB::CursorLocationEnum adCursorType ,			\
				   bool bExclusive , bool bReadOnly , int aiConnectionTimeout )
{
	if ( getDataBaseType() == CCDB_NODB )
	{
		NEEDCLEAR( m_connection );

		TESTHR(m_connection.CreateInstance(__uuidof(ADODB::Connection)));


		HRESULT  hr = S_OK;
		try
		{
			m_connection->ConnectionString = asConnectionString;

			m_connection->CursorLocation = adCursorType;	// adUseClient | adUseNone| adUseServer
			
			if (bExclusive)
				m_connection->Mode = ADODB::adModeShareExclusive;

			if (bReadOnly)
			{
				//if (bExclusive)
					//	TODO Error

				m_connection->Mode = ADODB::adModeShareExclusive;
			}
			
			m_connection->ConnectionTimeout = aiConnectionTimeout;	

			m_connection->Open("","","",NULL);
			
			if ( m_connection->State != ADODB::adStateOpen )
				ccThrowccException( IDS_CANT_OPEN_DATABASE ); 			
		
			setDataBaseType( CCDB_ADO );
		}
		catch(_com_error &e) 
		{ 
			ccThrowccException( getProviderError( m_connection) );	/*Delete unreference warning*/ (void*)&e; 
		}
	}
	else
	// Database already opened.
	{
		ccThrowccException( IDS_DB_ALREADY_OPENED );		
	}	
}

//

void ccDataBase::openADOMSSQLServer( LPCTSTR asServerName, LPCTSTR asDatabase, LPCTSTR asUser, LPCTSTR asPassword, 
								     ADODB::CursorLocationEnum adCursorType /*= adUseNone*/,
								     bool bExclusive /*= FALSE*/, bool bReadOnly /*= FALSE*/, int aiConnectionTimeout /*= 15*/ )
{
	char lsConStr[256];
	strcpy( lsConStr, "Provider=SQLOLEDB.1;");
	
	if (asServerName) strcat( lsConStr, "Server=");
	if (asServerName) strcat( lsConStr, asServerName);
	
	if (asDatabase) strcat( lsConStr, ";Database=");
	if (asDatabase) strcat( lsConStr, asDatabase);	
	
	if (asUser) 
	{
		strcat( lsConStr, ";User ID=");
		strcat( lsConStr, asUser);
	}
	else
		strcat( lsConStr, ";Integrated Security=SSPI;Persist Security Info=False;");	

	if (asPassword) strcat( lsConStr, ";Password=");
	if (asPassword) strcat( lsConStr, asPassword);

	openADO( lsConStr, adCursorType, bExclusive, bReadOnly, aiConnectionTimeout );	
}

//
void ccDataBase::openADOAccessDB( LPCTSTR asDataBaseURL, LPCTSTR asDataBasePassword, \
	 					   ADODB::CursorLocationEnum adCursorType/* = ADODB::adUseNone*/,
						   bool bExclusive/* = FALSE*/, bool bReadOnly/* = FALSE*/, \
						   int aiConnectionTimeout /*= 15*/ )
{
	char lsConStr[256];
	strcpy( lsConStr, "Provider=Microsoft.Jet.OLEDB.4.0");
		
	strcat( lsConStr, ";Data Source=");
	strcat( lsConStr, asDataBaseURL);	

	strcat( lsConStr, ";Persist Security Info=False");		

	strcat( lsConStr, ";Jet OLEDB:Database Password=");
	strcat( lsConStr, asDataBasePassword);	

	openADO( lsConStr, adCursorType, bExclusive, bReadOnly, aiConnectionTimeout );	
}

//

void ccDataBase::close( )
{
	switch( getDataBaseType() )
	{
	case CCDB_ADO :
		try
		{
			if(m_connection != NULL)
			{
				if(m_connection->State==ADODB::adStateOpen)
					m_connection->Close();
			}
		}
		catch(_com_error &e) 
		{ 
			ccThrowccException( getProviderError( m_connection) ); /*Delete unreference warning*/ (void*)&e; 
		}	
		break;
	case CCDB_ODBC:
		delete m_dataBase;
		m_dataBase = NULL;
	default:

		break;
	}
}

//

ccDataBase::~ccDataBase()
{
	close();
}

//////////////////////////////////////////////////////////////////////
// *** Properties
//////////////////////////////////////////////////////////////////////

BOOL ccDataBase::isOpen()
{
	switch( getDataBaseType() )
	{
	case CCDB_ODBC:
		return m_dataBase->IsOpen();
		break;
	case CCDB_ADO:
		return (m_connection->State == ADODB::adStateOpen);
		break;
	case CCDB_CCSERVICE:
		ccErr( "Not implemented");
		break;
	case CCDB_NODB :
	default:
		ccErr( IDS_INVALID_DATABASE_TYPE );		
		break;
	}	
	return false;
}

//////////////////////////////////////////////////////////////////////
// *** Input/Output
//////////////////////////////////////////////////////////////////////
ccdbSuperCtrl * ccDataBase::m_ctrnFocusCtrl = NULL;	// see setCtrnFocus

//////////////////////////////////////////////////////////////////////
// *** Implementation
//////////////////////////////////////////////////////////////////////

void ccDataBase::executeSQL( CString asqlStmt )
{
	TRY
	{
		getDataBase()->ExecuteSQL( asqlStmt );
	   	
	}

	CATCH(CDBException, e)
	{
   		ccErr( e->m_strError );
		e->Delete();
		return ;
	}

	END_CATCH
}

//////////////////////////////////////////////////////////////////////
// *** Implementation
//////////////////////////////////////////////////////////////////////

bool ccDataBase::getRS(LPCTSTR asSQL, ADODB::_RecordsetPtr &PopSet)
{
	VARIANT RecordsEffected;
	RecordsEffected.vt = VT_INT;
	try
	{
		if( isOpen() )
		{			
			PopSet.CreateInstance(__uuidof(ADODB::Recordset));		
			PopSet = m_connection->Execute(_bstr_t(asSQL),&RecordsEffected,ADODB::adCmdText);

			if(!PopSet->adoEOF)
			{				
				return true;
			}
			else			
			{			
				return false;
			}
		}
		else		
			ccThrowccException( IDS_DATABASE_ISNT_OPEN );
	}
	catch(_com_error &e) 
	{ 
		ccThrowccException( getProviderError( m_connection) ); /*Delete unreference warning*/ (void*)&e; 
	}	
	return false;
}

//

bool ccDataBase::getRSVariant(LPCTSTR asSQL, VARIANT *Result)
{
	VARIANT RecordsEffected;
	RecordsEffected.vt = VT_INT;
	try
	{
		if( isOpen() )
		{
			ADODB::_RecordsetPtr PopSet;
			PopSet.CreateInstance(__uuidof(ADODB::Recordset));		
			PopSet = m_connection->Execute(_bstr_t(asSQL),&RecordsEffected,ADODB::adCmdText);

			if(!PopSet->adoEOF)
			{
				*Result = PopSet->Fields->Item[_variant_t(0l)]->Value;
				return true;
			}
			else			
			{
				*Result = _variant_t("");							
				return false;
			}
		}
		else		
			ccThrowccException( IDS_DATABASE_ISNT_OPEN );
	}
	catch(_com_error &e) 
	{ 
		ccThrowccException( getProviderError( m_connection) ); /*Delete unreference warning*/ (void*)&e; 
	}	
	return false;
}

//

bool ccDataBase::getRSVArray( LPCTSTR  strSQL, CArray<_variant_t, _variant_t> &result )
{	
	VARIANT RecordsEffected;
	RecordsEffected.vt = VT_INT;
	try
	{
		if( isOpen() )
		{
			ADODB::_RecordsetPtr PopSet;
			PopSet.CreateInstance(__uuidof(ADODB::Recordset));		
			PopSet = m_connection->Execute(_bstr_t(strSQL),&RecordsEffected,ADODB::adCmdText);

			if(!PopSet->adoEOF)
			{
				for ( long i=0; i < result.GetSize(); i++)
				{
					result[i] = PopSet->Fields->Item[_variant_t(i)]->Value;
				}
				
				return true;
			}
			else			
			{										
				for ( long i=0; i < result.GetSize(); i++)
				{
					result[i] = _variant_t("");
				}
				return false;
			}
		}
		else		
			ccThrowccException( IDS_DATABASE_ISNT_OPEN );
	}
	catch(_com_error &e) 
	{ 
		ccThrowccException( getProviderError( m_connection) ); /*Delete unreference warning*/ (void*)&e; 
	}	
	return false;
}

//

bool ccDataBase::executeSQLNoRet(LPCTSTR asSQL, VARIANT &RecordsEffected)
{	
	RecordsEffected.vt = VT_INT;
	try
	{
		if( isOpen() )
		{			
			m_connection->Execute(_bstr_t(asSQL),&RecordsEffected,ADODB::adCmdText);			
			return true;									
		}
	}
	catch(_com_error &e) 
	{ 
		rollbackTrans();
		ccThrowccException( getProviderError( m_connection) ); /*Delete unreference warning*/ (void*)&e; 
	}	
	return false;
}

//

bool ccDataBase::getRSDelimited(LPCTSTR asSQL, LPCTSTR asDelimiter, VARIANT *Result)
{
	VARIANT RecordsEffected;
	RecordsEffected.vt = VT_INT;
	*Result = _variant_t("Yea");
	try
	{
		ADODB::_RecordsetPtr PopSet;
		if( !isOpen() )
		{
			ccThrowccException( IDS_DATABASE_ISNT_OPEN );			
		}
		PopSet = m_connection->Execute(asSQL,&RecordsEffected,ADODB::adCmdText);
		if(!PopSet->adoEOF)
		{
			BSTR bstrResult = NULL;
			_bstr_t btColDelim(L","); 
			_bstr_t btRowDelim(L"\r\n"); 
			_bstr_t btNullExp(L""); 
			PopSet->raw_GetString(ADODB::adClipString,-1,_bstr_t( asDelimiter ),btRowDelim,btNullExp,&bstrResult);
			*Result = _variant_t(bstrResult).Detach();			
		}
		else
			*Result = _variant_t("");
		return true;
	}
	catch(_com_error &e) 
	{ 
		ccThrowccException( getProviderError( m_connection) ); /*Disable unreference warning*/ (void*)&e; 
	}	
	return false;
}

//////////////////////////////////////////////////////////////////////
// *** GLOBAL ADO functions.
//////////////////////////////////////////////////////////////////////

CString getProviderError( ADODB::_ConnectionPtr pConnection )
{       
    // Print Provider Errors from Connection object.
    // pErr is a record object in the Connection's Error collection.
    //Define Other Variables
    HRESULT  hr = S_OK;        
	CString strAllErrors;

    try
    {
		ADODB::ErrorPtr  pErr = NULL;		
		

		if( (pConnection->Errors->Count) > 0)
		{
			long nCount = pConnection->Errors->Count;

			// Collection ranges from 0 to nCount -1.
			for(long i = 0; i < nCount; i++)
			{
				pErr = pConnection->Errors->GetItem(i);

				CString strBuf;
				CString strHelpFile;
				CString strHelpContext;
				if ((LPCSTR)pErr->GetHelpFile() == NULL)
				{
					strHelpFile = "No available";
					strHelpContext = "No available";
				}
				else
				{
					strHelpFile    = (LPCSTR) pErr->HelpFile;
					strHelpContext = (LPCSTR) pErr->HelpContext;
				}
        

				sprintf(strBuf.GetBuffer( 2000 ),"Provider Error\n"
													"\t Error Number: %x\n"
													"\t NativeError:  %x\n"
													"\t Source: %s\n"
													"\t SQLState: %s\n"
													"\t HelpFile:    %s\n"
													"\t HelpContext: %s\n"
													"\t %s\n"												
													, pErr->Number
													, pErr->NativeError
													, (LPCSTR)pErr->Source
													, (LPCSTR)pErr->SQLState
													, strHelpFile
													, strHelpContext
													, (LPCSTR)pErr->Description);

				strBuf.ReleaseBuffer();
				strAllErrors += strBuf;
			}
		}	
	}

    catch(_com_error &e)
    {
        // Notify the user of errors if any.
        strAllErrors = getComError(e);
    }

	return strAllErrors;
}

//

CString getComError(_com_error &e)
{
    _bstr_t bstrSource(e.Source());
    _bstr_t bstrDescription(e.Description());

	CString strBuf;
    // Print Com errors.  
    sprintf(strBuf.GetBuffer(500), "Error\n\tCode = %08lx\n\
										\tCode meaning = %s\n\
										\tSource = %s\n\
										\tDescription = %s\n ", \
					e.Error(), e.ErrorMessage(), (LPCSTR) bstrSource, (LPCSTR) bstrDescription );

	strBuf.ReleaseBuffer();
	return strBuf;
}

//
