//////////////////////////////////////////////////////////////////////
// $File name   : ccEdit.cpp: 
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ccgLib.h"
#include "ccEdit.h"
#include "ccTime.h"
#include "float.h"
#include "resource.h"
#include "ccEditSpinCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(ccEdit, CEdit)

/////////////////////////////////////////////////////////////////////////////
// *** Construction
/////////////////////////////////////////////////////////////////////////////

ccEdit::ccEdit()
: m_maskType(CCE_CUSTOM)
{
	m_bLastKeyWasDelete = false;
	m_bLastKeyWasBackspace = false;

	m_cPlaceHolder = '_';

	// Numeric mask 
	m_numberOfNumberAfterPoint = 0;	
	m_minNumValue = -FLT_MAX;
	m_maxNumValue = FLT_MAX;
	m_delta		  = FLT_ROUNDS;

	// SpinCtrl
	m_spinCtrl = NULL;
}

//

ccEdit::~ccEdit()
{
	//Delete all ctrls.
	if ( m_spinCtrl != NULL )
	{
		delete m_spinCtrl;
		m_spinCtrl = NULL;
	}	
}

/////////////////////////////////////////////////////////////////////////////
// *** Mask definations
/////////////////////////////////////////////////////////////////////////////

void ccEdit::setCustomMask( CString asMask, char acPlaceHolder /*= '_'*/ )
{
	m_maskType = CCE_CUSTOM;
	ModifyStyle(ES_NUMBER, 0, SWP_DRAWFRAME);
	setMask(asMask, acPlaceHolder );	
}

//

void ccEdit::setDateMask()
{
	m_maskType = CCE_DATE;
	ModifyStyle(ES_NUMBER, 0, SWP_DRAWFRAME);
	
	/*
	CString strData;
	int moo = GetLocaleInfo(
	  LOCALE_USER_DEFAULT,      // locale identifier
	  LOCALE_SSHORTDATE  ,    // information type
	  strData.GetBuffer(40),  // information buffer
	  40// size of buffer
	);
	strData.FreeExtra();

	setMask(strData);

	strData.Replace(dd, "01");
	strData.Replace(DD, "01");
	strData.Replace(mm, "01");
	strData.Replace(MM, "01");
	*/
	
	setMask("####-##-##");

	SetWindowText("19000101");
}

//

void ccEdit::setTimeMask()
{
	m_maskType = CCE_TIME;
	ModifyStyle(ES_NUMBER, 0, SWP_DRAWFRAME);
	setMask("##:##:##");
	SetWindowText("000000");
}

//

void ccEdit::setDateTimeMask()
{
	m_maskType = CCE_DATETIME;
	ModifyStyle(ES_NUMBER, 0, SWP_DRAWFRAME);
	setMask("####-##-## ##:##:##");
	SetWindowText("19000101000000");
}

//

void ccEdit::setTextMask(int aiLength)
{	
	NEEDVALUE( aiLength );
	m_maskType = CCE_TEXT;
	ModifyStyle(ES_NUMBER, 0, SWP_DRAWFRAME);
	SetLimitText( aiLength );
}

//

void ccEdit::setNumericMask( float aiMinNumValue, float aiMaxNumValue)
{	
	NEEDMORE(aiMaxNumValue, aiMinNumValue );				

	m_minNumValue = aiMinNumValue;
	m_maxNumValue = aiMaxNumValue;

	setNumericMask( 9999 );
}

//
void ccEdit::setNumericMask(int aiLength)
{
	NEEDVALUE( aiLength );

	m_maskType = CCE_NUMERIC;
	ModifyStyle(ES_NUMBER, 0, SWP_DRAWFRAME);	// The only number check is done in OnChar
	SetLimitText( aiLength );
}

//

void ccEdit::setCurrencyMask(int aiLength)
{
	NEEDVALUE( aiLength );

	m_maskType = CCE_CURRENCY;
	ModifyStyle(ES_NUMBER, 0, SWP_DRAWFRAME);
	char *lpCurrency = new char[20];	
	if ( GetLocaleInfo( LOCALE_USER_DEFAULT, LOCALE_SCURRENCY, lpCurrency, 20 ) == FALSE)
		DEBUGOUT( "Can't get the Currency" );

	setMask("####-##-## ##:##:##");
}


/////////////////////////////////////////////////////////////////////////////
// *** Mask handling 
/////////////////////////////////////////////////////////////////////////////

void ccEdit::setMask(LPCTSTR pszMask, char cPlaceHolder)
{
	m_strMaskText = pszMask;
	m_cPlaceHolder = cPlaceHolder;

	m_strSaveText.Empty();

	int nIndex = 0;
	while (nIndex < m_strMaskText.GetLength())
	{
		if (isMaskChar(nIndex))
			m_strSaveText += m_strMaskText[nIndex];
		else
			m_strSaveText += m_cPlaceHolder;

		nIndex++;
	}

	if ( (m_maskType == CCE_DATE) || (m_maskType == CCE_DATETIME) || (m_maskType == CCE_TIME) ) 
		m_strSaveText = validateDate( m_strSaveText );
	CWnd::SetWindowText(m_strSaveText);
}

//

CString ccEdit::validateDate(CString asDate)
{	
	// If the string doesn't contain a valid date, create a valid date.
	asDate.Replace(m_cPlaceHolder,'0');

	// Convert string to a ccTime.
	ccTime lTime;

	// Validate the date and return a valid date.
	CString lsRet;	
	switch ( m_maskType )
	{
	case CCE_DATE:
		lTime.setDate( asDate );
		lsRet = lTime.getDate();
		break;
	case CCE_TIME:
		lTime.setTime( asDate );
		lsRet = lTime.getTime();
		break;
	case CCE_DATETIME:
		lTime.setDateTime( asDate );
		lsRet = lTime.getDateTime();
		break;
	default:
		lsRet = asDate;
		break;
	}		
	return lsRet;
}

//

bool ccEdit::isMaskChar(int nPos)
{
	ASSERT(nPos < m_strMaskText.GetLength());

	return (m_strMaskText[nPos] != '?' &&
			m_strMaskText[nPos] != '#' &&
			m_strMaskText[nPos] != 'd' && m_strMaskText[nPos] != 'D' &&
			m_strMaskText[nPos] != 'm' && m_strMaskText[nPos] != 'M' &&
			m_strMaskText[nPos] != 'y' && m_strMaskText[nPos] != 'Y' &&			
			m_strMaskText[nPos] != '%'); // Add new masks as needed
}

//

int ccEdit::gotoNextEntryChar(int nPos)
{
	if (m_bLastKeyWasBackspace)
	{
		nPos--;

		while (nPos >= 0 && isMaskChar(nPos))
			nPos--;

		nPos++;
	}
	else
	{
		while (nPos < m_strMaskText.GetLength() && isMaskChar(nPos))
			nPos++;
	}
	return nPos;
}

//

bool ccEdit::getReplacementChar(int nPos, char cWant, char& cGet)
{
	if (nPos >= m_strMaskText.GetLength())
		return false;

	switch (m_strMaskText[nPos])
	{
	// Any char
	case '?':
		cGet = cWant;
		break;

	// Only numbers
	case '#':
	case 'd': case 'D':
	case 'm': case 'M':
	case 'y': case 'Y':
		if (0 == isdigit(cWant))
			return false;

		cGet = cWant;
		break;

	// Only alpha numeric.
	case '%':
		if (0 == isalpha(cWant))
			return false;

		cGet = cWant;
		break;

	default:
		cGet = m_strMaskText[nPos];
		break;
	}
	return true;
}

/////////////////////////////////////////////////////////////////////////////
// *** Numeric mask functions.
/////////////////////////////////////////////////////////////////////////////

void ccEdit::setDelta(float delta)
{
	m_delta = delta;
}

//

float ccEdit::getDelta()
{
	return m_delta;
}

//

int ccEdit::isFloatValid( CString& str) 
{
	NEEDVALUE( m_maskType == CCE_NUMERIC);

	int res = VALID;
	float f;
	char lp[10];
	if (str.GetLength() != 0)
		if ((str.GetLength() == 1) && ((str[0] == '+') || (str[0] == '-'))) 
			res = MINUS_PLUS;
		else
			if (sscanf(str, "%f%s", &f, lp) != 1) res = INVALID_CHAR;
		else
			if ( (f > m_maxNumValue) || (f < m_minNumValue)) res = OUT_OF_RANGE;

	return res;
}

//

CString& ccEdit::constructNumFormat(CString &str, float num)
{
	NEEDVALUE( m_maskType == CCE_NUMERIC);

	str.Format("%G", num);
	int n = str.Find('.');
	if (n >= 0)
	{
		n = str.GetLength() - n - 1;
		if (n > m_numberOfNumberAfterPoint) m_numberOfNumberAfterPoint = n;
	}
	str.Format("%%.%df", m_numberOfNumberAfterPoint);
	return str;
}

//

void ccEdit::enableSpinCtrl()
{
	if ( m_spinCtrl == NULL )
	{
		m_spinCtrl = new ccEditSpinCtrl;
		
		m_spinCtrl->Create( WS_VISIBLE | UDS_ARROWKEYS | UDS_ALIGNRIGHT, CRect(0,0,20,20), GetParent(), ID_CCEDIT_SPINCTRL );
		m_spinCtrl->SetBuddy( this );		
	}

	switch ( m_maskType )
	{
		case CCE_NUMERIC:
		case CCE_TEXT:
		default:
			break;
	}
}

/////////////////////////////////////////////////////////////////////////////
// *** Overrides
/////////////////////////////////////////////////////////////////////////////

void ccEdit::SetWindowText(LPCTSTR pszText)
{
	switch (m_maskType)
	{
	// Don't do any masking stuff on CCE_TEXT and CCE_NUMERIC masks.
	//
	case CCE_TEXT:
		m_strSaveText = pszText;
		break;
	case CCE_NUMERIC:		
		{
			int val = isFloatValid( CString(pszText) );
			switch (val)
			{
			case VALID:
			case MINUS_PLUS:				
				m_strSaveText = pszText;		
				break;
			default:
				return;
				break;
			}
		}
		break;

	default:		
		{
			CString strText(pszText);

			m_strSaveText.Empty();

			int nMaskIndex = 0;
			int nTextIndex = 0;

			while (nMaskIndex < m_strMaskText.GetLength())
			{
				if (!isMaskChar(nMaskIndex))
				{
					if (nTextIndex < strText.GetLength())
						m_strSaveText += strText[nTextIndex++];
					else
						m_strSaveText += m_cPlaceHolder;
				}
				else
					m_strSaveText += m_strMaskText[nMaskIndex];

				nMaskIndex++;
			}
		}
		break;
	}

	// Validate date and time mask types.
	if ( (m_maskType == CCE_DATE) || (m_maskType == CCE_DATETIME) || (m_maskType == CCE_TIME) ) 
		m_strSaveText = validateDate( m_strSaveText );

	CWnd::SetWindowText(m_strSaveText);
}

//

void ccEdit::GetWindowText( CString& rString ) 
{
	switch (m_maskType)
	{
	// Don't do any masking stuff on CCE_TEXT and CCE_NUMERIC masks.
	//
	case CCE_DATETIME:
	case CCE_DATE:	
	case CCE_TIME:
	case CCE_NUMERIC:
	case CCE_TEXT:
	case CCE_BYTE:
	case CCE_WORD:
	case CCE_INT:
	case CCE_UINT:
		rString = m_strSaveText;
		break;

	default:		

		rString.Empty();	

		int nMaskIndex = 0;
		while (nMaskIndex < m_strMaskText.GetLength())
		{
			if (!isMaskChar(nMaskIndex))
				rString += m_strSaveText[nMaskIndex];

			nMaskIndex++;
		}
		break;
	}
}

//

int ccEdit::GetWindowText( LPTSTR lpszStringBuf, int nMaxCount ) 
{
	NEEDVALUE( nMaxCount );
	CString lsString;
	GetWindowText( lsString );
		
	int liStringLength = lsString.GetLength();

	// If string length is larger then allowed
	// truncate it to allowed size.
	if ( liStringLength > nMaxCount )
	{
		strcpy( lpszStringBuf, lsString.Left( nMaxCount ) );
		liStringLength = nMaxCount;
	}
	else 
		strcpy( lpszStringBuf, lsString );

	return liStringLength;
}


BEGIN_MESSAGE_MAP(ccEdit, CEdit)
	//{{AFX_MSG_MAP(ccEdit)
	ON_CONTROL_REFLECT(EN_UPDATE, OnUpdate)
	ON_WM_KEYDOWN()
	ON_WM_CHAR()
	ON_WM_CREATE()
	ON_NOTIFY(UDN_DELTAPOS, ID_CCEDIT_SPINCTRL, OnDeltaposSpinCtrl)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void ccEdit::OnUpdate() 
{	
	switch (m_maskType)
	{
		//Dont do any masking stuff on a CCE_TEXT and CCE_NUMERIC type mask

	case CCE_NUMERIC:
		{								
			CString strEditText;
			CEdit::GetWindowText(strEditText);

			// Return if no new value has been typed.
			if (strEditText == m_strSaveText)
				return;

			int val = isFloatValid( strEditText );
			switch (val)
			{
			case VALID:
			case MINUS_PLUS:
				m_strSaveText = strEditText;
				break;
			default:
				CWnd::SetWindowText(m_strSaveText);
				SetSel(0, -1);
				break;
			}			
		}
		break;

	case CCE_TEXT:
		CEdit::GetWindowText(m_strSaveText);
		break;
	
	default:
		{
			int nStart, nEnd;
			GetSel(nStart, nEnd);

			CString strEditText;
			CEdit::GetWindowText(strEditText);

			if (strEditText == m_strSaveText)
				return;

			if (m_bLastKeyWasBackspace)
			{
				m_strSaveText.SetAt(nStart, isMaskChar(nStart) ? m_strMaskText[nStart] : m_cPlaceHolder);

				if ( (m_maskType == CCE_DATE) || (m_maskType == CCE_DATETIME) || (m_maskType == CCE_TIME) ) 
					m_strSaveText = validateDate( m_strSaveText );
				CWnd::SetWindowText(m_strSaveText);
				int nNext = gotoNextEntryChar(nStart);
				SetSel(nNext, nNext);
			}
			else if (m_bLastKeyWasDelete)
			{
				m_strSaveText.SetAt(nStart, isMaskChar(nStart) ? m_strMaskText[nStart] : m_cPlaceHolder);

				if ( (m_maskType == CCE_DATE) || (m_maskType == CCE_DATETIME) || (m_maskType == CCE_TIME) ) 
					m_strSaveText = validateDate( m_strSaveText );
				CWnd::SetWindowText(m_strSaveText);
				int nNext = gotoNextEntryChar(nStart + 1);
				SetSel(nNext, nNext);
			}
			else
			{
				if (nStart - 1 < 0)
					return;

				char cWanted = strEditText[nStart - 1];

				char cReplace = 0;

				if (getReplacementChar(nStart - 1, cWanted, cReplace))
				{
					if (m_strSaveText.GetLength() < nStart)
						m_strSaveText.GetBufferSetLength(nStart);

					m_strSaveText.SetAt(nStart - 1, cReplace);
					m_strSaveText.ReleaseBuffer();
				}

				if ( (m_maskType == CCE_DATE) || (m_maskType == CCE_DATETIME) || (m_maskType == CCE_TIME) ) 
					m_strSaveText = validateDate( m_strSaveText );
				CWnd::SetWindowText(m_strSaveText);
				int nNext = gotoNextEntryChar(nStart);
				SetSel(nNext, nNext);
			}
		}
	}
}

//

void ccEdit::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	m_bLastKeyWasDelete = (VK_DELETE == nChar);
	m_bLastKeyWasBackspace = (VK_BACK == nChar);
	
	CEdit::OnKeyDown(nChar, nRepCnt, nFlags);
}

//

void ccEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch ( m_maskType )
	{
		case CCE_NUMERIC:
			if (nChar == ' ') return;
			CEdit::OnChar(nChar, nRepCnt, nFlags);			
			break;

		default:
			CEdit::OnChar(nChar, nRepCnt, nFlags);
			break;
	}
}
//

void ccEdit::OnDeltaposSpinCtrl(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	
	switch ( m_maskType )
	{
		case CCE_NUMERIC:
			{
				CString strEditText;
				CEdit::GetWindowText(strEditText);

				int val = isFloatValid( strEditText );
				switch (val)
				{
				case VALID:
				case MINUS_PLUS:
					{
						// Convert from string to float
						float f = m_minNumValue;
						sscanf(strEditText, "%f", &f);

						// Inc/Dec the value
						f -= pNMUpDown->iDelta*m_delta;

						// Set the new float value
						CString str, s;
						str.Format(constructNumFormat(s, f), f);
						SetWindowText(str);
					}				
					break;
				default:
					break;
				}		
			}
			break;		
		default:	
			break;
	}	
	
	*pResult = 0;
}

//
