// ==========================================================================
//                    Class Specification : ccEdit
// ==========================================================================
//////////////////////////////////////////////////////////////////////
// $File name   : /ccLib/dev/ccgLib/ccEdit.h: 
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// $HomePage    : home.bip.net/CyberCow
// $ICQ         : 7983755
//
// Notice       : Copyright (c) CyberCow 1999
//                Knypplerskevagen 24
//                168 38 BROMMA
//                SWEDEN  
//                tel int + 46 (8) 26 45 95
//                
//                The copyright to the computer program(s) herein is the 
//				  property of CyberCow.The program(s) may be used and/or 
//				  copied only with the written permission of CyberCow or 
//				  in accordance with the terms and conditions stipulated 
//				  in the agreement/contract under which the program(s) 
//				  have been supplied. 
//
/////////////////////////////////////////////////////////////////////////////
///** $History$ *************************************************************
// *
// * $Date: 9/15/99 15:35 $
// * $Revision: 1 $
// * 
// * *****************  Version 1  *****************
// * User: Daniel Date: 9/15/99 15:35
// * Source-Code Traced [ ]; Reviewd [ ] 
// * Created in $/ccLib/dev/ccgLib
// * Initial release.
// * 
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
// ** $Properties$ **********************************************************
// * [ ] Abstract class (does not have any objects)
// * [ ] Uses exceptions
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
/*
$Description : 

  The ccEdit class is a CEdit derived class with masking support. It has a 
  coupple of predefined mask types. Date, Time, Date+Time, var length text,
  var length numbers.

*/
/////////////////////////////////////////////////////////////////////////////

#ifndef __CCEDIT_H__
#define __CCEDIT_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// Predifined masktypes.
//
enum CCE_TYPE
{
	CCE_CUSTOM = 0,
	CCE_DB_DEFAULT,			// Reserved for the dataDictionary classes.
	CCE_DATETIME,
	CCE_DATE,	
	CCE_TIME,
	CCE_NUMERIC,
	CCE_TEXT,
	CCE_BYTE,
	CCE_WORD,
	CCE_INT,
	CCE_UINT,
	CCE_CURRENCY
};

class ccEditSpinCtrl;

class AFX_EXT_CLASS ccEdit : public CEdit
{
	DECLARE_DYNAMIC(ccEdit)
// *** Construction
public:
	// Default constructor
	//
	ccEdit();
	~ccEdit();


// *** Mask definations
public:
	// Set a custom mask. Valid masks are '?' any char, '#' only numbers
	// '%' only alpha numeric
	//	
	void setCustomMask( CString asMask, char acPlaceHolder = '_' );

	// The mask will look like this ####-##-## and verfiy the date 
	// to a legal date between 1901-01-01 and 9999-12-31.
	//
	void setDateMask();

	// The mask will look like this ##:##:## and verfiy the time
	// to a legal time between 00:00:00 and 23:59:59.
	//
	void setTimeMask();

	// The mask will be the data and time mask together, and 
	// will also be verifyed as the date and time masks
	//
	void setDateTimeMask();

	// The mask will be a variable length number of chars.
	//
	void setTextMask(int aiLength);

	// The mask will be a variable length number of numberic chars.
	//
	void setNumericMask( int aiLength);
	void setNumericMask( float aiMinNumValue, float aiMaxNumValue);

	// Set currency mask
	//
	void setCurrencyMask(int aiLength);

	//
	//
	// **void setByteMask();
	// **void setWordMask();
	// **void setIntMask();
	// **void setUIntMask();
	
	// Get the current mask.
	//
	CString	getMask()											{ return m_strMaskText;				};

	// Returns the mask type
	//
	CCE_TYPE getMaskType() const								{ return m_maskType;				};


// *** Mask handling 
private:	
	// Set the mask. 
	//
	void setMask(LPCTSTR pszMask, char cPlaceHolder = '_');

	// If the mask type is date, time or datetime the contents
	// of the edit box will be verifyed every time
	// the user input some values into it.
	CString validateDate(CString asDate);

	//
	//
	bool	isMaskChar(int nPos);

	//
	//
	int		gotoNextEntryChar(int nPos);

	//
	//
	bool	getReplacementChar(int nPos, char cWant, char& cGet);


// *** Mask data members.
private:	
	CCE_TYPE m_maskType;				// Mask type applyed on the edit box.

	char	m_cPlaceHolder;				// Place holder in the mask
	
	CString m_strMaskText;				// How the mask looks like

	CString m_strSaveText;

	bool	m_bLastKeyWasDelete;

	bool	m_bLastKeyWasBackspace;

// *** Numeric mask functions.
public:
	// The numbers the spinctrl or arrow keys will increase/decrease the numeric
	// masked edit ctrl.
	//
	void  setDelta(float delta);
	float getDelta();

	int	  isFloatValid( CString & str);

protected:
	CString& constructNumFormat(CString &str, float num);

private:
	enum {VALID = 0x00, OUT_OF_RANGE = 0x01, INVALID_CHAR = 0x02, MINUS_PLUS = 0x03};

	BYTE m_numberOfNumberAfterPoint;
	float m_minNumValue;				// See setNumericMask
	float m_maxNumValue;				// See setNumericMask
	float m_delta;						// See set setDelta

// *** Spin Ctrl
public:
	// Enables a spin control in the edit ctrl.
	// If numeric mask
	//
	void enableSpinCtrl();
private:
	ccEditSpinCtrl * m_spinCtrl;


// *** Overrides
public:

	// Make the member compatible with the mask.
	//
	void SetWindowText(LPCTSTR lpszString);

	// Make the member compatible with the mask.
	//
	void GetWindowText( CString& rString );
	
	// Make the member compatible with the mask.
	//
	int GetWindowText( LPTSTR lpszStringBuf, int nMaxCount );

	//{{AFX_VIRTUAL(ccEdit)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(ccEdit)
	afx_msg void OnUpdate();
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);	
	afx_msg void OnDeltaposSpinCtrl(NMHDR* pNMHDR, LRESULT* pResult);
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // __CCEDIT_H__
