//////////////////////////////////////////////////////////////////////
// $File name   : ccError.cpp: 
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ccError.h"
#include "ccException.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// *** Construction
//////////////////////////////////////////////////////////////////////

ccError::ccError(BOOL abEnableLogging /*= FALSE*/, BOOL abAppend /*= TRUE*/, const char *asFileName /*= "ccError.err"*/)
{
	enableJEDebugMode( FALSE );

	setFileName( asFileName );	
	enableLogging( abEnableLogging, abAppend );
}

//////////////////////////////////////////////////////////////////////
// *** Destruction
//////////////////////////////////////////////////////////////////////

ccError::~ccError()
{
}
	
//////////////////////////////////////////////////////////////////////
// *** Input/Output
//////////////////////////////////////////////////////////////////////

int ccError::error( E_ERROR_MODE aiErrorMode /*= -1*/, int aiErrorNumber /*= NULL*/, int aiPrefixText  /*= NULL*/,
			        int aiSuffixText /*= NULL*/, int aiType /*= CCE_NO_OUTPUT*/ )
{
	CString lsErrorMessage, lsPrefixText, lsSuffixText;

	switch ( aiErrorMode )
	{
	case CCEM_GET_LAST_ERROR :
		aiErrorNumber = GetLastError();
		
	case CCEM_ERROR_NUMBER   :
		{
			LPTSTR	lpBuf;
			// Convert the ErrorNumber to a system ErrorMessage string.
			if(::FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM |
								FORMAT_MESSAGE_ALLOCATE_BUFFER,
					 			NULL,
								aiErrorNumber,
								MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
								(LPTSTR)&lpBuf,
								0,
								NULL
							   )!=0)
			{
				lsErrorMessage	=	lpBuf;
				::LocalFree(lpBuf);
			}
			else
			{
				lsErrorMessage.Format("Error %i occurred (no further info)", aiErrorNumber);
			}		
		}
		break;

	case CCEM_ERROR_MESSAGE  :
		lsErrorMessage.LoadString(aiErrorNumber);
		break;
	}

	if ( aiPrefixText != NULL ) { lsPrefixText.LoadString( aiPrefixText ); };
	if ( aiSuffixText != NULL ) { lsSuffixText.LoadString( aiSuffixText ); };

	return error( lsErrorMessage, lsPrefixText, lsSuffixText, aiType );
}

//
	
int ccError::error( CString asErrorMessage /*= "" */, CString asPrefixText /*= ""*/, \
		            CString asSuffixText   /*= "" */, int aiType /*= CCE_NO_OUTPUT*/)
{
	CString	lsErrorMsg;
	
	// Create the error message, Prefix + errorMessage + suffix
	if ( !asPrefixText.IsEmpty() )	
		lsErrorMsg += asPrefixText + "\n";	
	
	lsErrorMsg += asErrorMessage;	
	
	if ( !asSuffixText.IsEmpty() )	
		lsErrorMsg = lsErrorMsg + "\n" + asSuffixText;

	// Display the error message to the screen. And get a user input.
	int liReturnValue = IDOK;
	if ( aiType != CCE_NO_OUTPUT )
	{  //GetForegroundWindow()
		liReturnValue = MessageBox(NULL, lsErrorMsg, "Error", MB_TASKMODAL | MB_ICONEXCLAMATION | aiType);		
	}

	// Add to the log file.
	addMessage( lsErrorMsg );

	// Dump to debug window in DevStudio	
	#ifdef _DEBUG   // entire file
		//TRACE( lsErrorMsg );
		afxDump << lsErrorMsg <<"\n";
	#endif
	//afxDump << sErrorMessage;

	// Dump to jeDebug Wnd	
	if ( oError.isJEDebugMode() ) DEBUGOUT( lsErrorMsg );

	return liReturnValue;
}

//////////////////////////////////////////////////////////////////////
// *** Globals
//////////////////////////////////////////////////////////////////////

ccError oError;

int ccErr( CString asErrorMessage, CString asPrefixText, CString asSuffixText, int aiType)
{	
	return oError.error( asErrorMessage, asPrefixText, asSuffixText, aiType );	
}

//

int ccErr( ccException & aExcept, CString asPrefixText, CString asSuffixText, int aiType)
{	
#ifdef _DEBUG
	return oError.error( aExcept.getFullDescription(), asPrefixText, asSuffixText, aiType );	
#else
	return oError.error( aExcept.getDescription(), asPrefixText, asSuffixText, aiType );	
#endif
}

//

int ccErrGLE( int aiPrefixText, int aiSuffixText, int aiType )
{
	return oError.error( CCEM_GET_LAST_ERROR, 0, aiPrefixText, aiSuffixText, aiType );
}

//

int ccErrEN( int aiErrorNumber, int aiPrefixText, int aiSuffixText, int aiType )
{
	return oError.error( CCEM_ERROR_NUMBER, aiErrorNumber, aiPrefixText, aiSuffixText, aiType );
}

//

int ccErrEM( int aiErrorNumber, int aiPrefixText, int aiSuffixText, int aiType )
{
	return oError.error( CCEM_ERROR_MESSAGE, aiErrorNumber, aiPrefixText, aiSuffixText, aiType );	
}

