// ==========================================================================
//                    Class Specification : ccError
// ==========================================================================
//////////////////////////////////////////////////////////////////////
// $File name   : /ccLib/dev/ccgLib/ccError.h: 
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// $HomePage    : home.bip.net/CyberCow
// $ICQ         : 7983755
//
// Notice       : Copyright (c) CyberCow 1999
//                Knypplerskevagen 24
//                168 38 BROMMA
//                SWEDEN  
//                tel int + 46 (8) 26 45 95
//                
//                The copyright to the computer program(s) herein is the 
//				  property of CyberCow.The program(s) may be used and/or 
//				  copied only with the written permission of CyberCow or 
//				  in accordance with the terms and conditions stipulated 
//				  in the agreement/contract under which the program(s) 
//				  have been supplied. 
//
/////////////////////////////////////////////////////////////////////////////
///** $History$ *************************************************************
// *
// * $Date: 9/03/99 10:17 $
// * $Revision: 11 $
// * 
// * *****************  Version 1  *****************
// * User: Daniel Date: 9/03/99 10:17
// * Source-Code Traced [X]; Reviewd [X] 
// * Created in $/ccLib/dev/ccgLib
// * Initial release.
// * 
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
// ** $Properties$ **********************************************************
// * [ ] Abstract class (does not have any objects)
// * [ ] Uses exceptions
// **************************************************************************
/////////////////////////////////////////////////////////////////////////////
/*
$Description : 

A ccError object handles error detection, error number lookup, error logging, 
displaying error messages for MFC, win32 and ccLib routines and classes.

*/
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_CCERROR_H__3ADAD636_61CB_11D3_88DF_00609708DCFE__INCLUDED_)
#define AFX_CCERROR_H__3ADAD636_61CB_11D3_88DF_00609708DCFE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ccTraceFile.h"

// Used by ccError::error member.
//
enum E_ERROR_MODE
{
   CCEM_GET_LAST_ERROR = 0,
   CCEM_ERROR_NUMBER,
   CCEM_ERROR_MESSAGE
};

#define CCE_NO_OUTPUT -2

class AFX_EXT_CLASS ccError : public ccTraceFile  
{
// *** Data Members
private:	

// *** Construction
public:
	// Default Constructor
	//
	ccError( BOOL abEnableLogging = FALSE, BOOL abAppend = TRUE, const char *asFileName = "ccError.err" );

// *** Destruction
public:
	// Virtual Constructor
	//
	virtual ~ccError();

// *** Properties
public:
	// Handles if the error window should be displayed when error is called.
	//
	void enableJEDebugMode( BOOL abEnable = TRUE)		{ m_bJEDebugMode = abEnable;	}

	BOOL isJEDebugMode()								{ return m_bJEDebugMode;		}

private:	
	BOOL m_bJEDebugMode;	// If true the errors will be send to jeDebug wnd.

	
// *** Input/Output
public:
	// Shows the error message.
	//
	int error( E_ERROR_MODE aiErrorMode   = CCEM_GET_LAST_ERROR, int aiErrorNumber = NULL, int aiPrefixText  = NULL,\
			   int aiSuffixText = NULL, int aiType = CCE_NO_OUTPUT );

	int error( CString asErrorMessage = "", CString asPrefixText = "", \
		       CString asSuffixText   = "", int aiType = CCE_NO_OUTPUT);
};

extern AFX_EXT_DATA ccError oError;

// String error message 
//
int AFX_EXT_API ccErr( CString asErrorMessage = "", CString asPrefixText = "", \
					   CString asSuffixText   = "", int aiType = CCE_NO_OUTPUT);

// ccException error message 
//
class ccException;
int AFX_EXT_API ccErr( ccException & aExcept, CString asPrefixText = "", \
					   CString asSuffixText   = "", int aiType = CCE_NO_OUTPUT);
   
// Last Error
//
int AFX_EXT_API ccErrGLE( int aiPrefixText  = NULL, int aiSuffixText = NULL, int aiType = CCE_NO_OUTPUT );

// Error Number (system error)
//
int AFX_EXT_API ccErrEN( int aiErrorNumber = NULL, int aiPrefixText  = NULL,\
						 int aiSuffixText = NULL, int aiType = CCE_NO_OUTPUT );

// Message string from string resource
// 
int AFX_EXT_API ccErrEM( int aiErrorNumber = NULL, int aiPrefixText  = NULL,\
						 int aiSuffixText = NULL, int aiType = CCE_NO_OUTPUT );



///// Don't use those macros, use the procedures over this.
//
#define ccErrorEM(errorMessage)			 { oError.error( CCEM_ERROR_MESSAGE, errorMessage);					\
										   DEBUGOUTR( errorMessage );										}
//										 
#define ccErrorEMS(errorMessage, suffix) { oError.error( CCEM_ERROR_MESSAGE, errorMessage, NULL, suffix );	\
										   DEBUGOUTRS( errorMessage, prefix );								}
//
#endif // !defined(AFX_CCERROR_H__3ADAD636_61CB_11D3_88DF_00609708DCFE__INCLUDED_)
