// ccTime.cpp: implementation of the ccTime class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ccgLib.h"
#include "ccTime.h"
#include "ccString.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// *** Construction
//////////////////////////////////////////////////////////////////////

ccTime::ccTime()
{	
	CGoodTime::CGoodTime();
}

//

ccTime::ccTime( CString lsDateTime )
{
	setDateTime( lsDateTime );
}


//////////////////////////////////////////////////////////////////////
// *** Input/Output
//////////////////////////////////////////////////////////////////////

void ccTime::setDate( int aiYear, int aiMonth, int aiDay )
{
	setDateTime( aiYear, aiMonth, aiDay, 0, 0, 0 );
}

//

void ccTime::setDate( CString asDate )
{
	ccString lsYear;
	ccString lsMonth;
	ccString lsDay;

	int liDateIndex = 0;

	// Parse the asDate string and separate year, month and days.
	for (int liStrIndex = 0;liStrIndex < asDate.GetLength(); liStrIndex++)
	{
		if ( (asDate[liStrIndex] != ':') &&
			 (asDate[liStrIndex] != '-') &&
			 (asDate[liStrIndex] != ' ')  
		   )
		{					
			// Validate the date
			switch (liDateIndex)
			{
			//The Year			
			case 0:	
			case 1:	
			case 2: 
			case 3: 
				lsYear += asDate[liStrIndex];
				break;
			//The Month
			case 4: 
			case 5:	
				lsMonth += asDate[liStrIndex];
				break;
			//The Day
			case 6:	
			case 7:
				lsDay += asDate[liStrIndex];
				break;
			default:
				DEBUGOUT( "The date string is to long." );
			}
			liDateIndex++;
		}		
	}			
	
	setDate( lsYear, lsMonth, lsDay );
}

//

CString ccTime::getDate()
{
	CString lsYear;
	CString lsMonth;
	CString lsDay;

	lsYear += ccString( GetYear() );
	
	if ( GetMonth() < 10 )
		lsMonth += "0";
	lsMonth	+= ccString( GetMonth() );

	if ( GetDay() < 10 )
		lsDay += "0";

	lsDay += ccString ( GetDay() );

	CString lsDate;
	lsDate.Format( "%s-%s-%s", lsYear, lsMonth, lsDay );
	return lsDate;
}

//

void ccTime::setTime( int aiHour, int aiMin, int aiSec )
{
	setDateTime( 0, 0, 0, aiHour, aiMin, aiSec );
}

//

void ccTime::setTime( CString asTime )
{
	ccString lsHour;
	ccString lsMinute;
	ccString lsSecond;

	int liDateIndex = 0;
	// Parse the asTime string and separate hour, minute and sec.
	for (int liStrIndex = 0;liStrIndex < asTime.GetLength(); liStrIndex++)
	{
		if ( (asTime[liStrIndex] != ':') &&
			 (asTime[liStrIndex] != '-') &&
			 (asTime[liStrIndex] != ' ')  
		   )
		{					
			//Validate the date
			switch (liDateIndex)
			{
			//The Hours
			case 0:
			case 1:
				lsHour += asTime[liStrIndex];
				break;
			//The Minute
			case 2:
			case 3:
				lsMinute += asTime[liStrIndex];
				break;
			//The Sec
			case 4:
			case 5:
				lsSecond +=asTime[liStrIndex];
				break;
			default:
				DEBUGOUT( "The time string is to long." );
			}
			liDateIndex++;
		}		
	}			
	
	setTime( lsHour, lsMinute, lsSecond );
}

//

CString ccTime::getTime()
{	
	CString lsHour;
	CString lsMinute;
	CString lsSecond;

	if ( GetHour() <10 )
		lsHour += "0";
	lsHour		+= ccString( GetHour() );

	if ( GetMinute() <10 )
		lsMinute += "0";
	lsMinute	+= ccString( GetMinute() );

	if ( GetSecond() <10 )
		lsSecond += "0";
	lsSecond	+= ccString( GetSecond() );


	CString lsTime;
	lsTime.Format("%s:%s:%s", lsHour, lsMinute, lsSecond );	
	return lsTime;

}

//

void ccTime::setDateTime(int nYear, int nMonth, int nDay, int nHour, int nMin, int nSec)
{
	//Validate Year
	if (nYear <1901)
		nYear=1901;	

	//Validate month
	if (nMonth < 1)
		nMonth = 1;
	if (nMonth >12)
		nMonth = 12;
	
	//Validate day
	if (nDay <1)
		nDay = 1;
	switch ( nMonth )
	{	  
		case 1:  // January
		case 3:  // March
		case 5:  // May
		case 7:  // July
		case 8:  // August
		case 10: // October
		case 12: // December
			if (nDay >31)
				nDay = 31;
			break;

		case 4:  // April
		case 6:  // June
		case 9:  // September
		case 11: // November
			if (nDay >30)
				nDay = 30;
			break;
		case 2:
			if ( IsLeapYear(nYear) )
			{
				if (nDay >29)
					nDay = 29;
			}
			else
			{
				if (nDay >28)
					nDay = 28;
			}			
			break;

		default: // Bad month
			ASSERT(FALSE);
	}

	//Validate hour
	if (nHour < 0)
		nMonth = 0;
	if (nHour >23)
		nHour = 23;	
	
	//Validate minute		
	if (nMin < 0)
		nMin = 0;
	if (nMin >59)
		nMin = 59;
	
	//Validate second	
	if (nSec < 0)
		nSec = 0;
	if (nSec >59)
		nSec = 59;
	
/*	START Ripped from CGoodTime::CGoodTime() we can't call the constructors here */
	struct tm atm;
	atm.tm_sec  = nSec;
	atm.tm_min  = nMin;
	atm.tm_hour = nHour;
	atm.tm_mday = nDay;
	atm.tm_mon  = nMonth - 1;        // tm_mon is 0 based	
	atm.tm_year = nYear - 1900;     // tm_year is 1900 based
   // Note: nDST < 0 we should adjust
 	atm.tm_isdst = -1;	
   // Set tm_yday and tm_wday
	SetDays( &atm );
	m_time = atm;

	/*	END Ripped from CGoodTime::CGoodTime() we can't call the constructor here */
}

//

void ccTime::setDateTime( CString asDateTime )
{
	ccString lsYear;
	ccString lsMonth;
	ccString lsDay;
	ccString lsHour;
	ccString lsMinute;
	ccString lsSecond;

	int liDateIndex = 0;
	//Initialize the CStrings
	for (int liStrIndex = 0;liStrIndex < asDateTime.GetLength(); liStrIndex++)
	{
		if ( (asDateTime[liStrIndex] != ':') &&
			 (asDateTime[liStrIndex] != '-') &&
			 (asDateTime[liStrIndex] != ' ')  
		   )
		{					
			//Validate the date
			switch (liDateIndex)
			{
			//The Year			
			case 0:	
			case 1:	
			case 2: 
			case 3: 
				lsYear += asDateTime[liStrIndex];
				break;
			//The Month
			case 4: 
			case 5:	
				lsMonth += asDateTime[liStrIndex];
				break;
			//The Day
			case 6:	
			case 7:
				lsDay += asDateTime[liStrIndex];
				break;
			//The Hours
			case 8:
			case 9:
				lsHour += asDateTime[liStrIndex];
				break;
			//The Minute
			case 10:
			case 11:
				lsMinute += asDateTime[liStrIndex];
				break;
			//The Sec
			case 12:
			case 13:
				lsSecond +=asDateTime[liStrIndex];
				break;
			default:
				MessageBox(NULL,"Date to long","Error",MB_OK);
			}
			liDateIndex++;
		}		
	}			
	
	setDateTime(lsYear, lsMonth, lsDay, lsHour, lsMinute, lsSecond);	
}

//

CString ccTime::getDateTime()
{
	CString lsDateTime;

	lsDateTime.Format("%s %s",getDate(), getTime());	
	return lsDateTime;
}
