//////////////////////////////////////////////////////////////////////
// $File name   : ccTraceFile.cpp: 
//
// $Author      : Daniel Lindh (Daniel.Lindh@home.se)
// Notice       : Copyright (c) CyberCow 1999
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "ccTraceFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
//Construction
//////////////////////////////////////////////////////////////////////

ccTraceFile::ccTraceFile()
{	
	m_traceing = false;		
	setLogSize( 20 );
	setFileName( _T(".\\ccTrace.trc") );	
}

//

E_ccErrRet ccTraceFile::enableLogging( BOOL abEnable /*= TRUE*/, BOOL abAppend /*= FALSE*/ )
{
	// Turn on the logging
	if ( abEnable == TRUE )
	{
		if ( !isTraceing() )
		{
			int liErrorMessage;
			CFileException oFileException;

			// Open the file and append to end of the file.
			if ( abAppend == TRUE )			
				liErrorMessage = Open( m_traceFile, CFile::modeCreate | CFile::modeNoTruncate 
									   | CFile::modeWrite , &oFileException );			
			// Open the file and truncate it.
			else
				liErrorMessage = Open( m_traceFile, CFile::modeCreate | CFile::modeWrite, &oFileException );

			// Failed
			if( liErrorMessage == 0 )   
			{										
				  // complain if an error happened
				  // no need to delete the ex object

				  TCHAR szError[1024];
				  oFileException.GetErrorMessage(szError, 1024);

				  CString strError;
				  strError.LoadString( IDS_ERR_FILE_COULDNT_BE_OPENED );
				  strError += "\n";
				  strError += szError;   
 
				if ( AfxMessageBox( strError, MB_RETRYCANCEL ) == IDRETRY )
					if ( enableLogging( abEnable, abAppend) == CCSUCCEDED )
						return CCSUCCEDED;

				return CCFAILED;
			}
			// Succeded
			else			
			{
				if ( abAppend == TRUE )
					SeekToEnd();

				m_traceing = true;	

				// Write a start sequence in tracefile
				CTime oTime = CTime::GetCurrentTime();
				CString sMessage;
				sMessage = ("**********************************************************************\n" + \
						   oTime.Format( "%m/%d/%y %H:%M:%S Trace is started\n") +						   \
						   "**********************************************************************\n\n");				
				WriteString( sMessage );
			}			
		}
		else
		{
			//afxDump << "File is already opened " << "\n";
			return CCFAILED;
		}
	}
	
	// Turn off the logging
	else 
	{
		if ( isTraceing() )
		{
			Close();
			m_traceing = false;
		}
	}

	return CCSUCCEDED;
}

//////////////////////////////////////////////////////////////////////
//Destruction
//////////////////////////////////////////////////////////////////////

ccTraceFile::~ccTraceFile()
{
	enableLogging( FALSE );
}

//////////////////////////////////////////////////////////////////////
//Properties
//////////////////////////////////////////////////////////////////////

void ccTraceFile::setFileName( CString asFileName )
{
	// If a file is opened, close it and reopen it with the new name.
	if ( isTraceing() )
	{
		enableLogging( FALSE );		
		m_traceFile = asFileName;
		enableLogging(TRUE, TRUE);
	}
	else
		m_traceFile = asFileName;
}

//

void ccTraceFile::browseTraceFile()
{
	CString lsTraceFile;
	lsTraceFile = getSaveAsFile("ccTrace.trc", "trc" );
	if ( lsTraceFile != "-1" )
	{
		setFileName( lsTraceFile );
	}
}

//

CString ccTraceFile::getFileName()
{
	return m_traceFile;
}

//

void ccTraceFile::setLogSize( int aiLogSize )
{
	m_logFileSize = aiLogSize;
}

//

int	ccTraceFile::getLogSize( )
{
	return m_logFileSize;
}

//////////////////////////////////////////////////////////////////////
//Input/Output
//////////////////////////////////////////////////////////////////////

void ccTraceFile::addMessage( CString asMessage )
{
	if ( isTraceing() )
	{
		// Write the trace message on it's own line in the following format
		// 12/24/99 11:23:59  
		//					 Prefix 
		//					 This is the message				   
		//					 Sufix
		CTime oTime = CTime::GetCurrentTime();
		asMessage = ( ( oTime.Format( "%m/%d/%y %H:%M:%S  ") ) + "\n" + asMessage  );		
		asMessage.Replace("\n", "\n\t\t  ");
		asMessage += "\n\n";

		WriteString( asMessage );

		// Check if the file is larger then the allowed size in kilo bytes.
		int liFileSize = GetLength( ) / 1024;
		if ( liFileSize  >=  getLogSize() )
		{
			//Show error message and increace the log file size.
			//AfxMessageBox(IDS_CCTRACEFILE_ERR_MAX_SIZE, MB_ICONQUESTION  );			
			setLogSize( getLogSize() + 10 );
		}
	}
}

//

void ccTraceFile::view()
{
	enableLogging( FALSE );
	while ( (int)ShellExecute( NULL, "open", "notepad", m_traceFile, ".", SW_SHOW ) <= 32)	
	{
		if ( AfxMessageBox( IDS_ERR_FILE_COULDNT_BE_OPENED, MB_RETRYCANCEL ) != IDRETRY )
			break;
	}
	
	// Wait a while, so Notepad got time to open the trace file, before 
	// the file will be opened again by the ccTraceFile class.
	Sleep(1000);		
}

//

void ccTraceFile::clear()
{
	if ( isTraceing() )
	{			
		enableLogging( FALSE );
		// The file will be truncated to 0 when opened.
		enableLogging();
	}
	else
	{
		enableLogging( FALSE );
		// The file will be truncated to 0 when opened.
		enableLogging();
		enableLogging( FALSE );
	}
}

//////////////////////////////////////////////////////////////////////
//Status
//////////////////////////////////////////////////////////////////////

BOOL ccTraceFile::isTraceing( )
{
	return m_traceing;
}
