// jeDebug.cpp: implementation of the jeDebug class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "jeDebug.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#ifdef JESYS_DEBUG

jeDebug::jeDebug(void)
{
}

jeDebug::~jeDebug(void)
{
}

jeDebugEvent::jeDebugEvent(void)
{
	Initialize();
}

jeDebugEvent::~jeDebugEvent(void)
{
	if (data)
	{
		delete data;
		data = 0;
	}
}

void jeDebugEvent::Initialize()
{
	data = new jeDebugEventData;

	if (data)
	{
		data->size = sizeof(struct jeDebugEventData);
	
		data->timestamp = time(0);

		strcpy(data->process,AfxGetAppName());

		data->desc[0] = 0;
	}
}

void jeDebugEvent::SetLine(unsigned long line)
{
	if (data)
	{
		data->line = line;
	}
}

void jeDebugEvent::SetSource(const char * source)
{
	if (data)
	{
		if (source && source[0])
		{
			strcpy(data->source,source);
		}
	}
}

void jeDebugEvent::SetDescription(const char * desc)
{
	if (data)
	{
		if (desc && desc[0])
		{
			strcpy(data->desc,desc);
		}
	}
}

void jeDebugEvent::SetCode(unsigned long code)
{
	if (data)
	{
		data->code = code;
	}
}

void jeDebugEvent::SetType(unsigned long type)
{
	if (data)
	{
		data->type = type;
	}
}

HWND jeDebug::GetDebugWindow(void)
{
	HWND wnd = ::FindWindow("#32770","jeDebug");

	if (!wnd)
	{
		short ok = StartDebugProgram();

		if (ok) wnd = ::FindWindow("#32770","jeDebug");
	}

	return (wnd);
}

void jeDebug::Send(jeDebugEvent *event)
{
	Send(event->GetData());
}

void jeDebug::Send(jeDebugEventData *data)
{
	HWND wnd = GetDebugWindow();

	if (wnd)
	{
		COPYDATASTRUCT cds;

		cds.cbData = data->size;
		cds.lpData = data;

		::SendMessage(wnd,WM_COPYDATA,0,(long) &cds);
	}
	ASSERT(1);
}

jeDebugEventData * jeDebugEvent::GetData()
{
	return (data);
}

void jeDebug::NeedValue(const char * source, unsigned long line,const char * valstr)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
	event.SetDescription(valstr);
	event.SetType(jedebug);
	event.SetCode(needvalue);

	Send(&event);
}

void jeDebug::NeedSame(const char * source, unsigned long line,const char * valstr1,const char * valstr2)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
	event.SetDescription(valstr1);
	event.SetType(jedebug);
	event.SetCode(needsame);

	Send(&event);
}

short jeDebug::StartDebugProgram()
{
	BOOL ok = FALSE;

	char filename[256];

	if (jeDebug::GetProgramLocation(filename,sizeof(filename)))
	{
	HANDLE event = OpenEvent(EVENT_ALL_ACCESS,FALSE,"jeDebug_Event"); // try to open it if it exists

	if (!event)	event = CreateEvent(0,FALSE,FALSE,"jeDebug_Event"); // else create

	if (event)
	{
//		STARTUPINFO si;

//		si.cb = sizeof(STARTUPINFO);

//		PROCESS_INFORMATION pi;
//		ok = CreateProcess("jeDebug.exe",0,0,0,FALSE,0,0,0,&si,&pi);

		strcat(filename,"\\jeDebug.exe");
		UINT retcode = WinExec(filename,SW_SHOW);
//AfxMessageBox("yo!",MB_OK);
		if (retcode > 31)
		{
			DWORD retcode = WaitForSingleObjectEx(event,3000,FALSE);
	
			if (retcode == WAIT_OBJECT_0) ok = TRUE;
			else
			{
				ok = FALSE;
				AfxMessageBox("WaitForSingleObjectEx() timeout!",MB_OK);
			}
		}
		else AfxMessageBox("Could not execute jeDebug.exe!",MB_OK);

		CloseHandle(event);
	}
	else AfxMessageBox("Could not create event!",MB_OK);
	}

	return (ok ? 1 : 0);
}

short jeDebug::GetProgramLocation(char * string, size_t maxlen)
{
	HKEY key;

	long result = RegOpenKeyEx(HKEY_LOCAL_MACHINE,"Software\\Nebol Software\\jeDebug",0,KEY_READ,&key);

	if (result == ERROR_SUCCESS)
	{
		DWORD type;

		result = RegQueryValueEx(key,"Path",0,&type,(unsigned char *) string,(unsigned long *) &maxlen);

		if (result == ERROR_SUCCESS)
		{
			return (1);
		}

		RegCloseKey(key);
	}

	return (0);
}

void jeDebug::NeedClear(const char * source,unsigned long line,void * value,const char * valstr)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
	event.SetDescription(valstr);
	event.SetType(jedebug);
	event.SetCode(needclear);
// what to do about value ?
	Send(&event);
}

void jeDebug::NeedDiff(const char * source,unsigned long line,const char * valstr1,const char * valstr2)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
	event.SetDescription(valstr1);
	event.SetType(jedebug);
	event.SetCode(needdiff);
// what to do about value ?
	Send(&event);
}

void jeDebug::NeedLess(const char * source,unsigned long line,const char * valstr1,const char * valstr2)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
	event.SetDescription(valstr1);
	event.SetType(jedebug);
	event.SetCode(needless);
// what to do about value ?
	Send(&event);
}

void jeDebug::NeedMore(const char * source,unsigned long line,const char * valstr1,const char * valstr2)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
	event.SetType(jedebug);
	event.SetCode(needmore);

	char temp[256];
	sprintf(temp,"%s is NOT more than %s",valstr1,valstr2);
	event.SetDescription(temp);

	Send(&event);
}

void jeDebug::StringOut(const char * source,unsigned long line,const char * string)
{
	if (string)
	{
		jeDebugEvent event;

//		if (!len) len = strlen(string);

		event.SetSource(source);
		event.SetLine(line);
		event.SetType(jedebug);
		event.SetCode(stringout);
		event.SetDescription(string);

		Send(&event);
	}
}

void jeDebug::StringOutR(const char * source,unsigned long line, int nResource)
{
	CString string;
	string.LoadString( nResource );

	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
	event.SetType(jedebug);
	event.SetCode(stringout);
	event.SetDescription(string);

	Send(&event);
}

//

void jeDebug::StringOutRS(const char *source,unsigned long line, int nResource, const char *suffixString)
{
	CString string;
	string.LoadString( nResource );
	string += suffixString;

	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
	event.SetType(jedebug);
	event.SetCode(stringout);
	event.SetDescription(string);

	Send(&event);
}

//

void jeDebug::StringOutF(const char *source,unsigned long line,const char *string, ...)
{
	va_list marker;

	va_start(marker,string);

	char text[1024];

	vsprintf(text,string,marker);

	if (text)
	{
		jeDebugEvent event;

//		if (!len) len = strlen(string);

		event.SetSource(source);
		event.SetLine(line);
		event.SetType(jedebug);
		event.SetCode(stringoutf);
		event.SetDescription(text);

		Send(&event);

		va_end( marker );
	}
}


void jeDebug::ReportWin32Error(const char *source,unsigned long line,DWORD error)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
//	event.SetDescription(valstr1);
	event.SetType(win32);
	event.SetCode(error);
// what to do about value ?
	Send(&event);

}

void jeDebug::ReportDirectDrawError(const char *source,unsigned long line,HRESULT error)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
//	event.SetDescription(valstr1);
	event.SetType(ddraw);
	event.SetCode(error);
// what to do about value ?
	Send(&event);
}

void jeDebug::ReportDirectInputError(const char *source,unsigned long line,HRESULT error)
{
	jeDebugEvent event;

	event.SetSource(source);
	event.SetLine(line);
//	event.SetDescription(valstr1);
	event.SetType(dinput);
	event.SetCode(error);
// what to do about value ?
	Send(&event);
}

#endif // JESYS_DEBUG