/*

	jeDebug - Copyright (c) Johan Eliasson 1998.

	http://www.omtanken.se/valheru

	mailto:nebol@home.se

	This is the replacement for the jeDebug functionality in JEPP.

*/

#ifdef _DEBUG
#define JESYS_DEBUG 
#endif

#ifndef __JEDEBUG__
#define __JEDEBUG__

#ifndef _INC_TIME
#include <time.h>
#endif

#ifndef __wtypes_h__
#include <wtypes.h>
#endif

#ifdef JESYS_DEBUG // then use these macros...

#define NEEDVALUE(x)\
	if (!(x)) jeDebug::NeedValue(__FILE__,__LINE__,#x)

#define ANEEDVALUE(x)\
	if (!(x)) {jeDebug::NeedValue(__FILE__,__LINE__,#x); ASSERT(x);}

#define NEEDVALUES(x,y)\
	if (!(x)) jeDebug::NeedValue(__FILE__,__LINE__,#x);\
	if (!(y)) jeDebug::NeedValue(__FILE__,__LINE__,#y)

#define NEEDSTRING(x)\
  if (!(x)) jeDebug::NeedValue(__FILE__,__LINE__,#x);\
	else if (!(x[0])) jeDebug::NeedValue(__FILE__,__LINE__,#x"[0]")

#define NEEDSAME(x,y)\
	if (!((x) == (y))) jeDebug::NeedSame(__FILE__,__LINE__,#x,#y)

// NOT A SAFE MACRO !
#define NEEDCLEAR(x)\
	if (x) jeDebug::NeedClear(__FILE__,__LINE__,(void *) (x),#x)

#define NEEDDIFF(x,y)\
	if ((x) == (y)) jeDebug::NeedDiff(__FILE__,__LINE__,#x,#y)

#define NEEDLESS(x,y)\
	if ((x) >= (y)) jeDebug::NeedLess(__FILE__,__LINE__,#x,#y)

#define NEEDMORE(x,y)\
	if ((x) <= (y)) jeDebug::NeedMore(__FILE__,__LINE__,#x,#y)

#define DEBUGOUT(x) jeDebug::StringOut(__FILE__,__LINE__,(x))
#define DEBUGOUTR(x) jeDebug::StringOutR(__FILE__,__LINE__,(x))
#define DEBUGOUTRS(x,suffix) jeDebug::StringOutRS(__FILE__,__LINE__,(x),(suffix))


//#define DEBUGOUTF jeDebug::StringOutF(__FILE__,__LINE__,

#define REPORTWIN32ERROR(x)\
	jeDebug::ReportWin32Error(__FILE__,__LINE__,(x))

#define REPORTLASTWIN32ERROR\
	jeDebug::ReportWin32Error(__FILE__,__LINE__,GetLastError())

#define REPORTDIRECTDRAWERROR(x)\
	jeDebug::ReportDirectDrawError(__FILE__,__LINE__,x);

#define REPORTDIRECTINPUTERROR(x)\
	jeDebug::ReportDirectInputError(__FILE__,__LINE__,x);

#else // if JESYS_DEBUG is NOT defined ---> nothing
#define NEEDVALUE(x)		((void) 0)
#define ANEEDVALUE(x)		((void) 0)
#define NEEDVALUES(x,y) ((void) 0)
#define NEEDCLEAR(x)		((void) 0)
#define NEEDSAME(x,y)		((void) 0)
#define NEEDDIFF(x,y)		((void) 0)
#define NEEDSTRING(x)		((void) 0)
#define NEEDLESS(x,y)		((void) 0)
#define NEEDMORE(x,y)		((void) 0)
#define DEBUGOUT(x)			((void) 0)
#define DEBUGOUTR(x)		((void) 0)
#define DEBUGOUTRS(x,suffix)((void) 0)
#define REPORTWIN32ERROR(x)	((void) 0)
#define REPORTLASTWIN32ERROR	((void) 0)
#define REPORTDIRECTDRAWERROR(x)	((void) 0)
#define REPORTDIRECTINPUTERROR(x)	((void) 0)
#endif // JESYS_DEBUG

#define NEEDEQUAL NEEDSAME
#define NEEDZERO	NEEDCLEAR
#define NEEDNULL	NEEDCLEAR
#define DOUT		DEBUGOUT
#define REPORTLATESTWIN32ERROR	REPORTLASTWIN32ERROR

#ifdef JESYS_DEBUG

struct jeDebugEventData
{
	size_t size;				// size of this struct
	time_t timestamp;		// time of error
	unsigned long line;	// line in file
	unsigned long type;	// jeDebug error? Win32 error? JEPP error? other?
	unsigned long code;	// error number/code
	char process[80];		// name of process
	char source[256];		// full path + file
	char desc[2048];			// description
};

class AFX_EXT_CLASS jeDebugEvent
{
protected:

	jeDebugEventData *data;

	void Initialize(void); // called by constructor

public:

	jeDebugEvent(void);
	virtual ~jeDebugEvent(void);

	void SetType(unsigned long type);
	void SetCode(unsigned long code);
	void SetDescription(const char *desc);
	void SetSource(const char *source);
	void SetLine(unsigned long line);

	jeDebugEventData * GetData(void);
};

class AFX_EXT_CLASS jeDebug  
{
protected:

	static short GetProgramLocation(char *string,size_t maxlen);
	static short StartDebugProgram(void);

public:

	jeDebug(void);
	virtual ~jeDebug(void);

	enum error_types
	{
		jedebug = 1,
		win32,
		ddraw,
		dinput
	};

	enum error_codes_for_type_jedebug
	{
		needvalue = 1,
		needsame,
		needclear,
		needdiff,
		needless,
		needmore,

		stringout,
		stringoutr,
		stringoutf
	};

	static void NeedValue(const char *source,unsigned long line,const char *valstr);
    static void NeedSame (const char *source,unsigned long line,const char *valstr1,const char *valstr2);
	static void NeedMore (const char *source,unsigned long line,const char *valstr1,const char *valstr2);
	static void NeedLess (const char *source,unsigned long line,const char *valstr1,const char *valstr2);
	static void NeedDiff (const char *source,unsigned long line,const char *valstr1,const char *valstr2);
	static void NeedClear(const char *source,unsigned long line,void *value,const char *valstr);

	static void StringOut (const char *source,unsigned long line,const char *string);
	static void StringOutR (const char *source,unsigned long line, int nResource);
	static void StringOutRS(const char *source,unsigned long line, int nResource, const char *suffixString);
	static void StringOutF(const char *source,unsigned long line,const char *string, ...);

	static void ReportWin32Error(const char *source,unsigned long line,DWORD error);

	static void ReportDirectDrawError(const char *source,unsigned long line,HRESULT error);
	static void ReportDirectInputError(const char *source,unsigned long line,HRESULT error);

	static void Send(jeDebugEventData *data);
	static void Send(jeDebugEvent *event);
	static HWND GetDebugWindow(void);
};

#endif // JESYS_DEBUG

#endif
